@@ -17,6 +17,7 @@ except NameError:
         import __builtin__ as builtins
     builtins._ASTROPY_SETUP_ = False
     del version_info
+    del builtins
 
 try:
     from .version import version as __version__
@@ -30,7 +31,7 @@ except ImportError:
     __githash__ = ''
 
 
-import logging
+__minimum_numpy_version__ = '1.5.0'
 
 
 # The location of the online documentation for astropy
@@ -41,6 +42,47 @@ else:
     online_docs_root = 'http://docs.astropy.org/en/{0}/'.format(__version__)
 
 
+def _check_numpy():
+    """
+    Check that Numpy is installed and it is of the minimum version we
+    require.
+    """
+    # Note: We could have used distutils.version for this comparison,
+    # but it seems like overkill to import distutils at runtime.
+    requirement_met = False
+
+    try:
+        import numpy
+    except ImportError:
+        pass
+    else:
+        major, minor, rest = numpy.__version__.split(".", 2)
+        rmajor, rminor, rest = __minimum_numpy_version__.split(".", 2)
+        requirement_met = ((int(major), int(minor)) >=
+                           (int(rmajor), int(rminor)))
+
+    if not requirement_met:
+        msg = ("numpy version {0} or later must be installed to use "
+               "astropy".format(
+                   __minimum_numpy_version__))
+        raise ImportError(msg)
+
+    return numpy
+
+
+if not _ASTROPY_SETUP_:
+    _check_numpy()
+
+
+from .config import ConfigurationItem
+
+
+UNICODE_OUTPUT = ConfigurationItem(
+    'unicode_output', False,
+    'Use Unicode characters when outputting values, and writing widgets '
+    'to the console.')
+
+
 # set up the test command
 def _get_test_runner():
     from .tests.helper import TestRunner
@@ -49,7 +91,7 @@ def _get_test_runner():
 
 def test(package=None, test_path=None, args=None, plugins=None,
          verbose=False, pastebin=None, remote_data=False, pep8=False,
-         pdb=False, coverage=False, open_files=False):
+         pdb=False, coverage=False, open_files=False, parallel=0):
     """
     Run Astropy tests using py.test. A proper set of arguments is
     constructed and passed to `pytest.main`.
@@ -104,6 +146,12 @@ def test(package=None, test_path=None, args=None, plugins=None,
         this adds extra run time to the test suite.  Works only on
         platforms with a working `lsof` command.
 
+    parallel : int, optional
+        When provided, run the tests in parallel on the specified
+        number of CPUs.  If parallel is negative, it will use the all
+        the cores on the machine.  Requires the `pytest-xdist` plugin
+        is installed.
+
     See Also
     --------
     pytest.main : py.test function wrapped by `run_tests`.
@@ -114,33 +162,49 @@ def test(package=None, test_path=None, args=None, plugins=None,
         package=package, test_path=test_path, args=args,
         plugins=plugins, verbose=verbose, pastebin=pastebin,
         remote_data=remote_data, pep8=pep8, pdb=pdb,
-        coverage=coverage, open_files=open_files)
+        coverage=coverage, open_files=open_files,
+        parallel=parallel)
 
 
-# Use the root logger as a dummy log before initilizing Astropy's logger
-log = logging.getLogger()
-
 # if we are *not* in setup mode, import the logger and possibly populate the
 # configuration file with the defaults
-if not _ASTROPY_SETUP_:
-    from .logger import _init_log
+def _initialize_astropy():
     from . import config
 
     import os
     import sys
     from warnings import warn
 
-    log = _init_log()
+    def _rollback_import(message):
+        log.error(message)
+        # Now disable exception logging to avoid an annoying error in the
+        # exception logger before we raise the import error:
+        _teardown_log()
+
+        # Roll back any astropy sub-modules that have been imported thus
+        # far
+
+        for key in list(sys.modules):
+            if key.startswith('astropy.'):
+                del sys.modules[key]
+        raise ImportError('astropy')
+
+    if sys.version_info[0] >= 3 and os.path.exists('setup.py'):
+        _rollback_import(
+            "You appear to be trying to import astropy from within a source "
+            "checkout. This is currently not possible using Python 3 due to "
+            "the reliance of 2to3 to convert some of Astropy's subpackages "
+            "for Python 3 compatibility.")
 
     try:
         from .utils import _compiler
     except ImportError:
         if os.path.exists('setup.py'):
-            log.error('You appear to be trying to import astropy from within '
-                      'a source checkout; please run `./setup.py develop` or '
-                      '`./setup.py build_ext --inplace` first so that '
-                      'extension modules can be compiled and made importable.')
-            sys.exit(1)
+            _rollback_import(
+                'You appear to be trying to import astropy from within a '
+                'source checkout; please run `./setup.py develop` or '
+                '`./setup.py build_ext --inplace` first so that extension '
+                'modules can be compiled and made importable.')
         else:
             # Outright broken installation; don't be nice.
             raise
@@ -155,6 +219,19 @@ if not _ASTROPY_SETUP_:
         wmsg = (e.args[0] + " Cannot install default profile. If you are "
                 "importing from source, this is expected.")
         warn(config.configuration.ConfigurationDefaultMissingWarning(wmsg))
-        del e
 
-    del _init_log, os, warn, config_dir  # clean up namespace
+
+import logging
+
+# Use the root logger as a dummy log before initilizing Astropy's logger
+log = logging.getLogger()
+
+
+if not _ASTROPY_SETUP_:
+    from .logger import _init_log, _teardown_log
+
+    log = _init_log()
+
+    _initialize_astropy()
+
+    from .utils.misc import find_api_page
