@@ -5,6 +5,8 @@ This module contains configuration and setup utilities for the
 `astropy` project. This includes all functionality related to the
 affiliated package index.
 """
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
 
 from .paths import *
 from .configuration import *
