@@ -8,14 +8,24 @@ configuration files for Astropy and affiliated packages.
     `ConfigParser`. More information and documentation for configobj can be
     found at http://www.voidspace.org.uk/python/configobj.html.
 """
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+from ..extern import six
 
-from __future__ import division
-
+import inspect
+import pkgutil
 import re
+import sys
 import textwrap
-from contextlib import contextmanager
+import types
 
+from contextlib import contextmanager
+from os import path
+from warnings import warn
+
+from ..extern import six
 from ..extern.configobj import configobj, validate
+from ..utils.exceptions import AstropyWarning
 
 
 __all__ = ['ConfigurationItem', 'InvalidConfigurationItemWarning',
@@ -23,14 +33,14 @@ __all__ = ['ConfigurationItem', 'InvalidConfigurationItemWarning',
            'reload_config']
 
 
-class InvalidConfigurationItemWarning(Warning):
+class InvalidConfigurationItemWarning(AstropyWarning):
     """ A Warning that is issued when the configuration value specified in the
     astropy configuration file does not match the type expected for that
     configuration value.
     """
 
 
-class ConfigurationMissingWarning(Warning):
+class ConfigurationMissingWarning(AstropyWarning):
     """ A Warning that is issued when the configuration directory cannot be
     accessed (usually due to a permissions problem). If this warning appears,
     configuration items will be set to their defaults rather than read from the
@@ -45,8 +55,8 @@ class ConfigurationDefaultMissingError(ValueError):
     """
 
 
-#this is used in astropy/__init__.py
-class ConfigurationDefaultMissingWarning(Warning):
+# this is used in astropy/__init__.py
+class ConfigurationDefaultMissingWarning(AstropyWarning):
     """ A warning that is issued when the configuration defaults (which
     should be generated at build-time) are missing.
     """
@@ -111,13 +121,12 @@ class ConfigurationItem(object):
 
     """
 
-    #this is used to make validation faster so a Validator object doesn't
-    #have to be created every time
+    # this is used to make validation faster so a Validator object doesn't
+    # have to be created every time
     _validator = validate.Validator()
 
     def __init__(self, name, defaultvalue='', description=None, cfgtype=None,
-                      module=None):
-        from warnings import warn
+                 module=None):
         from ..utils import find_current_module
         from ..utils import isiterable
 
@@ -134,11 +143,11 @@ class ConfigurationItem(object):
         self.module = module
         self.description = description
 
-        #now determine cfgtype if it is not given
+        # now determine cfgtype if it is not given
         if cfgtype is None:
             if (isiterable(defaultvalue) and not
-                isinstance(defaultvalue, basestring)):
-                #it is an options list
+                    isinstance(defaultvalue, six.string_types)):
+                # it is an options list
                 dvstr = [str(v) for v in defaultvalue]
                 cfgtype = 'option(' + ', '.join(dvstr) + ')'
                 defaultvalue = dvstr[0]
@@ -157,12 +166,12 @@ class ConfigurationItem(object):
         self._validate_val(defaultvalue)
         self.defaultvalue = defaultvalue
 
-        #note that the actual value is stored in the ConfigObj file for this
-        #package
+        # note that the actual value is stored in the ConfigObj file for this
+        # package
 
-        #this checks the current value to make sure it's valid for the type
-        #as well as updating the ConfigObj with the default value, if it's not
-        #actually in the ConfigObj
+        # this checks the current value to make sure it's valid for the type
+        # as well as updating the ConfigObj with the default value, if it's not
+        # actually in the ConfigObj
         try:
             self()
         except TypeError as e:
@@ -257,16 +266,16 @@ class ConfigurationItem(object):
             msg = 'Provided value for configuration item {0} not valid: {1}'
             raise TypeError(msg.format(self.name, e.args[0]))
 
-        #Now find the  ConfigObj that this is based on
+        # Now find the  ConfigObj that this is based on
         baseobj = get_config(self.module)
         secname = baseobj.name
         cobj = baseobj
-        #a ConfigObj's parent is itself, so we look for the parent with that
+        # a ConfigObj's parent is itself, so we look for the parent with that
         while cobj.parent is not cobj:
             cobj = cobj.parent
 
-        #use the current on disk version, which will be modified with the
-        #given value and type/description
+        # use the current on disk version, which will be modified with the
+        # given value and type/description
         newobj = configobj.ConfigObj(cobj.filename, interpolation=False)
         if secname is not None:
             if secname not in newobj:
@@ -290,7 +299,7 @@ class ConfigurationItem(object):
         secname = baseobj.name
 
         cobj = baseobj
-        #a ConfigObj's parent is itself, so we look for the parent with that
+        # a ConfigObj's parent is itself, so we look for the parent with that
         while cobj.parent is not cobj:
             cobj = cobj.parent
 
@@ -332,7 +341,7 @@ class ConfigurationItem(object):
             If the configuration value as stored is not this item's type.
         """
 
-        #get the value from the relevant `configobj.ConfigObj` object
+        # get the value from the relevant `configobj.ConfigObj` object
         sec = get_config(self.module)
         if self.name not in sec:
             self.set(self.defaultvalue)
@@ -349,9 +358,9 @@ class ConfigurationItem(object):
 
         throws the underlying configobj exception if it fails
         """
-        #note that this will normally use the *class* attribute `_validator`,
-        #but if some arcane reason is needed for making a special one for an
-        #instance or sub-class, it will be used
+        # note that this will normally use the *class* attribute `_validator`,
+        # but if some arcane reason is needed for making a special one for an
+        # instance or sub-class, it will be used
         return self._validator.check(self.cfgtype, val)
 
     def _generate_comments(self):
@@ -395,8 +404,6 @@ def get_config(packageormod=None, reload=False):
         be determined.
 
     """
-    from os.path import join
-    from warnings import warn
 
     from .paths import get_config_dir
     from ..utils import find_current_module
@@ -422,7 +429,7 @@ def get_config(packageormod=None, reload=False):
             cobj = configobj.ConfigObj(interpolation=False)
         else:
             try:
-                cfgfn = join(get_config_dir(), rootname + '.cfg')
+                cfgfn = path.join(get_config_dir(), rootname + '.cfg')
                 cobj = configobj.ConfigObj(cfgfn, interpolation=False)
             except (IOError, OSError) as e:
                 msg = ('Configuration defaults will be used, and '
@@ -470,7 +477,7 @@ def save_config(packageormod=None, filename=None):
     """
 
     sec = get_config(packageormod)
-    #look for the section that is its own parent - that's the base object
+    # look for the section that is its own parent - that's the base object
     while sec.parent is not sec:
         sec = sec.parent
     if filename is not None:
@@ -495,7 +502,7 @@ def reload_config(packageormod=None):
         The package or module name - see `get_config` for details.
     """
     sec = get_config(packageormod)
-    #look for the section that is its own parent - that's the base object
+    # look for the section that is its own parent - that's the base object
     while sec.parent is not sec:
         sec = sec.parent
     sec.reload()
@@ -518,8 +525,6 @@ def get_config_items(packageormod=None):
         objects.
 
     """
-    import sys
-    from inspect import ismodule
 
     from ..utils import find_current_module
 
@@ -529,17 +534,17 @@ def get_config_items(packageormod=None):
             msg1 = 'Cannot automatically determine get_config module, '
             msg2 = 'because it is not called from inside a valid module'
             raise RuntimeError(msg1 + msg2)
-    elif isinstance(packageormod, basestring):
+    elif isinstance(packageormod, six.string_types):
         __import__(packageormod)
         packageormod = sys.modules[packageormod]
-    elif ismodule(packageormod):
+    elif inspect.ismodule(packageormod):
         pass
     else:
         raise TypeError('packageormod in get_config_items is invalid')
 
     configitems = {}
-    for n, obj in packageormod.__dict__.iteritems():
-        #if it's not a new-style object, it's certainly not a ConfigurationItem
+    for n, obj in six.iteritems(packageormod.__dict__):
+        # if it's not a new-style object, it's certainly not a ConfigurationItem
         if hasattr(obj, '__class__'):
             fqn = obj.__class__.__module__ + '.' + obj.__class__.__name__
             if fqn == 'astropy.config.configuration.ConfigurationItem':
@@ -560,7 +565,7 @@ def _fix_section_blank_lines(sec, recurse=True, gotoroot=True):
     if not hasattr(sec, 'sections'):
         sec = get_config(sec)
 
-        #look for the section that is its own parent - that's the base object
+        # look for the section that is its own parent - that's the base object
         if gotoroot:
             while sec.parent is not sec:
                 sec = sec.parent
@@ -614,18 +619,16 @@ def generate_all_config_items(pkgornm=None, reset_to_default=False,
         The filename of the generated configuration item.
 
     """
-    from os.path import split
-    from types import ModuleType
-    from pkgutil import get_loader, walk_packages
 
     from ..utils import find_current_module
 
     if pkgornm is None:
         pkgornm = find_current_module(1).__name__.split('.')[0]
 
-    if isinstance(pkgornm, basestring):
-        package = get_loader(pkgornm).load_module(pkgornm)
-    elif isinstance(pkgornm, ModuleType) and '__init__' in pkgornm.__file__:
+    if isinstance(pkgornm, six.string_types):
+        package = pkgutil.get_loader(pkgornm).load_module(pkgornm)
+    elif (isinstance(pkgornm, types.ModuleType) and
+            '__init__' in pkgornm.__file__):
         package = pkgornm
     else:
         msg = 'generate_all_config_items was not given a package/package name'
@@ -634,16 +637,19 @@ def generate_all_config_items(pkgornm=None, reset_to_default=False,
     if hasattr(package, '__path__'):
         pkgpath = package.__path__
     elif hasattr(package, '__file__'):
-        pkgpath = split(package.__file__)[0]
+        pkgpath = path.split(package.__file__)[0]
     else:
         raise AttributeError('package to generate config items for does not '
                              'have __file__ or __path__')
 
-    for imper, nm, ispkg in walk_packages(pkgpath, package.__name__ + '.'):
+    prefix = package.__name__ + '.'
+    for imper, nm, ispkg in pkgutil.walk_packages(pkgpath, prefix):
+        if nm == 'astropy.config.tests.test_configs':
+            continue
         if not _unsafe_import_regex.match(nm):
             imper.find_module(nm)
             if reset_to_default:
-                for cfgitem in get_config_items(nm).itervalues():
+                for cfgitem in six.itervalues(get_config_items(nm)):
                     cfgitem.set(cfgitem.defaultvalue)
 
     _fix_section_blank_lines(package.__name__, True, True)
@@ -681,25 +687,25 @@ def update_default_config(pkg, default_cfg_dir_or_fn):
         If the default configuration could not be found.
 
     """
-    import os
 
     cfgfn = get_config(pkg).filename
 
-    if os.path.exists(cfgfn):
+    if path.exists(cfgfn):
         with open(cfgfn) as f:
             doupdate = f.read() == ''
     else:
         doupdate = True
 
     if doupdate:
-        if os.path.isdir(default_cfg_dir_or_fn):
-            default_cfgfn = os.path.join(default_cfg_dir_or_fn, pkg + '.cfg')
+        if path.isdir(default_cfg_dir_or_fn):
+            default_cfgfn = path.join(default_cfg_dir_or_fn, pkg + '.cfg')
         else:
             default_cfgfn = default_cfg_dir_or_fn
 
-        if not os.path.isfile(default_cfgfn):
-            raise ConfigurationDefaultMissingError('Requested default configuration file {0} is '
-                                                   'not a file.'.format(default_cfgfn))
+        if not path.isfile(default_cfgfn):
+            raise ConfigurationDefaultMissingError(
+                'Requested default configuration file {0} is '
+                'not a file.'.format(default_cfgfn))
 
         with open(cfgfn, 'w') as fw:
             with open(default_cfgfn) as fr:
