@@ -1,15 +1,20 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
 
 import io
+import os
+import shutil
+import sys
 
 try:
-    #used by test_get_config_items
+    # used by test_get_config_items
     from ..configuration import ConfigurationItem
 
     TESTCONF1 = ConfigurationItem('test1', 1, 'descr')
     TESTCONF2 = ConfigurationItem('test2', 2, 'descr')
 except:
-    #if this fails on import, don't worry - the tests will catch it.
+    # if this fails on import, don't worry - the tests will catch it.
     pass
 
 
@@ -22,7 +27,6 @@ def test_paths():
 
 def test_config_file():
     from ..configuration import get_config, reload_config, save_config
-    from os.path import exists
 
     apycfg = get_config('astropy')
     assert apycfg.filename.endswith('astropy.cfg')
@@ -34,9 +38,9 @@ def test_config_file():
 
     reload_config('astropy')
 
-    #saving shouldn't change the file, because reload should have made sure it
-    #is based on the current file.  But don't do it if there's no file
-    if exists(apycfg.filename):
+    # saving shouldn't change the file, because reload should have made sure it
+    # is based on the current file.  But don't do it if there's no file
+    if os.path.exists(apycfg.filename):
         save_config('astropy')
 
 
@@ -59,8 +63,8 @@ def test_configitem():
     assert ci() == 32
     assert sec.comments['tstnm'][1] == 'updated Descr'
 
-    #It's useful to go back to the default to allow other test functions to
-    #call this one and still be in the default configuration.
+    # It's useful to go back to the default to allow other test functions to
+    # call this one and still be in the default configuration.
     ci.description = 'this is a Description'
     ci.set(34)
     assert ci() == 34
@@ -69,12 +73,11 @@ def test_configitem():
 
 def test_configitem_save(tmpdir):
     from ..configuration import ConfigurationItem, get_config
-    from shutil import copy
 
     ci = ConfigurationItem('tstnm2', 42, 'this is another Description')
     apycfg = get_config(ci.module)
 
-    #now try saving
+    # now try saving
 
     while apycfg.parent is not apycfg:
         apycfg = apycfg.parent
@@ -92,7 +95,7 @@ def test_configitem_save(tmpdir):
         # We had used LocalPath's `copy` method here, but it copies
         # the file in TEXT mode, destroying newlines on Windows.  We
         # need to do a binary copy.
-        copy(tmpdir.join('astropy.cfg').strpath,
+        shutil.copy(tmpdir.join('astropy.cfg').strpath,
              tmpdir.join('astropy.cfg2').strpath)
         # tmpdir.join('astropy.cfg').copy(tmpdir.join('astropy.cfg2'))
         apycfg.filename = tmpdir.join('astropy.cfg2').realpath().strpath
@@ -112,7 +115,7 @@ def test_configitem_save(tmpdir):
             assert '[config.tests.test_configs]' in lns
             assert 'tstnm2 = 31' in lns
 
-        #also try to save one that doesn't yet exist
+        # also try to save one that doesn't yet exist
         apycfg.filename = tmpdir.join('astropy.cfg3').realpath().strpath
         ci.save()
 
@@ -166,7 +169,7 @@ def test_configitem_options(tmpdir):
         cio.set('op5')
     assert sec['tstnmo'] == 'op2'
 
-    #now try saving
+    # now try saving
     apycfg = sec
     while apycfg.parent is not apycfg:
         apycfg = apycfg.parent
@@ -188,7 +191,7 @@ def test_config_noastropy_fallback(monkeypatch, recwarn):
     from ...tests.helper import pytest
     from .. import paths, configuration
 
-    #make sure the config directory is not searched
+    # make sure the config directory is not searched
     monkeypatch.setenv('XDG_CONFIG_HOME', 'foo')
     monkeypatch.delenv('XDG_CONFIG_HOME')
 
@@ -202,7 +205,7 @@ def test_config_noastropy_fallback(monkeypatch, recwarn):
     monkeypatch.setattr(configuration, '_cfgobjs', {})
 
     with pytest.raises(OSError):
-        #make sure the config dir search fails
+        # make sure the config dir search fails
         paths.get_config_dir()
 
     # now run the basic tests, and make sure the warning about no astropy
@@ -219,7 +222,6 @@ def test_get_config_items():
     """ Checks if the get_config_items function is working correctly, using
     `ConfigurationItem` objects from this module.
     """
-    import sys
 
     from ..configuration import get_config_items
 
