@@ -5,6 +5,8 @@ This subpackage contains classes and functions for celestial coordinates
 of astronomical objects. It also contains a framework for conversions
 between coordinate systems.
 """
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
 
 from .errors import *
 from .angles import *
@@ -13,5 +15,7 @@ from .distances import *
 from .transformations import *
 from .builtin_systems import *
 from .name_resolve import *
+from .matching import *
+from .old_builtin_systems_names import *  # TODO: remove this in next version, along with module file
 
-__doc__ += builtin_systems._transform_graph_docs
\ No newline at end of file
+__doc__ += builtin_systems._transform_graph_docs
