@@ -4,18 +4,22 @@
 This module contains the implementations of specific coordinate systems
 and the conversions between them.
 """
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+import inspect
+import textwrap
 
 import numpy as np
 
+from ..extern import six
 from .angles import Angle
 from .coordsystems import SphericalCoordinatesBase
 from ..time import Time
 from . import transformations
 from .. import units as u
 
-__all__ = ['ICRSCoordinates', 'FK5Coordinates', 'FK4Coordinates',
-           'FK4NoETermCoordinates', 'GalacticCoordinates', 'HorizontalCoordinates'
-          ]
+__all__ = ['ICRS', 'FK5', 'FK4', 'FK4NoETerms', 'Galactic', 'AltAz']
 
 # The UTC time scale is not properly defined prior to 1960, so Time('B1950',
 # scale='utc') will emit a warning. Instead, we use Time('B1950', scale='tai')
@@ -26,12 +30,12 @@ _EQUINOX_B1950 = Time('B1950', scale='tai')
 
 #<--------------Coordinate definitions; transformations are below-------------->
 @transformations.coordinate_alias('icrs')
-class ICRSCoordinates(SphericalCoordinatesBase):
+class ICRS(SphericalCoordinatesBase):
     """
     A coordinate in the ICRS.
 
     If you're looking for "J2000" coordinates, and aren't sure if you
-    want to use this or `FK5Coordinates`, you probably want to use ICRS.
+    want to use this or `FK5`, you probably want to use ICRS.
     It's more well-defined as a catalog coordinate and is an inertial
     system.
 
@@ -44,14 +48,14 @@ class ICRSCoordinates(SphericalCoordinatesBase):
         to be the same as the `equinox`.
 
     Alternatively, a single argument that is any kind of spherical coordinate
-    can be provided, and will be converted to ICRSCoordinates and used as this
+    can be provided, and will be converted to ICRS and used as this
     coordinate.
 
     """
     __doc__ = __doc__.format(params=SphericalCoordinatesBase._init_docstring_param_templ.format(lonnm='ra', latnm='dec'))
 
     def __init__(self, *args, **kwargs):
-        super(ICRSCoordinates, self).__init__()
+        super(ICRS, self).__init__()
 
         self._obstime = kwargs.pop('obstime', None)
 
@@ -60,29 +64,26 @@ class ICRSCoordinates(SphericalCoordinatesBase):
 
         if len(args) == 1 and len(kwargs) == 0 and isinstance(args[0], SphericalCoordinatesBase):
             newcoord = args[0].transform_to(self.__class__)
-            self.ra = newcoord.ra
-            self.dec = newcoord.dec
+            self._latangle = newcoord._latangle
+            self._lonangle = newcoord._lonangle
             self._distance = newcoord._distance
         else:
-            super(ICRSCoordinates, self)._initialize_latlon('ra', 'dec', True, args, kwargs)
+            super(ICRS, self)._initialize_latlon('ra', 'dec', args, kwargs)
 
-    def __repr__(self):
-        if self.distance is not None:
-            diststr = ', Distance={0:.2g} {1!s}'.format(self.distance._value, self.distance._unit)
-        else:
-            diststr = ''
+    #strings used for making __repr__ work
+    _repr_lon_name = 'RA'
+    _repr_lat_name = 'Dec'
 
-        msg = "<{0} RA={1:.5f} deg, Dec={2:.5f} deg{3}>"
-        return msg.format(self.__class__.__name__, self.ra.degrees,
-                          self.dec.degrees, diststr)
+    # Default format for to_string
+    _default_string_style = 'hmsdms'
 
     @property
-    def lonangle(self):
-        return self.ra
+    def ra(self):
+        return self._lonangle
 
     @property
-    def latangle(self):
-        return self.dec
+    def dec(self):
+        return self._latangle
 
     @property
     def equinox(self):
@@ -97,7 +98,7 @@ class ICRSCoordinates(SphericalCoordinatesBase):
 
 
 @transformations.coordinate_alias('fk5')
-class FK5Coordinates(SphericalCoordinatesBase):
+class FK5(SphericalCoordinatesBase):
     """
     A coordinate in the FK5 system.
 
@@ -111,13 +112,13 @@ class FK5Coordinates(SphericalCoordinatesBase):
         to be the same as the `equinox`.
 
     Alternatively, a single argument that is any kind of spherical coordinate
-    can be provided, and will be converted to `FK5Coordinates` and used as this
+    can be provided, and will be converted to `FK5` and used as this
     coordinate.
     """
     __doc__ = __doc__.format(params=SphericalCoordinatesBase._init_docstring_param_templ.format(lonnm='ra', latnm='dec'))
 
     def __init__(self, *args, **kwargs):
-        super(FK5Coordinates, self).__init__()
+        super(FK5, self).__init__()
 
         self._equinox = kwargs.pop('equinox', _EQUINOX_J2000)
         self._obstime = kwargs.pop('obstime', None)
@@ -126,32 +127,31 @@ class FK5Coordinates(SphericalCoordinatesBase):
             raise TypeError('specified equinox is not a Time object')
         if self._obstime is not None and not isinstance(self._obstime, Time):
             raise TypeError('specified obstime is not None or a Time object')
+        if not self._equinox.isscalar:
+            raise ValueError('specified equinox is not a scalar')
 
         if len(args) == 1 and len(kwargs) == 0 and isinstance(args[0], SphericalCoordinatesBase):
             newcoord = args[0].transform_to(self.__class__)
-            self.ra = newcoord.ra
-            self.dec = newcoord.dec
+            self._latangle = newcoord._latangle
+            self._lonangle = newcoord._lonangle
             self._distance = newcoord._distance
         else:
-            super(FK5Coordinates, self)._initialize_latlon('ra', 'dec', True, args, kwargs)
+            super(FK5, self)._initialize_latlon('ra', 'dec', args, kwargs)
 
-    def __repr__(self):
-        if self.distance is not None:
-            diststr = ', Distance={0:.2g} {1!s}'.format(self.distance._value, self.distance._unit)
-        else:
-            diststr = ''
+    #strings used for making __repr__ work
+    _repr_lon_name = 'RA'
+    _repr_lat_name = 'Dec'
 
-        msg = "<{0} RA={1:.5f} deg, Dec={2:.5f} deg{3}>"
-        return msg.format(self.__class__.__name__, self.ra.degrees,
-                          self.dec.degrees, diststr)
+    # Default format for to_string
+    _default_string_style = 'hmsdms'
 
     @property
-    def lonangle(self):
-        return self.ra
+    def ra(self):
+        return self._lonangle
 
     @property
-    def latangle(self):
-        return self.dec
+    def dec(self):
+        return self._latangle
 
     @property
     def equinox(self):
@@ -176,24 +176,24 @@ class FK5Coordinates(SphericalCoordinatesBase):
 
         Returns
         -------
-        newcoord : FK5Coordinates
+        newcoord : FK5
             The new coordinate
         """
         from .earth_orientation import precession_matrix_Capitaine
 
         pmat = precession_matrix_Capitaine(self._equinox, newequinox)
 
-        v = [self.x, self.y, self.z]
+        v = [self.x.value, self.y.value, self.z.value]
         x, y, z = np.dot(pmat.A, v)
 
         if self.distance is not None:
-            return self.__class__(x=x, y=y, z=z, unit=self.distance._unit, equinox=newequinox)
+            return self.__class__(x=x, y=y, z=z, unit=self.distance.unit, equinox=newequinox)
         else:
             return self.__class__(x=x, y=y, z=z, equinox=newequinox)
 
 
 @transformations.coordinate_alias('fk4')
-class FK4Coordinates(SphericalCoordinatesBase):
+class FK4(SphericalCoordinatesBase):
     """
     A coordinate in the FK4 system.
 
@@ -208,13 +208,13 @@ class FK4Coordinates(SphericalCoordinatesBase):
         to be the same as the `equinox`.
 
     Alternatively, a single argument that is any kind of spherical coordinate
-    can be provided, and will be converted to `FK4Coordinates` and used as this
+    can be provided, and will be converted to `FK4` and used as this
     coordinate.
     """
     __doc__ = __doc__.format(params=SphericalCoordinatesBase._init_docstring_param_templ.format(lonnm='ra', latnm='dec'))
 
     def __init__(self, *args, **kwargs):
-        super(FK4Coordinates, self).__init__()
+        super(FK4, self).__init__()
 
         self._equinox = kwargs.pop('equinox', _EQUINOX_B1950)
         self._obstime = kwargs.pop('obstime', None)
@@ -223,32 +223,31 @@ class FK4Coordinates(SphericalCoordinatesBase):
             raise TypeError('specified equinox is not a Time object')
         if self._obstime is not None and not isinstance(self._obstime, Time):
             raise TypeError('specified obstime is not None or a Time object')
+        if not self._equinox.isscalar:
+            raise ValueError('specified equinox is not a scalar')
 
         if len(args) == 1 and len(kwargs) == 0 and isinstance(args[0], SphericalCoordinatesBase):
             newcoord = args[0].transform_to(self.__class__)
-            self.ra = newcoord.ra
-            self.dec = newcoord.dec
+            self._latangle = newcoord._latangle
+            self._lonangle = newcoord._lonangle
             self._distance = newcoord._distance
         else:
-            super(FK4Coordinates, self)._initialize_latlon('ra', 'dec', True, args, kwargs)
+            super(FK4, self)._initialize_latlon('ra', 'dec', args, kwargs)
 
-    def __repr__(self):
-        if self.distance is not None:
-            diststr = ', Distance={0:.2g} {1!s}'.format(self.distance._value, self.distance._unit)
-        else:
-            diststr = ''
+    #strings used for making __repr__ work
+    _repr_lon_name = 'RA'
+    _repr_lat_name = 'Dec'
 
-        msg = "<{0} RA={1:.5f} deg, Dec={2:.5f} deg{3}>"
-        return msg.format(self.__class__.__name__, self.ra.degrees,
-                          self.dec.degrees, diststr)
+    # Default format for to_string
+    _default_string_style = 'hmsdms'
 
     @property
-    def lonangle(self):
-        return self.ra
+    def ra(self):
+        return self._lonangle
 
     @property
-    def latangle(self):
-        return self.dec
+    def dec(self):
+        return self._latangle
 
     @property
     def equinox(self):
@@ -272,24 +271,24 @@ class FK4Coordinates(SphericalCoordinatesBase):
 
         Returns
         -------
-        newcoord : FK4Coordinates
+        newcoord : FK4
             The new coordinate
         """
         from .earth_orientation import _precession_matrix_besselian
 
         pmat = _precession_matrix_besselian(self._equinox.byear, newequinox.byear)
 
-        v = [self.x, self.y, self.z]
+        v = [self.x.value, self.y.value, self.z.value]
         x, y, z = np.dot(pmat.A, v)
 
         if self.distance is not None:
-            return self.__class__(x=x, y=y, z=z, unit=self.distance._unit, equinox=newequinox)
+            return self.__class__(x=x, y=y, z=z, unit=self.distance.unit, equinox=newequinox)
         else:
             return self.__class__(x=x, y=y, z=z, equinox=newequinox)
 
 
 @transformations.coordinate_alias('fk4_no_e')
-class FK4NoETermCoordinates(SphericalCoordinatesBase):
+class FK4NoETerms(SphericalCoordinatesBase):
     """
     A coordinate in the FK4 system.
 
@@ -304,13 +303,13 @@ class FK4NoETermCoordinates(SphericalCoordinatesBase):
         to be the same as the `equinox`.
 
     Alternatively, a single argument that is any kind of spherical coordinate
-    can be provided, and will be converted to `FK4NoETermCoordinates` and used as this
+    can be provided, and will be converted to `FK4NoETerms` and used as this
     coordinate.
     """
     __doc__ = __doc__.format(params=SphericalCoordinatesBase._init_docstring_param_templ.format(lonnm='ra', latnm='dec'))
 
     def __init__(self, *args, **kwargs):
-        super(FK4NoETermCoordinates, self).__init__()
+        super(FK4NoETerms, self).__init__()
 
         self._equinox = kwargs.pop('equinox', _EQUINOX_B1950)
         self._obstime = kwargs.pop('obstime', None)
@@ -319,32 +318,31 @@ class FK4NoETermCoordinates(SphericalCoordinatesBase):
             raise TypeError('specified equinox is not a Time object')
         if self._obstime is not None and not isinstance(self._obstime, Time):
             raise TypeError('specified obstime is not None or a Time object')
+        if not self._equinox.isscalar:
+            raise ValueError('specified equinox is not a scalar')
 
         if len(args) == 1 and len(kwargs) == 0 and isinstance(args[0], SphericalCoordinatesBase):
             newcoord = args[0].transform_to(self.__class__)
-            self.ra = newcoord.ra
-            self.dec = newcoord.dec
+            self._latangle = newcoord._latangle
+            self._lonangle = newcoord._lonangle
             self._distance = newcoord._distance
         else:
-            super(FK4NoETermCoordinates, self)._initialize_latlon('ra', 'dec', True, args, kwargs)
+            super(FK4NoETerms, self)._initialize_latlon('ra', 'dec', args, kwargs)
 
-    def __repr__(self):
-        if self.distance is not None:
-            diststr = ', Distance={0:.2g} {1!s}'.format(self.distance._value, self.distance._unit)
-        else:
-            diststr = ''
+    #strings used for making __repr__ work
+    _repr_lon_name = 'RA'
+    _repr_lat_name = 'Dec'
 
-        msg = "<{0} RA={1:.5f} deg, Dec={2:.5f} deg{3}>"
-        return msg.format(self.__class__.__name__, self.ra.degrees,
-                          self.dec.degrees, diststr)
+    # Default format for to_string
+    _default_string_style = 'hmsdms'
 
     @property
-    def lonangle(self):
-        return self.ra
+    def ra(self):
+        return self._lonangle
 
     @property
-    def latangle(self):
-        return self.dec
+    def dec(self):
+        return self._latangle
 
     @property
     def equinox(self):
@@ -368,24 +366,24 @@ class FK4NoETermCoordinates(SphericalCoordinatesBase):
 
         Returns
         -------
-        newcoord : FK4NoETermCoordinates
+        newcoord : FK4NoETerms
             The new coordinate
         """
         from .earth_orientation import _precession_matrix_besselian
 
         pmat = _precession_matrix_besselian(self._equinox.byear, newequinox.byear)
 
-        v = [self.x, self.y, self.z]
+        v = [self.x.value, self.y.value, self.z.value]
         x, y, z = np.dot(pmat.A, v)
 
         if self.distance is not None:
-            return self.__class__(x=x, y=y, z=z, unit=self.distance._unit, equinox=newequinox)
+            return self.__class__(x=x, y=y, z=z, unit=self.distance.unit, equinox=newequinox)
         else:
             return self.__class__(x=x, y=y, z=z, equinox=newequinox)
 
 
 @transformations.coordinate_alias('galactic')
-class GalacticCoordinates(SphericalCoordinatesBase):
+class Galactic(SphericalCoordinatesBase):
     """
     A coordinate in Galactic Coordinates.
 
@@ -408,7 +406,7 @@ class GalacticCoordinates(SphericalCoordinatesBase):
         to be the same as the `equinox`.
 
     Alternatively, a single argument that is any kind of spherical coordinate
-    can be provided, and will be converted to `GalacticCoordinates` and
+    can be provided, and will be converted to `Galactic` and
     used as this coordinate.
     """
     __doc__ = __doc__.format(params=SphericalCoordinatesBase._init_docstring_param_templ.format(lonnm='l', latnm='b'))
@@ -417,14 +415,14 @@ class GalacticCoordinates(SphericalCoordinatesBase):
     # transformations to/from FK4/5
 
     # These are from Reid & Brunthaler 2004
-    _ngp_J2000 = FK5Coordinates(192.859508, 27.128336, unit=(u.degree, u.degree))
+    _ngp_J2000 = FK5(192.859508, 27.128336, unit=(u.degree, u.degree))
     _lon0_J2000 = Angle(122.932, unit=u.degree)
     # These are from the IAU's definition of galactic coordinates
-    _ngp_B1950 = FK4Coordinates(192.25, 27.4, unit=(u.degree, u.degree))
+    _ngp_B1950 = FK4(192.25, 27.4, unit=(u.degree, u.degree))
     _lon0_B1950 = Angle(123, unit=u.degree)
 
     def __init__(self, *args, **kwargs):
-        super(GalacticCoordinates, self).__init__()
+        super(Galactic, self).__init__()
 
         self._obstime = kwargs.pop('obstime', None)
 
@@ -433,35 +431,32 @@ class GalacticCoordinates(SphericalCoordinatesBase):
 
         if len(args) == 1 and len(kwargs) == 0 and isinstance(args[0], SphericalCoordinatesBase):
             newcoord = args[0].transform_to(self.__class__)
-            self.l = newcoord.l
-            self.b = newcoord.b
+            self._latangle = newcoord._latangle
+            self._lonangle = newcoord._lonangle
             self._distance = newcoord._distance
         else:
-            super(GalacticCoordinates, self)._initialize_latlon('l', 'b', False, args, kwargs)
+            super(Galactic, self)._initialize_latlon('l', 'b', args, kwargs)
 
-    def __repr__(self):
-        if self.distance is not None:
-            diststr = ', Distance={0:.2g} {1!s}'.format(self.distance._value, self.distance._unit)
-        else:
-            diststr = ''
+    #strings used for making __repr__ work
+    _repr_lon_name = 'l'
+    _repr_lat_name = 'b'
 
-        msg = "<{0} l={1:.5f} deg, b={2:.5f} deg{3}>"
-        return msg.format(self.__class__.__name__, self.l.degrees,
-                          self.b.degrees, diststr)
+    # Default format for to_string
+    _default_string_style = 'dms'
 
     @property
-    def lonangle(self):
-        return self.l
+    def l(self):
+        return self._lonangle
 
     @property
-    def latangle(self):
-        return self.b
+    def b(self):
+        return self._latangle
 
 
-@transformations.coordinate_alias('horizontal')
-class HorizontalCoordinates(SphericalCoordinatesBase):
+@transformations.coordinate_alias('altaz')
+class AltAz(SphericalCoordinatesBase):
     """
-    A coordinate in the Horizontal or "az/el" system.
+    A coordinate in the altitude/azimuth or "horizontal" system.
 
     Parameters
     ----------
@@ -473,13 +468,13 @@ class HorizontalCoordinates(SphericalCoordinatesBase):
         to be the same as the `equinox`.
 
     Alternatively, a single argument that is any kind of spherical coordinate
-    can be provided, and will be converted to `HorizontalCoordinates` and used
+    can be provided, and will be converted to `AltAz` and used
     as this coordinate.
     """
     __doc__ = __doc__.format(params=SphericalCoordinatesBase._init_docstring_param_templ.format(lonnm='az', latnm='el'))
 
     def __init__(self, *args, **kwargs):
-        super(HorizontalCoordinates, self).__init__()
+        super(AltAz, self).__init__()
 
         self._equinox = kwargs.pop('equinox', _EQUINOX_J2000)
         self._obstime = kwargs.pop('obstime', None)
@@ -488,32 +483,31 @@ class HorizontalCoordinates(SphericalCoordinatesBase):
             raise TypeError('specified equinox is not a Time object')
         if self._obstime is not None and not isinstance(self._obstime, Time):
             raise TypeError('specified obstime is not None or a Time object')
+        if not self._equinox.isscalar:
+            raise ValueError('specified equinox is not a scalar')
 
         if len(args) == 1 and len(kwargs) == 0 and isinstance(args[0], SphericalCoordinatesBase):
             newcoord = args[0].transform_to(self.__class__)
-            self.az = newcoord.az
-            self.el = newcoord.el
+            self._latangle = newcoord._latangle
+            self._lonangle = newcoord._lonangle
             self._distance = newcoord._distance
         else:
-            super(HorizontalCoordinates, self)._initialize_latlon('az', 'el', False, args, kwargs)
+            super(AltAz, self)._initialize_latlon('az', 'alt', args, kwargs)
 
-    def __repr__(self):
-        if self.distance is not None:
-            diststr = ', Distance={0:.2g} {1!s}'.format(self.distance._value, self.distance._unit)
-        else:
-            diststr = ''
+    #strings used for making __repr__ work
+    _repr_lon_name = 'az'
+    _repr_lat_name = 'alt'
 
-        msg = "<{0} az={1:.5f} deg, el={2:.5f} deg{3}>"
-        return msg.format(self.__class__.__name__, self.az.degrees,
-                          self.el.degrees, diststr)
+    # Default format for to_string
+    _default_string_style = 'dms'
 
     @property
-    def lonangle(self):
-        return self.az
+    def az(self):
+        return self._lonangle
 
     @property
-    def latangle(self):
-        return self.el
+    def alt(self):
+        return self._latangle
 
     @property
     def equinox(self):
@@ -529,7 +523,7 @@ class HorizontalCoordinates(SphericalCoordinatesBase):
 
 #<--------------------------------transformations------------------------------>
 # ICRS to/from FK5
-@transformations.static_transform_matrix(ICRSCoordinates, FK5Coordinates)
+@transformations.static_transform_matrix(ICRS, FK5)
 def icrs_to_fk5():
     """
     B-matrix from USNO circular 179
@@ -548,7 +542,7 @@ def icrs_to_fk5():
 
 
 # can't be static because the equinox is needed
-@transformations.dynamic_transform_matrix(FK5Coordinates, ICRSCoordinates)
+@transformations.dynamic_transform_matrix(FK5, ICRS)
 def fk5_to_icrs(fk5c):
     from .earth_orientation import _precess_from_J2000_Capitaine
 
@@ -562,8 +556,8 @@ def fk5_to_icrs(fk5c):
 # FK4-NO-E to/from FK4
 
 # In the present framework, we include two coordinate classes for FK4
-# coordinates - one including the E-terms of aberration (FK4Coordinates), and
-# one not including them (FK4NoETermCoordinates). In the following functions,
+# coordinates - one including the E-terms of aberration (FK4), and
+# one not including them (FK4NoETerms). In the following functions,
 # we describe the transformation between these two.
 
 def fk4_e_terms(equinox):
@@ -598,18 +592,19 @@ def fk4_e_terms(equinox):
            -e * k * np.cos(g) * np.sin(o)
 
 
-@transformations.transform_function(FK4Coordinates, FK4NoETermCoordinates, priority=1)
+@transformations.transform_function(FK4, FK4NoETerms, priority=1)
 def fk4_to_fk4_no_e(fk4c):
 
     # Extract cartesian vector
-    r = np.array([fk4c.x, fk4c.y, fk4c.z])
+    c = fk4c.cartesian
+    r = np.asarray(c.reshape((3, c.size // 3)))
 
     # Find distance (for re-normalization)
     d_orig = np.sqrt(np.sum(r ** 2))
 
     # Apply E-terms of aberration
-    eterms_a = fk4_e_terms(fk4c.equinox)
-    r = r - eterms_a + np.dot(r, eterms_a) * r
+    eterms_a = np.asarray(fk4_e_terms(fk4c.equinox))
+    r = r - eterms_a.reshape(3, 1) + np.dot(eterms_a, r) * r
 
     # Find new distance (for re-normalization)
     d_new = np.sqrt(np.sum(r ** 2))
@@ -617,26 +612,30 @@ def fk4_to_fk4_no_e(fk4c):
     # Renormalize
     r = r * d_orig / d_new
 
-    unit = None if fk4c.distance is None else fk4c.distance._unit
-    result = FK4NoETermCoordinates(x=r[0], y=r[1], z=r[2], unit=unit, equinox=fk4c.equinox)
+    subshape = c.shape[1:]
+    x = r[0].reshape(subshape)
+    y = r[1].reshape(subshape)
+    z = r[2].reshape(subshape)
 
-    return result
+    newunit = None if fk4c.distance is None else fk4c.distance.unit
+    return FK4NoETerms(x=x, y=y, z=z, unit=newunit, equinox=fk4c.equinox)
 
 
-@transformations.transform_function(FK4NoETermCoordinates, FK4Coordinates, priority=1)
+@transformations.transform_function(FK4NoETerms, FK4, priority=1)
 def fk4_no_e_to_fk4(fk4c):
 
     # Extract cartesian vector
-    r = np.array([fk4c.x, fk4c.y, fk4c.z])
+    c = fk4c.cartesian
+    r = np.asarray(c.reshape((3, c.size // 3)))
 
     # Find distance (for re-normalization)
     d_orig = np.sqrt(np.sum(r ** 2))
 
     # Apply E-terms of aberration
-    eterms_a = fk4_e_terms(fk4c.equinox)
+    eterms_a = np.asarray(fk4_e_terms(fk4c.equinox))
     r0 = r.copy()
-    for j in range(10):
-        r = (r0 + eterms_a) / (1. + np.dot(r, eterms_a))
+    for _ in range(10):
+        r = (eterms_a.reshape(3, 1) + r0) / (1. + np.dot(eterms_a, r))
 
     # Find new distance (for re-normalization)
     d_new = np.sqrt(np.sum(r ** 2))
@@ -644,10 +643,13 @@ def fk4_no_e_to_fk4(fk4c):
     # Renormalize
     r = r * d_orig / d_new
 
-    unit = None if fk4c.distance is None else fk4c.distance._unit
-    result = FK4Coordinates(x=r[0], y=r[1], z=r[2], unit=unit, equinox=fk4c.equinox)
+    subshape = c.shape[1:]
+    x = r[0].reshape(subshape)
+    y = r[1].reshape(subshape)
+    z = r[2].reshape(subshape)
 
-    return result
+    newunit = None if fk4c.distance is None else fk4c.distance.unit
+    return FK4(x=x, y=y, z=z, unit=newunit, equinox=fk4c.equinox)
 
 # FK5 to/from FK4
 
@@ -667,7 +669,7 @@ FK4_CORR = \
             [-2.1112979048, -0.0056024448, +0.0102587734]]) * 1.e-6
 
 # This transformation can't be static because the observation date is needed.
-@transformations.dynamic_transform_matrix(FK4NoETermCoordinates, FK5Coordinates, priority=1)
+@transformations.dynamic_transform_matrix(FK4NoETerms, FK5, priority=1)
 def fk4_no_e_to_fk5(fk4c, skip_precession=False):
 
     # Add in correction terms for FK4 rotating system - Murray 89 eqn 29
@@ -684,7 +686,7 @@ def fk4_no_e_to_fk5(fk4c, skip_precession=False):
         return B * _precession_matrix_besselian(fk4c.equinox.byear, 1950)
 
 # This transformation can't be static because the observation date is needed.
-@transformations.dynamic_transform_matrix(FK5Coordinates, FK4NoETermCoordinates, priority=1)
+@transformations.dynamic_transform_matrix(FK5, FK4NoETerms, priority=1)
 def fk5_to_fk4_no_e(fk5c):
 
     # Get transposed matrix from FK4 -> FK5 assuming equinox B1950 -> J2000
@@ -697,9 +699,9 @@ def fk5_to_fk4_no_e(fk5c):
         from .earth_orientation import precession_matrix_Capitaine
         return B * precession_matrix_Capitaine(fk5c.equinox, _EQUINOX_J2000)
 
-# GalacticCoordinates to/from FK4/FK5
+# Galactic to/from FK4/FK5
 # can't be static because the equinox is needed
-@transformations.dynamic_transform_matrix(FK5Coordinates, GalacticCoordinates)
+@transformations.dynamic_transform_matrix(FK5, Galactic)
 def _fk5_to_gal(fk5coords):
     from .angles import rotation_matrix
     from .earth_orientation import _precess_from_J2000_Capitaine
@@ -707,20 +709,20 @@ def _fk5_to_gal(fk5coords):
     # needed mainly to support inverse from galactic
     jequinox = 2000 if fk5coords.equinox is None else fk5coords.equinox.jyear
 
-    mat1 = rotation_matrix(180 - GalacticCoordinates._lon0_J2000.degrees, 'z')
-    mat2 = rotation_matrix(90 - GalacticCoordinates._ngp_J2000.dec.degrees, 'y')
-    mat3 = rotation_matrix(GalacticCoordinates._ngp_J2000.ra.degrees, 'z')
+    mat1 = rotation_matrix(180 - Galactic._lon0_J2000.degree, 'z')
+    mat2 = rotation_matrix(90 - Galactic._ngp_J2000.dec.degree, 'y')
+    mat3 = rotation_matrix(Galactic._ngp_J2000.ra.degree, 'z')
     # transpose gets equinox -> J2000
     matprec = _precess_from_J2000_Capitaine(jequinox).T
     return mat1 * mat2 * mat3 * matprec
 
 
-@transformations.dynamic_transform_matrix(GalacticCoordinates, FK5Coordinates)
+@transformations.dynamic_transform_matrix(Galactic, FK5)
 def _gal_to_fk5(galcoords):
     return _fk5_to_gal(galcoords).T
 
 
-@transformations.dynamic_transform_matrix(FK4NoETermCoordinates, GalacticCoordinates, priority=1)
+@transformations.dynamic_transform_matrix(FK4NoETerms, Galactic, priority=1)
 def _fk4_to_gal(fk4coords):
     from .angles import rotation_matrix
     from .earth_orientation import _precession_matrix_besselian
@@ -728,14 +730,14 @@ def _fk4_to_gal(fk4coords):
     # needed mainly to support inverse from galactic
     bequinox = 1950 if fk4coords.equinox is None else fk4coords.equinox.byear
 
-    mat1 = rotation_matrix(180 - GalacticCoordinates._lon0_B1950.degrees, 'z')
-    mat2 = rotation_matrix(90 - GalacticCoordinates._ngp_B1950.dec.degrees, 'y')
-    mat3 = rotation_matrix(GalacticCoordinates._ngp_B1950.ra.degrees, 'z')
+    mat1 = rotation_matrix(180 - Galactic._lon0_B1950.degree, 'z')
+    mat2 = rotation_matrix(90 - Galactic._ngp_B1950.dec.degree, 'y')
+    mat3 = rotation_matrix(Galactic._ngp_B1950.ra.degree, 'z')
     matprec = _precession_matrix_besselian(bequinox, 1950)
     return mat1 * mat2 * mat3 * matprec
 
 
-@transformations.dynamic_transform_matrix(GalacticCoordinates, FK4NoETermCoordinates, priority=1)
+@transformations.dynamic_transform_matrix(Galactic, FK4NoETerms, priority=1)
 def _gal_to_fk4(galcoords):
     return _fk4_to_gal(galcoords).T
 
@@ -745,12 +747,11 @@ def _make_transform_graph_docs():
     Generates a string for use with the coordinate package's docstring
     to show the available transforms and coordinate systems
     """
-    from inspect import isclass
-    from textwrap import dedent
 
     from .transformations import master_transform_graph
 
-    coosys = [item for item in globals().values()
+    isclass = inspect.isclass
+    coosys = [item for item in list(six.itervalues(globals()))
               if isclass(item) and issubclass(item, SphericalCoordinatesBase)]
     coosys.remove(SphericalCoordinatesBase)
     graphstr = master_transform_graph.to_dot_graph(addnodes=coosys)
@@ -768,15 +769,12 @@ def _make_transform_graph_docs():
     number next to the arrow.  These priorities are used to decide the
     preferred order when two trasnformation paths have the same number
     of steps.  These priorities are defined such that path with a
-    *smaller* total priority are favored over larger.
-    E.g., the path from `ICRSCoordinates` to `GalacticCoordinates` goes
-    through `FK5Coordinates` because the total path length is 2 instead
-    of 2.03.
+    *smaller* total priority are favored over one with larger.
 
 
     .. graphviz::
 
     """
 
-    return dedent(docstr) + '    ' + graphstr.replace('\n', '\n    ')
+    return textwrap.dedent(docstr) + '    ' + graphstr.replace('\n', '\n    ')
 _transform_graph_docs = _make_transform_graph_docs()
