@@ -3,15 +3,23 @@
 """
 This module contains the base classes and frameworks for coordinate objects.
 """
+
+import copy
+
 from abc import ABCMeta, abstractproperty, abstractmethod
 
+from ..extern import six
 from .. import units as u
-from .angles import RA, Dec, Angle, AngularSeparation
-from .distances import *
+from .angles import Longitude, Latitude, Angle
+from .distances import Distance, CartesianPoints, cartesian_to_spherical, spherical_to_cartesian
+from ..utils.compat.misc import override__dir__
+from . import angle_utilities
+import numpy as np
 
 __all__ = ['SphericalCoordinatesBase']
 
 
+@six.add_metaclass(ABCMeta)
 class SphericalCoordinatesBase(object):
     """
     Abstract superclass for all coordinate classes representing points
@@ -30,8 +38,6 @@ class SphericalCoordinatesBase(object):
     `astropy.coordinates.builtin_systems` for examples of this.
     """
 
-    __metaclass__ = ABCMeta
-
     @abstractmethod
     def __init__(self, *args, **kwargs):
         """
@@ -90,7 +96,7 @@ class SphericalCoordinatesBase(object):
         * If `x`, `y`, and `z` are given:
             `unit` must be a single unit with dimensions of length"""
 
-    def _initialize_latlon(self, lonname, latname, useradec, initargs, initkwargs):
+    def _initialize_latlon(self, lonname, latname, initargs, initkwargs):
         """
         Subclasses should use this to initialize standard lat/lon-style
         coordinates.
@@ -103,9 +109,6 @@ class SphericalCoordinatesBase(object):
             The name of the longitude-like coordinate attribute
         latname : str
             The name of the latitude-like coordinate attribute
-        useradec : bool
-            If True, the `RA` and `Dec` classes will be used for the
-            angles.  Otherwise, a basic `Angle` will be used.
         initargs : list
             The ``*args`` from the initializer
         initkwargs : dict
@@ -186,7 +189,7 @@ class SphericalCoordinatesBase(object):
                 # need to try to parse the coordinate from a single argument
                 # populates latval and lonval variables, which then get made
                 # into coordinates below
-                if isinstance(coordstr, basestring):
+                if isinstance(coordstr, six.string_types):
                     if "," in coordstr:
                         lonval, latval = coordstr.split(",")
                     else:
@@ -207,16 +210,8 @@ class SphericalCoordinatesBase(object):
                                      "'{1}', must be a string.".format(sclsnm, type(coordstr).__name__))
 
             # now actually create the angle objects
-            if useradec:
-                lonang = RA(lonval, unit=units[0])
-                latang = Dec(latval, unit=units[1])
-            else:
-                if isinstance(lonval, RA):
-                    raise TypeError('Cannot provide an RA object to non-RA/Dec system {0}'.format(sclsnm))
-                if isinstance(latval, Dec):
-                    raise TypeError('Cannot provide a Dec object to non-RA/Dec system {0}'.format(sclsnm))
-                lonang = Angle(lonval, unit=units[0])
-                latang = Angle(latval, unit=units[1])
+            lonang = Longitude(lonval, unit=units[0])
+            latang = Latitude(latval, unit=units[1])
 
             dist = None if distval is None else Distance(distval)  # copy
 
@@ -239,12 +234,8 @@ class SphericalCoordinatesBase(object):
                 unit = cartpoint.unit
             r, latval, lonval = cartesian_to_spherical(x, y, z)
 
-            if useradec:
-                lonang = RA(lonval, unit=u.radian)
-                latang = Dec(latval, unit=u.radian)
-            else:
-                lonang = Angle(lonval, unit=u.radian)
-                latang = Angle(latval, unit=u.radian)
+            lonang = Longitude(lonval, unit=u.radian)
+            latang = Latitude(latval, unit=u.radian)
 
             dist = None if unit is None else Distance(r, unit)
 
@@ -255,35 +246,90 @@ class SphericalCoordinatesBase(object):
                                       lonname=lonname))
 
         # now actually set the values
-        setattr(self, lonname, lonang)
-        setattr(self, latname, latang)
+        self._lonangle = lonang
+        self._latangle = latang
         self._distance = dist
 
-    @abstractproperty
+
+        #sanity-check that they are all consistent shapes
+        if self.lonangle.shape != self.latangle.shape:
+            raise ValueError('lonangle and latangle do not have matching shapes')
+
+        if self._distance is not None and self._distance.shape != self.lonangle.shape:
+            raise ValueError('distance and angles do not have matching shapes')
+
+    def __repr__(self):
+        if self.distance is not None:
+            if self.isscalar:
+                diststr = ', Distance={0:.2g} {1!s}'.format(self.distance.value, self.distance.unit)
+            else:
+                diststr = ', Distance={0} {1!s}'.format(self.distance.value, self.distance.unit)
+        else:
+            diststr = ''
+
+        if self.isscalar:
+            msg = "<{clsnm} {lonnm}={lonval:.5f} deg, {latnm}={latval:.5f} deg{diststr}>"
+        else:
+            msg = "<{clsnm} {lonnm}={lonval} deg, {latnm}={latval} deg{diststr}>"
+        return msg.format(clsnm=self.__class__.__name__, lonval=self.lonangle.degree,
+                          latval=self.latangle.degree, lonnm=self._repr_lon_name,
+                          latnm=self._repr_lat_name, diststr=diststr)
+
+    def __getitem__(self, key):
+        from copy import deepcopy
+
+        oldlat = self._latangle 
+        oldlon = self._lonangle
+        olddist = self._distance
+
+        newlat = oldlat[key]
+        newlon = oldlon[key]
+        if olddist is not None:
+            newdist = olddist[key]
+        else:
+            newdist = None
+
+        try:
+            #don't want to copy the old values, because we've already
+            #copied them above as new*
+            self._latangle = None
+            self._lonangle = None
+            self._distance = None
+
+            newcoo =  deepcopy(self)
+
+            newcoo._latangle = newlat
+            newcoo._lonangle = newlon
+            newcoo._distance = newdist
+
+            return newcoo
+        finally:
+            self._latangle = oldlat
+            self._lonangle = oldlon
+            self._distance = olddist
+
+
+    @property
     def latangle(self):
         """
         The latitudinal/elevation angle for these coordinates as an
-        `~astropy.coorinates.angles.Angle` object.
+        `~astropy.coordinates.angles.Angle` object.
 
-        .. note ::
-            This should be overridden in subclasses as a read-only property that
-            just returns an attribute a way to abstract the exact choice of
-            names for the coordiantes. E.g., `ICRSCoordinates` implements this
-            by doing ``return self.ra``.
+        Subclasses will often provide properties returning the same object, but
+        with a name more appropriate for the particular subclass.
         """
+        return self._latangle
 
-    @abstractproperty
+    @property
     def lonangle(self):
         """
         The longitudinal/azimuthal angle for these coordinates as an
-        `~astropy.coorinates.angles.Angle` object.
+        `~astropy.coordinates.angles.Angle` object.
 
-        .. note ::
-            This should be overridden in subclasses as a read-only property that
-            just returns an attribute a way to abstract the exact choice of
-            names for the coordinates. E.g., `ICRSCoordinates` implements this
-            by doing ``return self.dec``.
+        Subclasses will often provide properties returning the same object, but
+        with a name more appropriate for the particular subclass.
         """
+        return self._lonangle
 
     @property
     def equinox(self):
@@ -351,38 +397,46 @@ class SphericalCoordinatesBase(object):
         if override or self._cartpoint is None:
             if self._distance is None:
                 r = 1
-                runit = None
+                runit = u.dimensionless_unscaled
             else:
-                r = self._distance._value
-                runit = self._distance._unit
-            x, y, z = spherical_to_cartesian(r, self.latangle.radians,
-                                                self.lonangle.radians)
+                r = self._distance.value
+                runit = self._distance.unit
+            x, y, z = spherical_to_cartesian(r, self.latangle.radian,
+                                                self.lonangle.radian)
             self._cartpoint = CartesianPoints(x, y, z, runit)
 
     def separation(self, other):
         """
         Computes on-sky separation between this coordinate and another.
 
-        See the `~astropy.coordinates.angles.AngularSeparation` docstring
-        for further details on the actual calculation.
-
         Parameters
         ----------
         other : `~astropy.coordinates.coordsystems.SphericalCoordinatesBase`
-            The coordinate system to get the separation to.
+            The coordinate to get the separation to.
 
         Returns
         -------
-        sep : `~astropy.coordinates.angles.AngularSeparation`
+        sep : `~astropy.coordinates.angles.Angle`
             The on-sky separation between this and the `other` coordinate.
+
+        Notes
+        -----
+        The separation is calculated using the Vincenty formula, which
+        is stable at all locations, including poles and antipodes [1]_.
+
+        .. [1] http://en.wikipedia.org/wiki/Great-circle_distance
+
         """
         other_in_self_system = other.transform_to(self.__class__)
 
-        lat1 = self.latangle.radians
-        lat2 = other_in_self_system.latangle.radians
-        lon1 = self.lonangle.radians
-        lon2 = other_in_self_system.lonangle.radians
-        return AngularSeparation(lat1, lon1, lat2, lon2, u.radian)
+        lon1 = self.lonangle
+        lat1 = self.latangle
+        lon2 = other_in_self_system.lonangle
+        lat2 = other_in_self_system.latangle
+
+        # Get the separation as a Quantity, convert to Angle in degrees
+        sep = angle_utilities.angular_separation(lon1, lat1, lon2, lat2)
+        return Angle(sep, unit=u.degree)
 
     def separation_3d(self, other):
         """
@@ -415,14 +469,100 @@ class SphericalCoordinatesBase(object):
             raise ValueError('The other object does not have a distance; '
                              'cannot compute 3d separation.')
 
-        dscale = other_in_self_system._distance._unit.to(self._distance._unit, 1)
+        dx = self.x - other_in_self_system.x
+        dy = self.y - other_in_self_system.y
+        dz = self.z - other_in_self_system.z
 
-        dx = self.x - other_in_self_system.x * dscale
-        dy = self.y - other_in_self_system.y * dscale
-        dz = self.z - other_in_self_system.z * dscale
+        distval = (dx.value ** 2 + dy.value ** 2 + dz.value ** 2) ** 0.5
+        return Distance(distval, dx.unit)
 
-        distval = (dx ** 2 + dy ** 2 + dz ** 2) ** 0.5
-        return Distance(distval, self._distance._unit)
+    def match_to_catalog_3d(self, catalogcoord, nthneighbor=1):
+        """
+        Finds the nearest 3-dimensional matches of this coordinate to a set
+        of catalog coordinates.
+
+        This finds the 3-dimensional closest neighbor, which is only different
+        from the on-sky distance if `distance` is set in this object or the 
+        `catalogcoord` object.
+        
+        Parameters
+        ----------
+        catalogcoord : `~astropy.coordinates.SphericalCoordinatesBase`
+            The base catalog in which to search for matches. Typically this 
+            will be a coordinate object that is an array (i.e., 
+            ``catalogcoord.isscalar == False``)
+        nthneighbor : int, optional
+            Which closest neighbor to search for.  Typically ``1`` is desired here,
+            as that is correct for matching one set of coordinates to another.
+            The next likely use case is ``2``, for matching a coordinate catalog
+            against *itself* (``1`` is inappropriate because each point will find 
+            itself as the closest match).
+
+        Returns
+        -------
+        idx : integer array
+            Indecies into `catalogcoord` to get the matched points for each 
+            `matchcoord`. Shape matches this coordinate.
+        sep2d : `~astropy.units.quantity.Angle` 
+            The on-sky separation between the closest match for each `matchcoord` and 
+            the `matchcoord`. Shape matches `matchcoord`.
+        dist3d : `~astropy.units.quantity.Quantity` 
+            The 3D distance between the closest match for each `matchcoord` and 
+            the `matchcoord`. Shape matches this coordinate.
+
+        Notes
+        -----
+        This method requires `scipy` to be installed or it will fail.
+
+        See Also
+        --------
+        astropy.coordinates.matching.match_coordinates_3d
+        """
+        from .matching import match_coordinates_3d
+
+        return match_coordinates_3d(self, catalogcoord, nthneighbor=nthneighbor, storekdtree=True)
+
+    def match_to_catalog_sky(self, catalogcoord, nthneighbor=1):
+        """
+        Finds the nearest on-sky matches of this coordinate in a set of 
+        catalog coordinates.
+        
+        Parameters
+        ----------
+        catalogcoord : `~astropy.coordinates.SphericalCoordinatesBase`
+            The base catalog in which to search for matches. Typically this 
+            will be a coordinate object that is an array (i.e., 
+            ``catalogcoord.isscalar == False``)
+        nthneighbor : int, optional
+            Which closest neighbor to search for.  Typically ``1`` is desired here,
+            as that is correct for matching one set of coordinates to another.
+            The next likely use case is ``2``, for matching a coordinate catalog
+            against *itself* (``1`` is inappropriate because each point will find 
+            itself as the closest match).
+
+        Returns
+        -------
+        idx : integer array
+            Indecies into `catalogcoord` to get the matched points for each 
+            `matchcoord`. Shape matches this coordinate.
+        sep2d : `~astropy.units.quantity.Angle` 
+            The on-sky separation between the closest match for each `matchcoord` and 
+            the `matchcoord`. Shape matches `matchcoord`.
+        dist3d : `~astropy.units.quantity.Quantity` 
+            The 3D distance between the closest match for each `matchcoord` and 
+            the `matchcoord`. Shape matches this coordinate.
+
+        Notes
+        -----
+        This method requires `scipy` to be installed or it will fail.
+        
+        See Also
+        --------
+        astropy.coordinates.matching.match_coordinates_sky
+        """
+        from .matching import match_coordinates_sky
+
+        return match_coordinates_sky(self, catalogcoord, nthneighbor=nthneighbor, storekdtree=True)
 
     #<------------transformation-related stuff here-------------------->
     def transform_to(self, tosys):
@@ -444,12 +584,12 @@ class SphericalCoordinatesBase(object):
         ValueError
             If there is no possible transformation route.
         """
-        from copy import deepcopy
+
         from .transformations import master_transform_graph
         from .errors import ConvertError
 
         if tosys is self.__class__:
-            return deepcopy(self)
+            return copy.deepcopy(self)
 
         trans = master_transform_graph.get_transform(self.__class__, tosys)
         if trans is None:
@@ -497,6 +637,7 @@ class SphericalCoordinatesBase(object):
             msg = "'{0}' object has no attribute '{1}', nor a transform."
             raise AttributeError(msg.format(self.__class__.__name__, name))
 
+    @override__dir__
     def __dir__(self):
         """
         Overriding the builtin `dir` behavior allows us to add the
@@ -505,16 +646,25 @@ class SphericalCoordinatesBase(object):
         """
         from .transformations import master_transform_graph
 
-        # the stuff `dir` normally gives
-        dir_items = dir(type(self)) + self.__dict__.keys()
+        dir_items = set()
 
         # determine the aliases that this can be transformed to.
         for alias in master_transform_graph.get_aliases():
             tosys = master_transform_graph.lookup_name(alias)
             if self.is_transformable_to(tosys):
-                dir_items.append(alias)
+                dir_items.add(alias)
+
+        return dir_items
+
+    @property
+    def isscalar(self):
+        """
+        True if this coordinate contains scalar angles/distances, or False if
+        they are array-like
+        """
+        #assumes input-validation occurs and thus lat/lon/dist consistent
+        return self.lonangle.isscalar
 
-        return sorted(set(dir_items))
 
     # Name resolve
     @classmethod
@@ -536,8 +686,8 @@ class SphericalCoordinatesBase(object):
         -------
         coord : SphericalCoordinatesBase
             Instance of a Coordinates class, specified by the class this is
-            called on, e.g. if `GalacticCoordinates.from_name('m42')`, will
-            get an instance of `GalacticCoordinates` representing the
+            called on, e.g. if `Galactic.from_name('m42')`, will
+            get an instance of `Galactic` representing the
             position of M42.
         """
 
@@ -548,3 +698,61 @@ class SphericalCoordinatesBase(object):
             return icrs
         else:
             return icrs.transform_to(cls)
+
+    _default_string_style = 'dms'
+
+    def to_string(self, style=None, **kwargs):
+        """
+        A string representation of the coordinates.
+
+        See :meth:`astropy.coordinates.Angle.to_string` for details and keyword
+        arguments (the two angles forming the coordinates are are both
+        :class:`astropy.coordinates.Angle` instances). Keyword arguments are passed to
+        :meth:`astropy.coordinates.Angle.to_string`.
+
+        Parameters
+        ----------
+        style : {'hmsdms', 'dms', 'decimal', None}
+            The formatting specification to use. These encode the three most
+            common ways to represent coordinates. If `None` is passed, the
+            defaults for the current coordinate class is used.
+        kwargs
+            Keyword arguments are passed to :meth:`astropy.coordinates.Angle.to_string`.
+        """
+
+        if style is None:
+            style = self._default_string_style
+
+        styles = {
+                  'hmsdms': {'lonargs': {'unit':u.hour},
+                             'latargs': {'unit':u.degree}},
+                  'dms':    {'lonargs': {'unit':u.degree},
+                             'latargs': {'unit':u.degree}},
+                  'decimal':{'lonargs': {'unit':u.degree,'decimal':True},
+                             'latargs': {'unit':u.degree,'decimal':True}}
+                 }
+
+        lonargs = kwargs.copy()
+        latargs = kwargs.copy()
+
+        if style in styles:
+            lonargs.update(styles[style]['lonargs'])
+            latargs.update(styles[style]['latargs'])
+        else:
+            raise ValueError('Invalid style.  Valid options are: '+",".join(styles))
+
+        if np.isscalar(self.lonangle.value):
+            coord_string = (self.lonangle.to_string(**lonargs)
+                            + " " +
+                            self.latangle.to_string(**latargs))
+        else:
+            coord_string = []
+            for lonangle, latangle in zip(self.lonangle, self.latangle):
+                coord_string += [(lonangle.to_string(**lonargs)
+                                 + " " +
+                                 latangle.to_string(**latargs))]
+
+        if hasattr(coord_string,'decode'):
+            return coord_string.decode()
+
+        return coord_string
