@@ -1,13 +1,12 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-import re
-import glob
-import numpy as np
+
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from io import StringIO
+
 from ... import ascii as asciitable
-
-io = asciitable.core.io
-
-from .common import (raises,
-                     assert_equal, assert_almost_equal, assert_true,
+from .common import (assert_equal, assert_almost_equal,
                      setup_function, teardown_function)
 
 
@@ -25,7 +24,7 @@ def test_read_normal():
 """
     reader = asciitable.get_reader(Reader=asciitable.FixedWidth)
     dat = reader.read(table)
-    assert_equal(reader.header.colnames, ('Col1', 'Col2'))
+    assert_equal(dat.colnames, ['Col1', 'Col2'])
     assert_almost_equal(dat[1][0], 2.4)
     assert_equal(dat[0][1], '"hello"')
     assert_equal(dat[1][1], "'s worlds")
@@ -42,7 +41,7 @@ def test_read_normal_names():
     reader = asciitable.get_reader(Reader=asciitable.FixedWidth,
                                    names=('name1', 'name2'))
     dat = reader.read(table)
-    assert_equal(reader.header.colnames, ('name1', 'name2'))
+    assert_equal(dat.colnames, ['name1', 'name2'])
     assert_almost_equal(dat[1][0], 2.4)
 
 
@@ -58,7 +57,7 @@ def test_read_normal_names_include():
                                    names=('name1', 'name2', 'name3'),
                                    include_names=('name1', 'name3'))
     dat = reader.read(table)
-    assert_equal(reader.header.colnames, ('name1', 'name3'))
+    assert_equal(dat.colnames, ['name1', 'name3'])
     assert_almost_equal(dat[1][0], 2.4)
     assert_equal(dat[0][1], 3)
 
@@ -74,7 +73,7 @@ def test_read_normal_exclude():
     reader = asciitable.get_reader(Reader=asciitable.FixedWidth,
                                    exclude_names=('Col1',))
     dat = reader.read(table)
-    assert_equal(reader.header.colnames, ('Col2',))
+    assert_equal(dat.colnames, ['Col2'])
     assert_equal(dat[1][0], "'s worlds")
 
 
@@ -87,7 +86,7 @@ def test_read_weird():
 """
     reader = asciitable.get_reader(Reader=asciitable.FixedWidth)
     dat = reader.read(table)
-    assert_equal(reader.header.colnames, ('Col1', 'Col2'))
+    assert_equal(dat.colnames, ['Col1', 'Col2'])
     assert_almost_equal(dat[1][0], 2.4)
     assert_equal(dat[0][1], '"hel')
     assert_equal(dat[1][1], "df's wo")
@@ -144,8 +143,8 @@ def test_read_no_header_names():
     and third rows also have hanging spaces after final |."""
     table = """
 |  John  | 555-1234 |192.168.1.10|
-|  Mary  | 555-2134 |192.168.1.12|  
-|   Bob  | 555-4527 | 192.168.1.9|  
+|  Mary  | 555-2134 |192.168.1.12|
+|   Bob  | 555-4527 | 192.168.1.9|
 """
     dat = asciitable.read(table, Reader=asciitable.FixedWidth, guess=False,
                           header_start=None, data_start=0,
@@ -175,8 +174,8 @@ def test_read_no_header_names_NoHeader():
     and third rows also have hanging spaces after final |."""
     table = """
 |  John  | 555-1234 |192.168.1.10|
-|  Mary  | 555-2134 |192.168.1.12|  
-|   Bob  | 555-4527 | 192.168.1.9|  
+|  Mary  | 555-2134 |192.168.1.12|
+|   Bob  | 555-4527 | 192.168.1.9|
 """
     dat = asciitable.read(table, Reader=asciitable.FixedWidthNoHeader,
                           names=('Name', 'Phone', 'TCP'))
@@ -217,7 +216,7 @@ dat = asciitable.read(table, Reader=asciitable.FixedWidth)
 
 def test_write_normal():
     """Write a table as a normal fixed width table."""
-    out = io.StringIO()
+    out = StringIO()
     asciitable.write(dat, out, Writer=asciitable.FixedWidth)
     assert_equal_splitlines(out.getvalue(), """\
 | Col1 |      Col2 | Col3 | Col4 |
@@ -226,9 +225,21 @@ def test_write_normal():
 """)
 
 
+def test_write_fill_values():
+    """Write a table as a normal fixed width table."""
+    out = StringIO()
+    asciitable.write(dat, out, Writer=asciitable.FixedWidth,
+                     fill_values=('a', 'N/A'))
+    assert_equal_splitlines(out.getvalue(), """\
+| Col1 |      Col2 | Col3 | Col4 |
+|  1.2 |   "hello" |    1 |  N/A |
+|  2.4 | 's worlds |    2 |    2 |
+""")
+
+
 def test_write_no_pad():
     """Write a table as a fixed width table with no padding."""
-    out = io.StringIO()
+    out = StringIO()
     asciitable.write(dat, out, Writer=asciitable.FixedWidth,
                      delimiter_pad=None)
     assert_equal_splitlines(out.getvalue(), """\
@@ -240,7 +251,7 @@ def test_write_no_pad():
 
 def test_write_no_bookend():
     """Write a table as a fixed width table with no bookend."""
-    out = io.StringIO()
+    out = StringIO()
     asciitable.write(dat, out, Writer=asciitable.FixedWidth, bookend=False)
     assert_equal_splitlines(out.getvalue(), """\
 Col1 |      Col2 | Col3 | Col4
@@ -251,7 +262,7 @@ Col1 |      Col2 | Col3 | Col4
 
 def test_write_no_delimiter():
     """Write a table as a fixed width table with no delimiter."""
-    out = io.StringIO()
+    out = StringIO()
     asciitable.write(dat, out, Writer=asciitable.FixedWidth, bookend=False,
                      delimiter=None)
     assert_equal_splitlines(out.getvalue(), """\
@@ -263,7 +274,7 @@ Col1       Col2  Col3  Col4
 
 def test_write_noheader_normal():
     """Write a table as a normal fixed width table."""
-    out = io.StringIO()
+    out = StringIO()
     asciitable.write(dat, out, Writer=asciitable.FixedWidthNoHeader)
     assert_equal_splitlines(out.getvalue(), """\
 | 1.2 |   "hello" | 1 | a |
@@ -273,7 +284,7 @@ def test_write_noheader_normal():
 
 def test_write_noheader_no_pad():
     """Write a table as a fixed width table with no padding."""
-    out = io.StringIO()
+    out = StringIO()
     asciitable.write(dat, out, Writer=asciitable.FixedWidthNoHeader,
                      delimiter_pad=None)
     assert_equal_splitlines(out.getvalue(), """\
@@ -284,7 +295,7 @@ def test_write_noheader_no_pad():
 
 def test_write_noheader_no_bookend():
     """Write a table as a fixed width table with no bookend."""
-    out = io.StringIO()
+    out = StringIO()
     asciitable.write(dat, out, Writer=asciitable.FixedWidthNoHeader,
                      bookend=False)
     assert_equal_splitlines(out.getvalue(), """\
@@ -295,7 +306,7 @@ def test_write_noheader_no_bookend():
 
 def test_write_noheader_no_delimiter():
     """Write a table as a fixed width table with no delimiter."""
-    out = io.StringIO()
+    out = StringIO()
     asciitable.write(dat, out, Writer=asciitable.FixedWidthNoHeader, bookend=False,
                      delimiter=None)
     assert_equal_splitlines(out.getvalue(), """\
@@ -306,7 +317,7 @@ def test_write_noheader_no_delimiter():
 
 def test_write_formats():
     """Write a table as a fixed width table with no delimiter."""
-    out = io.StringIO()
+    out = StringIO()
     asciitable.write(dat, out, Writer=asciitable.FixedWidth,
                      formats={'Col1': '%-8.3f', 'Col2': '%-15s'})
     assert_equal_splitlines(out.getvalue(), """\
@@ -320,9 +331,9 @@ def test_read_twoline_normal():
     """Typical fixed format table with two header lines (with some cruft
     thrown in to test column positioning"""
     table = """
-  Col1    Col2 
-  ----  --------- 
-   1.2xx"hello" 
+  Col1    Col2
+  ----  ---------
+   1.2xx"hello"
   2.4   's worlds
 """
     dat = asciitable.read(table, Reader=asciitable.FixedWidthTwoLine)
@@ -336,9 +347,9 @@ def test_read_twoline_ReST():
     """Read restructured text table"""
     table = """
 ======= ===========
-  Col1    Col2 
+  Col1    Col2
 ======= ===========
-  1.2   "hello" 
+  1.2   "hello"
   2.4   's worlds
 ======= ===========
 """
@@ -373,7 +384,7 @@ def test_read_twoline_human():
 
 def test_write_twoline_normal():
     """Write a table as a normal fixed width table."""
-    out = io.StringIO()
+    out = StringIO()
     asciitable.write(dat, out, Writer=asciitable.FixedWidthTwoLine)
     assert_equal_splitlines(out.getvalue(), """\
 Col1      Col2 Col3 Col4
@@ -385,7 +396,7 @@ Col1      Col2 Col3 Col4
 
 def test_write_twoline_no_pad():
     """Write a table as a fixed width table with no padding."""
-    out = io.StringIO()
+    out = StringIO()
     asciitable.write(dat, out, Writer=asciitable.FixedWidthTwoLine,
                      delimiter_pad=' ', position_char='=')
     assert_equal_splitlines(out.getvalue(), """\
@@ -398,7 +409,7 @@ Col1        Col2   Col3   Col4
 
 def test_write_twoline_no_bookend():
     """Write a table as a fixed width table with no bookend."""
-    out = io.StringIO()
+    out = StringIO()
     asciitable.write(dat, out, Writer=asciitable.FixedWidthTwoLine,
                      bookend=True, delimiter='|')
     assert_equal_splitlines(out.getvalue(), """\
