@@ -2,8 +2,6 @@
 
 import copy
 import re
-import string
-import sys
 import warnings
 
 import numpy as np
@@ -12,12 +10,13 @@ from .util import _str_to_num, _is_int, maketrans, translate, _words_group
 from .verify import _Verify, _ErrList, VerifyError, VerifyWarning
 
 from . import ENABLE_RECORD_VALUED_KEYWORD_CARDS, STRIP_HEADER_WHITESPACE
+
 from ...utils import deprecated
+from ...utils.exceptions import AstropyUserWarning, AstropyDeprecationWarning
 
 
 __all__ = ['Card', 'CardList', 'create_card', 'create_card_from_string',
-           'upper_key', 'createCard', 'createCardFromString', 'upperKey',
-           'Undefined']
+           'upper_key', 'Undefined']
 
 
 FIX_FP_TABLE = maketrans('de', 'DE')
@@ -69,7 +68,7 @@ class CardList(list):
                 'functionality has been subsumed by the Header class, so '
                 'CardList objects should not be directly created.  See the '
                 'PyFITS 3.1.0 CHANGELOG for more details.',
-                DeprecationWarning)
+                AstropyDeprecationWarning)
 
         # This is necessary for now to prevent a circular import
         from .header import Header
@@ -288,11 +287,6 @@ class CardList(list):
 
         return CardList(self._header[key])
 
-    # For API backwards-compatibility
-    @deprecated('3.0', alternative=':meth:`filter_list`', pending=False)
-    def filterList(self, key):
-        return self.filter_list(key)
-
     @deprecated('3.1', pending=False)
     def count_blanks(self):
         """
@@ -784,9 +778,11 @@ class Card(_Verify):
         Examples
         --------
 
-        >>> self._check_if_rvkc('DP1', 'AXIS.1: 2')
-        >>> self._check_if_rvkc('DP1.AXIS.1', 2)
-        >>> self._check_if_rvkc('DP1     = AXIS.1: 2')
+        ::
+
+            self._check_if_rvkc('DP1', 'AXIS.1: 2')
+            self._check_if_rvkc('DP1.AXIS.1', 2)
+            self._check_if_rvkc('DP1     = AXIS.1: 2')
         """
 
         if not ENABLE_RECORD_VALUED_KEYWORD_CARDS():
@@ -866,7 +862,7 @@ class Card(_Verify):
         else:
             warnings.warn('The following header keyword is invalid or follows '
                           'an unrecognized non-standard convention:\n%s' %
-                          self._image)
+                          self._image, AstropyUserWarning)
             self._invalid = True
             return keyword
 
@@ -1287,34 +1283,27 @@ def create_card(key='', value='', comment=''):
     return Card(key, value, comment)
 create_card.__doc__ = Card.__init__.__doc__
 # For API backwards-compatibility
-createCard = deprecated('3.0', name='createCard',
-                        alternative=':meth:`Card.__init__`')(create_card)
 create_card = deprecated('3.1', name='create_card',
                          alternative=':meth:`Card.__init__`',
-                         pending=True)(create_card)
+                         pending=False)(create_card)
 
 
 def create_card_from_string(input):
     return Card.fromstring(input)
 create_card_from_string.__doc__ = Card.fromstring.__doc__
 # For API backwards-compat
-createCardFromString = \
-    deprecated('3.0', name='createCardFromString',
-               alternative=':meth:`Card.fromstring`')(create_card_from_string)
 create_card_from_string = deprecated('3.1', name='create_card_from_string',
                                      alternative=':meth:`Card.fromstring`',
-                                     pending=True)(create_card_from_string)
+                                     pending=False)(create_card_from_string)
 
 
 def upper_key(key):
     return Card.normalize_keyword(key)
 upper_key.__doc__ = Card.normalize_keyword.__doc__
 # For API backwards-compat
-upperKey = deprecated('3.0', name='upperKey',
-                      alternative=':meth:`Card.normalize_keyword`')(upper_key)
 upper_key = deprecated('3.1', name='upper_key',
                        alternative=':meth:`Card.normalize_keyword`',
-                       pending=True)(upper_key)
+                       pending=False)(upper_key)
 
 
 def _int_or_float(s):
