@@ -1,12 +1,10 @@
 # Licensed under a 3-clause BSD style license - see PYFITS.rst
 
-from __future__ import division
-from __future__ import with_statement
+from __future__ import division, with_statement
 
 import gzip
 import mmap
 import os
-import sys
 import tempfile
 import urllib
 import warnings
@@ -18,12 +16,38 @@ from numpy import memmap as Memmap
 from .util import (isreadable, iswritable, isfile, fileobj_open, fileobj_name,
                    fileobj_closed, fileobj_mode, _array_from_file,
                    _array_to_file, _write_string, b)
-from ...utils import deprecated
+from ...utils.exceptions import AstropyUserWarning
 
 
-# File object open modes
-PYTHON_MODES = {'readonly': 'rb', 'copyonwrite': 'rb', 'update': 'rb+',
-                'append': 'ab+', 'ostream': 'wb', 'denywrite': 'rb'}
+# Maps PyFITS-specific file mode names to the appropriate file modes to use
+# for the underlying raw files
+# TODO: This should probably renamed IO_FITS_MODES or something, but since it's
+# used primarily internally I'm going to leave PYFITS in the name for now for
+# in the off chance any third-party software is trying to do anything with this
+# object.
+PYFITS_MODES = {
+    'readonly': 'rb',
+    'copyonwrite': 'rb',
+    'update': 'rb+',
+    'append': 'ab+',
+    'ostream': 'wb',
+    'denywrite': 'rb'}
+
+# This is the old name of the PYFITS_MODES dict; it is maintained here for
+# backwards compatibility and should be removed no sooner than PyFITS 3.4
+PYTHON_MODES = PYFITS_MODES
+
+# Maps OS-level file modes to the appropriate PyFITS specific mode to use
+# when given file objects but no mode specified; obviously in PYFITS_MODES
+# there are overlaps; for example 'readonly' and 'denywrite' both require
+# the file to be opened in 'rb' mode.  But 'readonly' is the default
+# behavior for such files if not otherwise specified.
+# Note: 'ab' is only supported for 'ostream' which is output-only.
+FILE_MODES = {
+    'rb': 'readonly', 'rb+': 'update',
+    'wb': 'ostream', 'wb+': 'update',
+    'ab': 'ostream', 'ab+': 'append'}
+
 
 # readonly actually uses copyonwrite for mmap so that readonly without mmap and
 # with mmap still have to same behavior with regard to updating the array.  To
@@ -52,10 +76,11 @@ class _File(object):
     # See self._test_mmap
     _mmap_available = None
 
-    def __init__(self, fileobj=None, mode='readonly', memmap=False):
+    def __init__(self, fileobj=None, mode=None, memmap=False, clobber=False):
         if fileobj is None:
             self.__file = None
             self.closed = False
+            self.binary = True
             self.mode = mode
             self.memmap = memmap
             self.compression = None
@@ -66,20 +91,35 @@ class _File(object):
         else:
             self.simulateonly = False
 
-        if mode not in PYTHON_MODES:
+        if mode is None:
+            if _is_random_access_file_backed(fileobj):
+                fmode = fileobj_mode(fileobj)
+                # If the mode is unsupported just leave it as None; we'll
+                # catch this case below
+                mode = FILE_MODES.get(fmode)
+            else:
+                mode = 'readonly'  # The default
+
+        if mode not in PYFITS_MODES:
             raise ValueError("Mode '%s' not recognized" % mode)
 
-        if (isinstance(fileobj, basestring) and mode != 'append' and
-            not os.path.exists(fileobj) and
-            not os.path.splitdrive(fileobj)[0]):
-                #
-                # Not writing file and file does not exist on local machine and
-                # name does not begin with a drive letter (Windows), try to
-                # get it over the web.
-                #
+        if (isinstance(fileobj, basestring) and
+                mode not in ('ostream', 'append') and
+                not os.path.exists(fileobj)):
+
+            # Not writing file and file does not exist on local machine and
+            # name does not begin with a drive letter (Windows), try to get it
+            # over the web.
             try:
-                self.name, _ = urllib.urlretrieve(fileobj)
-            except (TypeError, ValueError):
+                if not os.path.splitdrive(fileobj)[0]:
+                    # Basically if the filename (on Windows anyways) doesn't
+                    # have a drive letter try to open it as a URL
+                    self.name, _ = urllib.urlretrieve(fileobj)
+                else:
+                    # Otherwise the file was already not found so just raise
+                    # a ValueError
+                    raise ValueError("File not found")
+            except (TypeError, ValueError, IOError):
                 # A couple different exceptions can occur here when passing a
                 # filename into urlretrieve in Python 3
                 raise IOError('File does not exist: %r' % fileobj)
@@ -87,6 +127,7 @@ class _File(object):
             self.name = fileobj_name(fileobj)
 
         self.closed = False
+        self.binary = True
         self.mode = mode
         self.memmap = memmap
 
@@ -99,12 +140,12 @@ class _File(object):
         self.writeonly = False
 
         # Initialize the internal self.__file object
-        if isfile(fileobj) or isinstance(fileobj, gzip.GzipFile):
-            self._open_fileobj(fileobj, mode)
+        if _is_random_access_file_backed(fileobj):
+            self._open_fileobj(fileobj, mode, clobber)
         elif isinstance(fileobj, basestring):
-            self._open_filename(fileobj, mode)
+            self._open_filename(fileobj, mode, clobber)
         else:
-            self._open_filelike(fileobj, mode)
+            self._open_filelike(fileobj, mode, clobber)
 
         if isinstance(fileobj, gzip.GzipFile):
             self.compression = 'gzip'
@@ -151,12 +192,6 @@ class _File(object):
     def __exit__(self, type, value, traceback):
         self.close()
 
-    @deprecated('3.0', message='This method should not be treated as public.')
-    def getfile(self):
-        """Will be going away as soon as I figure out how."""
-
-        return self.__file
-
     def readable(self):
         if self.writeonly:
             return False
@@ -165,7 +200,15 @@ class _File(object):
     def read(self, size=None):
         if not hasattr(self.__file, 'read'):
             raise EOFError
-        return self.__file.read(size)
+        try:
+            return self.__file.read(size)
+        except IOError:
+            # On some versions of Python, it appears, GzipFile will raise an
+            # IOError if you try to read past its end (as opposed to just
+            # returning '')
+            if self.compression == 'gzip':
+                return ''
+            raise
 
     def readarray(self, size=None, offset=0, dtype=np.uint8, shape=None):
         """
@@ -203,7 +246,7 @@ class _File(object):
 
         if not (size or shape):
             warnings.warn('No size or shape given to readarray(); assuming a '
-                          'shape of (1,)')
+                          'shape of (1,)', AstropyUserWarning)
             shape = (1,)
 
         if self.memmap:
@@ -263,7 +306,7 @@ class _File(object):
         if self.size and pos > self.size:
             warnings.warn('File may have been truncated: actual file length '
                           '(%i) is smaller than the expected size (%i)' %
-                          (self.size, pos))
+                          (self.size, pos), AstropyUserWarning)
 
     def tell(self):
         if not hasattr(self.__file, 'tell'):
@@ -284,31 +327,65 @@ class _File(object):
 
         self.closed = True
 
-    def _open_fileobj(self, fileobj, mode):
+    def _overwrite_existing(self, clobber, fileobj, closed):
+        """Overwrite an existing file if ``clobber`` is ``True``, otherwise
+        raise an IOError.  The exact behavior of this method depends on the
+        _File object state and is only meant for use within the ``_open_*``
+        internal methods.
+        """
+
+        # The file will be overwritten...
+        if ((self.file_like and
+                (hasattr(fileobj, 'len') and fileobj.len > 0)) or
+                (os.path.exists(self.name) and
+                 os.path.getsize(self.name) != 0)):
+            if clobber:
+                warnings.warn("Overwriting existing file %r." % self.name,
+                              AstropyUserWarning)
+                if self.file_like and hasattr(fileobj, 'truncate'):
+                    fileobj.truncate(0)
+                else:
+                    if not closed:
+                        fileobj.close()
+                    os.remove(self.name)
+            else:
+                raise IOError("File %r already exists." % self.name)
+
+    def _open_fileobj(self, fileobj, mode, clobber):
         """Open a FITS file from a file object or a GzipFile object."""
 
         closed = fileobj_closed(fileobj)
-        fmode = fileobj_mode(fileobj) or PYTHON_MODES[mode]
+        fmode = fileobj_mode(fileobj) or PYFITS_MODES[mode]
+
+        if mode == 'ostream':
+            self._overwrite_existing(clobber, fileobj, closed)
 
         if not closed:
-            # In some cases (like on Python 3) a file opened for appending
-            # still shows a mode of 'r+', hence the extra check for the append
-            # case
-            if ((mode == 'append' and fmode not in ('ab+', 'rb+')) or
-                (mode != 'append' and PYTHON_MODES[mode] != fmode)):
+            # Although we have a specific mapping in PYFITS_MODES from our
+            # custom file modes to raw file object modes, many of the latter
+            # can be used appropriately for the former.  So determine whether
+            # the modes match up appropriately
+            if ((mode in ('readonly', 'denywrite', 'copyonwrite') and
+                    not ('r' in fmode or '+' in fmode)) or
+                    (mode == 'append' and fmode not in ('ab+', 'rb+')) or
+                    (mode == 'ostream' and
+                     not ('w' in fmode or 'a' in fmode or '+' in fmode)) or
+                    (mode == 'update' and fmode not in ('rb+', 'wb+'))):
                 raise ValueError(
-                    "Input mode '%s' (%s) does not match mode of the "
-                    "input file (%s)." % (mode, PYTHON_MODES[mode], fmode))
+                    "Mode argument '%s' does not match mode of the input "
+                    "file (%s)." % (mode, fmode))
             self.__file = fileobj
         elif isfile(fileobj):
-            self.__file = fileobj_open(self.name, PYTHON_MODES[mode])
+            self.__file = fileobj_open(self.name, PYFITS_MODES[mode])
+        else:
+            self.__file = gzip.open(self.name, PYFITS_MODES[mode])
+
+        if fmode == 'ab+':
             # Return to the beginning of the file--in Python 3 when opening in
             # append mode the file pointer is at the end of the file
             self.__file.seek(0)
-        else:
-            self.__file = gzip.open(self.name, PYTHON_MODES[mode])
 
-    def _open_filelike(self, fileobj, mode):
+    def _open_filelike(self, fileobj, mode, clobber):
         """Open a FITS file from a file-like object, i.e. one that has
         read and/or write methods.
         """
@@ -316,54 +393,62 @@ class _File(object):
         self.file_like = True
         self.__file = fileobj
 
+        if fileobj_closed(fileobj):
+            raise IOError("Cannot read from/write to a closed file-like "
+                          "object (%r)." % fileobj)
+
+        if isinstance(fileobj, zipfile.ZipFile):
+            self._open_zipfile(fileobj, mode)
+            self.__file.seek(0)
+            # We can bypass any additional checks at this point since now
+            # self.__file points to the temp file extracted from the zip
+            return
+
         # If there is not seek or tell methods then set the mode to
         # output streaming.
         if (not hasattr(self.__file, 'seek') or
             not hasattr(self.__file, 'tell')):
             self.mode = mode = 'ostream'
 
-        if (self.mode in ('copyonwrite', 'update', 'append') and
+        if mode == 'ostream':
+            self._overwrite_existing(clobber, fileobj, False)
+
+        # Any "writeable" mode requires a write() method on the file object
+        if (self.mode in ('update', 'append', 'ostream') and
             not hasattr(self.__file, 'write')):
             raise IOError("File-like object does not have a 'write' "
                           "method, required for mode '%s'."
                           % self.mode)
 
-        if (self.mode in ('readonly', 'denywrite') and
-            not hasattr(self.__file, 'read')):
+        # Any mode except for 'ostream' requires readability
+        if self.mode != 'ostream' and not hasattr(self.__file, 'read'):
             raise IOError("File-like object does not have a 'read' "
                           "method, required for mode %r."
                           % self.mode)
 
-    def _open_filename(self, filename, mode):
+    def _open_filename(self, filename, mode, clobber):
         """Open a FITS file from a filename string."""
 
+        if mode == 'ostream':
+            self._overwrite_existing(clobber, None, True)
+
         if os.path.exists(self.name):
             with fileobj_open(self.name, 'rb') as f:
                 magic = f.read(4)
         else:
             magic = b('')
+
         ext = os.path.splitext(self.name)[1]
+
         if ext == '.gz' or magic.startswith(GZIP_MAGIC):
             # Handle gzip files
-            self.__file = gzip.open(self.name, PYTHON_MODES[mode])
+            self.__file = gzip.open(self.name, PYFITS_MODES[mode])
             self.compression = 'gzip'
         elif ext == '.zip' or magic.startswith(PKZIP_MAGIC):
             # Handle zip files
-            if mode in ['update', 'append']:
-                raise IOError(
-                      "Writing to zipped fits files is not currently "
-                      "supported")
-            zfile = zipfile.ZipFile(self.name)
-            namelist = zfile.namelist()
-            if len(namelist) != 1:
-                raise IOError(
-                  "Zip files with multiple members are not supported.")
-            self.__file = tempfile.NamedTemporaryFile(suffix='.fits')
-            self.__file.write(zfile.read(namelist[0]))
-            zfile.close()
-            self.compression = 'zip'
+            self._open_zipfile(self.name, mode)
         else:
-            self.__file = fileobj_open(self.name, PYTHON_MODES[mode])
+            self.__file = fileobj_open(self.name, PYFITS_MODES[mode])
             # Make certain we're back at the beginning of the file
         self.__file.seek(0)
 
@@ -388,14 +473,15 @@ class _File(object):
                 mm = mmap.mmap(tmpfd, 1, access=mmap.ACCESS_WRITE)
             except mmap.error as e:
                 warnings.warn('Failed to create mmap: %s; mmap use will be '
-                              'disabled' % str(e))
+                              'disabled' % str(e), AstropyUserWarning)
                 _File._mmap_available = False
                 return False
             try:
                 mm.flush()
             except mmap.error:
                 warnings.warn('mmap.flush is unavailable on this platform; '
-                              'using mmap in writeable mode will be disabled')
+                              'using mmap in writeable mode will be disabled',
+                              AstropyUserWarning)
                 _File._mmap_available = False
                 return False
             finally:
@@ -406,3 +492,44 @@ class _File(object):
 
         _File._mmap_available = True
         return True
+
+    def _open_zipfile(self, fileobj, mode):
+        """Limited support for zipfile.ZipFile objects containing a single
+        a file.  Allows reading only for now by extracting the file to a
+        tempfile.
+        """
+
+        if mode in ('update', 'append'):
+            raise IOError(
+                  "Writing to zipped fits files is not currently "
+                  "supported")
+
+        if not isinstance(fileobj, zipfile.ZipFile):
+            zfile = zipfile.ZipFile(fileobj)
+            close = True
+        else:
+            zfile = fileobj
+            close = False
+
+        namelist = zfile.namelist()
+        if len(namelist) != 1:
+            raise IOError(
+              "Zip files with multiple members are not supported.")
+        self.__file = tempfile.NamedTemporaryFile(suffix='.fits')
+        self.__file.write(zfile.read(namelist[0]))
+
+        if close:
+            zfile.close()
+        self.compression = 'zip'
+
+
+def _is_random_access_file_backed(fileobj):
+    """Returns `True` if fileobj is a `file` or `io.FileIO` object or a
+    `gzip.GzipFile` object.
+
+    Although reading from a zip file is supported, this does not include
+    support for random access, and we do not yet support reading directly
+    from an already opened `zipfile.ZipFile` object.
+    """
+
+    return isfile(fileobj) or isinstance(fileobj, gzip.GzipFile)
