@@ -7,7 +7,7 @@ from .image import _ImageBaseHDU, PrimaryHDU
 from .table import _TableLikeHDU
 from ..column import Column, ColDefs, FITS2NUMPY
 from ..fitsrec import FITS_rec, FITS_record
-from ..util import _is_int, _is_pseudo_unsigned
+from ..util import _is_int, _is_pseudo_unsigned, _unsigned_zero
 
 from ....utils import lazyproperty
 
@@ -93,7 +93,7 @@ class GroupData(FITS_rec):
 
     _record_type = Group
 
-    def __new__(subtype, input=None, bitpix=None, pardata=None, parnames=[],
+    def __new__(cls, input=None, bitpix=None, pardata=None, parnames=[],
                 bscale=None, bzero=None, parbscales=None, parbzeros=None):
         """
         Parameters
@@ -164,7 +164,7 @@ class GroupData(FITS_rec):
 
             coldefs = ColDefs(cols)
 
-            self = FITS_rec.__new__(subtype,
+            self = FITS_rec.__new__(cls,
                                     np.rec.array(None,
                                                  formats=formats,
                                                  names=coldefs.names,
@@ -185,7 +185,7 @@ class GroupData(FITS_rec):
             else:
                 np.rec.recarray.field(self, npars)[:] = input
         else:
-            self = FITS_rec.__new__(subtype, input)
+            self = FITS_rec.__new__(cls, input)
             self.parnames = None
         return self
 
@@ -240,7 +240,7 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
     _width2format = {8: 'B', 16: 'I', 32: 'J', 64: 'K', -32: 'E', -64: 'D'}
     _data_type = GroupData
 
-    def __init__(self, data=None, header=None, name=None):
+    def __init__(self, data=None, header=None):
         """
         TODO: Write me
         """
@@ -289,7 +289,7 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
 
     @lazyproperty
     def columns(self):
-        if self._data_loaded and hasattr(self.data, '_coldefs'):
+        if self._has_data and hasattr(self.data, '_coldefs'):
             return self.data._coldefs
 
         format = self._width2format[self._header['BITPIX']]
@@ -355,29 +355,29 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
 
         if self._data_loaded:
             if isinstance(self.data, GroupData):
-                field0 = self.data.dtype.names[0]
-                field0_code = self.data.dtype.fields[field0][0].name
-                self._header['BITPIX'] = _ImageBaseHDU.ImgCode[field0_code]
                 self._axes = list(self.data.data.shape)[1:]
                 self._axes.reverse()
                 self._axes = [0] + self._axes
+                field0 = self.data.dtype.names[0]
+                field0_code = self.data.dtype.fields[field0][0].name
             elif self.data is None:
-                self._axes = []
+                self._axes = [0]
+                field0_code = 'uint8'  # For lack of a better default
             else:
                 raise ValueError('incorrect array type')
 
+            self._header['BITPIX'] = _ImageBaseHDU.ImgCode[field0_code]
+
         self._header['NAXIS'] = len(self._axes)
 
         # add NAXISi if it does not exist
         for idx, axis in enumerate(self._axes):
-            try:
-                self._header['NAXIS' + str(idx + 1)] = axis
-            except KeyError:
-                if (idx == 0):
-                    after = 'NAXIS'
-                else:
-                    after = 'NAXIS' + str(idx)
-                self._header.set('NAXIS' + str(idx + 1), axis, after=after)
+            if (idx == 0):
+                after = 'NAXIS'
+            else:
+                after = 'NAXIS' + str(idx)
+
+            self._header.set('NAXIS' + str(idx + 1), axis, after=after)
 
         # delete extra NAXISi's
         for idx in range(len(self._axes) + 1, old_naxis + 1):
@@ -386,7 +386,7 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
             except KeyError:
                 pass
 
-        if self._data_loaded and isinstance(self.data, GroupData):
+        if self._has_data and isinstance(self.data, GroupData):
             self._header.set('GROUPS', True,
                              after='NAXIS' + str(len(self._axes)))
             self._header.set('PCOUNT', len(self.data.parnames), after='GROUPS')
@@ -494,7 +494,7 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
         Calculate the value for the ``DATASUM`` card in the HDU.
         """
 
-        if self._data_loaded and self.data is not None:
+        if self._has_data:
             # We have the data to be used.
             # Check the byte order of the data.  If it is little endian we
             # must swap it before calculating the datasum.
@@ -534,10 +534,9 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
         if shape:
             shape = shape[1:]
 
-        if self._data_loaded:
-            # Update the format
-            format = self.data.dtype.fields[self.data.dtype.names[0]][0].name
-            format = format[format.rfind('.') + 1:]
+            if shape and all(shape):
+                # Update the format
+                format = self.columns[0].dtype.name
 
         # Update the GCOUNT report
         gcount = '%d Groups  %d Parameters' % (self._gcount, self._pcount)
