@@ -6,17 +6,17 @@ import collections
 import copy
 import inspect
 import itertools
-import os
 import re
 import sys
 import warnings
 
 from .card import Card, CardList, _pad, BLANK_CARD, KEYWORD_LENGTH
-from .file import _File, PYTHON_MODES
-from .util import (encode_ascii, decode_ascii, fileobj_mode,
+from .file import _File
+from .util import (encode_ascii, decode_ascii, fileobj_mode, fileobj_closed,
                    fileobj_is_binary)
 
 from ...utils import deprecated, isiterable
+from ...utils.exceptions import AstropyUserWarning, AstropyDeprecationWarning
 
 
 PY3K = sys.version_info[:2] >= (3, 0)
@@ -32,6 +32,9 @@ HEADER_END_RE = re.compile(encode_ascii('END {77} *'))
 VALID_HEADER_CHARS = set(chr(x) for x in range(0x20, 0x7F))
 
 
+__doctest_skip__ = ['Header', 'Header.*']
+
+
 class Header(object):
     """
     FITS header class.  This class exposes both a dict-like interface and a
@@ -88,7 +91,7 @@ class Header(object):
             warnings.warn(
                 'The txtfile argument is deprecated.  Use Header.fromfile to '
                 'create a new Header object from a text file.',
-                DeprecationWarning)
+                AstropyDeprecationWarning)
             # get the cards from the input ASCII file
             self.update(self.fromfile(txtfile))
             self._modified = False
@@ -215,14 +218,14 @@ class Header(object):
                     raise KeyError("Keyword '%s' not found." % key)
                 # Warn everyone else.
                 # TODO: Remove this warning and make KeyError the default after
-                # a couple versions (by 3.2 or 3.3, say)
+                # a couple versions (by 3.3, say)
                 warnings.warn(
-                    'Deletetion of non-existent keyword %r: '
+                    'Deletion of non-existent keyword %r: '
                     'In a future Astropy version Header.__delitem__ may be '
                     'changed so that this raises a KeyError just like a dict '
                     'would. Please update your code so that KeyErrors are '
                     'caught and handled when deleting non-existent keywords.' %
-                    key, DeprecationWarning)
+                    key, AstropyDeprecationWarning)
                 return
             for idx in reversed(self._keyword_indices[key]):
                 # Have to copy the indices list since it will be modified below
@@ -475,7 +478,7 @@ class Header(object):
                 if is_eof and blocks.strip('\0') == '':
                     warnings.warn('Unexpected extra padding at the end of the '
                                   'file.  This padding may not be preserved '
-                                  'when saving changes.')
+                                  'when saving changes.', AstropyUserWarning)
                     raise EOFError()
                 else:
                     # Replace the illegal null bytes with spaces as required by
@@ -483,7 +486,7 @@ class Header(object):
                     warnings.warn('Header block contains null bytes instead '
                                   'of spaces for padding, and is not FITS-'
                                   'compliant. Nulls may be replaced with '
-                                  'spaces upon writing.')
+                                  'spaces upon writing.', AstropyUserWarning)
                     blocks.replace('\0', ' ')
 
             if not HEADER_END_RE.search(last_block) and endcard:
@@ -526,7 +529,7 @@ class Header(object):
 
         Returns
         -------
-        s : string
+        s : str
             A string representing a FITS header.
         """
 
@@ -578,34 +581,10 @@ class Header(object):
             If `True`, overwrites the output file if it already exists
         """
 
-        close_file = False
-
-        # check if the output file already exists
-        # TODO: Perhaps this sort of thing could be handled by the _File
-        # initializer...
-        if isinstance(fileobj, basestring):
-            if os.path.exists(fileobj) and os.path.getsize(fileobj) != 0:
-                if clobber:
-                    warnings.warn("Overwriting existing file '%s'." % fileobj)
-                    os.remove(fileobj)
-                else:
-                    raise IOError("File '%s' already exists." % fileobj)
-
-            fileobj = open(fileobj, 'wb')
-            close_file = True
+        close_file = fileobj_closed(fileobj)
 
         if not isinstance(fileobj, _File):
-            # TODO: There needs to be a way of handling this built into the
-            # _File class.  I think maybe there used to be, but I took it out;
-            # now the design is such that it would be better for it to go back
-            # in
-            mode = 'append'
-            fmode = fileobj_mode(fileobj) or 'ab+'
-            for key, val in PYTHON_MODES.iteritems():
-                if val == fmode:
-                    mode = key
-                    break
-            fileobj = _File(fileobj, mode=mode)
+            fileobj = _File(fileobj, mode='ostream', clobber=clobber)
 
         try:
             blocks = self.tostring(sep=sep, endcard=endcard, padding=padding)
@@ -816,12 +795,6 @@ class Header(object):
         else:
             self[keyword] = (value, comment)
 
-    @deprecated('3.0', alternative='``key in header`` syntax')
-    def has_key(self, key):
-        """Like :meth:`dict.has_key`."""
-
-        return key in self
-
     def items(self):
         """Like :meth:`dict.items`."""
 
@@ -908,13 +881,13 @@ class Header(object):
         (keyword, value, comment) tuples.
 
         Arbitrary arguments are also accepted, in which case the update() is
-        called again with the kwargs dict as its only argument.  That is,
+        called again with the kwargs dict as its only argument.  That is, ::
 
-            >>> header.update(NAXIS1=100, NAXIS2=100)
+            header.update(NAXIS1=100, NAXIS2=100)
 
-        is equivalent to
+        is equivalent to ::
 
-            >>> header.update({'NAXIS1': 100, 'NAXIS2': 100})
+            header.update({'NAXIS1': 100, 'NAXIS2': 100})
 
         .. warning::
             As this method works similarly to dict.update() it is very
@@ -926,38 +899,38 @@ class Header(object):
             **deprecated**.  Most uses of the old API can be replaced as
             follows:
 
-            * Replace
+            * Replace ::
 
-                  >>> header.update(keyword, value)
+                  header.update(keyword, value)
 
-              with
+              with ::
 
-                  >>> header[keyword] = value
+                  header[keyword] = value
 
-            * Replace
+            * Replace ::
 
-                  >>> header.update(keyword, value, comment=comment)
+                  header.update(keyword, value, comment=comment)
 
-              with
+              with ::
 
-                  >>> header[keyword] = (value, comment)
+                  header[keyword] = (value, comment)
 
-            * Replace
+            * Replace ::
 
-                  >>> header.update(keyword, value, before=before_keyword)
+                  header.update(keyword, value, before=before_keyword)
 
-              with
+              with ::
 
-                  >>> header.insert(before_keyword, (keyword, value))
+                  header.insert(before_keyword, (keyword, value))
 
-            * Replace
+            * Replace ::
 
-                  >>> header.update(keyword, value, after=after_keyword)
+                  header.update(keyword, value, after=after_keyword)
 
-              with
+              with ::
 
-                  >>> header.insert(after_keyword, (keyword, value),
-                  ...               after=True)
+                  header.insert(after_keyword, (keyword, value),
+                                after=True)
 
             See also :meth:`Header.set` which is a new method that provides an
             interface similar to the old Header.update() and may help make
@@ -1022,11 +995,11 @@ class Header(object):
             # This must be a legacy update()
             warnings.warn(
                 "The use of header.update() to add new keywords to a header "
-                "deprecated.  Instead, use either header.set() or simply "
+                "is deprecated.  Instead, use either header.set() or simply "
                 "`header[keyword] = value` or "
                 "`header[keyword] = (value, comment)`.  header.set() is only "
                 "necessary to use if you also want to use the before/after "
-                "keyword arguments.", DeprecationWarning)
+                "keyword arguments.", AstropyDeprecationWarning)
 
             for k, v in zip(legacy_args, args):
                 if k in kwargs:
@@ -1398,7 +1371,7 @@ class Header(object):
             if keyword not in Card._commentary_keywords:
                 warnings.warn(
                     'A %r keyword already exists in this header.  Inserting '
-                    'duplicate keyword.' % keyword)
+                    'duplicate keyword.' % keyword, AstropyUserWarning)
             self._keyword_indices[keyword].sort()
 
         if useblanks:
@@ -1840,19 +1813,11 @@ class Header(object):
 
         return CardList(self)
 
-    @deprecated('3.0', alternative='the `.ascard` attribute')
-    def ascardlist(self):
-        """
-        Returns a `CardList` object.
-        """
-
-        return self.ascard
-
     @deprecated('3.1', alternative=':meth:`Header.rename_keyword`')
     def rename_key(self, oldkey, newkey, force=False):
         self.rename_keyword(oldkey, newkey, force)
 
-    @deprecated('3.1', alternative="``header['HISTORY']``", pending=True)
+    @deprecated('3.1', alternative="``header['HISTORY']``")
     def get_history(self):
         """
         Get all history cards as a list of string texts.
@@ -1863,7 +1828,7 @@ class Header(object):
         else:
             return []
 
-    @deprecated('3.1', alternative="``header['COMMENT']``", pending=True)
+    @deprecated('3.1', alternative="``header['COMMENT']``")
     def get_comment(self):
         """
         Get all comment cards as a list of string texts.
@@ -1948,7 +1913,7 @@ class Header(object):
                         not (card.keyword == '' and card.value == '')):
                     # Don't add duplicate commentary cards (though completely
                     # blank cards are allowed to be duplicated)
-                    for idx, c in enumerate(self.cards):
+                    for c in self.cards:
                         if c.keyword == card.keyword and c.value == card.value:
                             break
                     else:
