@@ -15,19 +15,15 @@
 /* This module contains three functions that are callable from python.  The  */
 /* first is compress_hdu.  This function takes an                            */
 /* astropy.io.fits.CompImageHDU object containing the uncompressed image     */
-/* data and returns the compressed data for all tiles into the .compData     */
-/* attribute of that HDU.                                                    */
+/* data and returns the compressed data for all tiles into the               */
+/* .compressed_data attribute of that HDU.                                   */
 /*                                                                           */
 /* The second function is decompress_hdu.  It takes an                       */
 /* astropy.io.fits.CompImageHDU object that already has compressed data in   */
-/* its .compData attribute. It returns the decompressed image data into the  */
-/* HDU's .data attribute.                                                    */
+/* its .compressed_data attribute.  It returns the decompressed image data   */
+/* into the HDU's .data attribute.                                           */
 /*                                                                           */
-/* Finally, the utility function calc_max_elm is used internally by the      */
-/* CompImageHDU class to estimate how much memory to allocate for the        */
-/* compressed data when compressing an image.                                */
-/*                                                                           */
-/* Copyright (C) 2012 Association of Universities for Research in Astronomy  */
+/* Copyright (C) 2013 Association of Universities for Research in Astronomy  */
 /* (AURA)                                                                    */
 /*                                                                           */
 /* Redistribution and use in source and binary forms, with or without        */
@@ -659,14 +655,45 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
                 (0 == strcmp(tmp, "NONE"))) {
             Fptr->quantize_level = NO_QUANTIZE;
         } else if (0 == strcmp(tmp, "SUBTRACTIVE_DITHER_1")) {
-            Fptr->quantize_dither = SUBTRACTIVE_DITHER_1;
+#ifdef CFITSIO_SUPPORTS_SUBTRACTIVE_DITHER_2
+            // Added in CFITSIO 3.35, this also changed the name of the
+            // quantize_dither struct member to quantize_method
+            Fptr->quantize_method = SUBTRACTIVE_DITHER_1;
+        } else if (0 == strcmp(tmp, "SUBTRACTIVE_DITHER_2")) {
+            Fptr->quantize_method = SUBTRACTIVE_DITHER_2;
         } else {
-            Fptr->quantize_dither = 0;
+            Fptr->quantize_method = NO_DITHER;
         }
     } else {
-        Fptr->quantize_dither = 0;
+        Fptr->quantize_method = NO_DITHER;
     }
 
+    if (Fptr->quantize_method != NO_DITHER) {
+        if (0 != get_header_int(header, "ZDITHER0", &(Fptr->dither_seed), 0)) {
+            // ZDITHER0 keyword not found
+            Fptr->dither_seed = 0;
+            Fptr->request_dither_seed = 0;
+        }
+    }
+#else
+            Fptr->quantize_dither = SUBTRACTIVE_DITHER_1;
+        } else {
+            Fptr->quantize_dither = NO_DITHER;
+        }
+    } else {
+        Fptr->quantize_dither = NO_DITHER;
+    }
+
+    if (Fptr->quantize_dither != NO_DITHER) {
+        if (0 != get_header_int(header, "ZDITHER0", &(Fptr->dither_offset),
+                                0)) {
+            // ZDITHER0 keyword no found
+            Fptr->dither_offset = 0;
+            Fptr->request_dither_offset = 0;
+        }
+    }
+#endif
+
     Fptr->compressimg = 1;
     Fptr->maxelem = imcomp_calc_max_elem(Fptr->compress_type,
                                          Fptr->maxtilelen,
@@ -680,7 +707,7 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
 void init_output_buffer(PyObject* hdu, void** buf, size_t* bufsize) {
     // Determines a good size for the output data buffer and allocates
     // memory for it, returning the address and size of the allocated
-    // membory in to **buf and *bufsize respectively.
+    // memory into **buf and *bufsize respectively.
 
     PyObject* header = NULL;
     char keyword[9];
@@ -757,7 +784,7 @@ void get_hdu_data_base(PyObject* hdu, void** buf, size_t* bufsize) {
     PyArrayObject* base;
     PyArrayObject* tmp;
 
-    data = (PyArrayObject*) PyObject_GetAttrString(hdu, "compData");
+    data = (PyArrayObject*) PyObject_GetAttrString(hdu, "compressed_data");
     if (data == NULL) {
         goto fail;
     }
@@ -767,7 +794,7 @@ void get_hdu_data_base(PyObject* hdu, void** buf, size_t* bufsize) {
     // allocated for the table and its heap
     if (!PyObject_TypeCheck(data, &PyArray_Type)) {
         PyErr_SetString(PyExc_TypeError,
-                        "CompImageHDU.compData must be a numpy.ndarray");
+                        "CompImageHDU.compressed_data must be a numpy.ndarray");
         goto fail;
     }
 
@@ -789,7 +816,7 @@ fail:
 
 
 void open_from_hdu(fitsfile** fileptr, void** buf, size_t* bufsize,
-                   PyObject* hdu, tcolumn* columns) {
+                   PyObject* hdu, tcolumn** columns) {
     PyObject* header = NULL;
     FITSfile* Fptr;
 
@@ -843,7 +870,7 @@ void open_from_hdu(fitsfile** fileptr, void** buf, size_t* bufsize,
 
     // Configure the array of table column structs from the Astropy header
     // instead of allowing CFITSIO to try to read from the header
-    tcolumns_from_header(*fileptr, header, &columns);
+    tcolumns_from_header(*fileptr, header, columns);
     if (PyErr_Occurred()) {
         goto fail;
     }
@@ -876,6 +903,7 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
     unsigned long long heapsize;
 
     fitsfile* fileptr;
+    FITSfile* Fptr;
     int status = 0;
 
     if (!PyArg_ParseTuple(args, "O:compression.compress_hdu", &hdu))
@@ -890,12 +918,14 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
     // We just need to get the compressed bytes and Astropy will handle the
     // writing of them.
     init_output_buffer(hdu, &outbuf, &outbufsize);
-    open_from_hdu(&fileptr, &outbuf, &outbufsize, hdu, columns);
+    open_from_hdu(&fileptr, &outbuf, &outbufsize, hdu, &columns);
     if (PyErr_Occurred()) {
         return NULL;
     }
 
-    bitpix_to_datatypes(fileptr->Fptr->zbitpix, &datatype, &npdatatype);
+    Fptr = fileptr->Fptr;
+
+    bitpix_to_datatypes(Fptr->zbitpix, &datatype, &npdatatype);
     if (PyErr_Occurred()) {
         return NULL;
     }
@@ -915,16 +945,34 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
         goto fail;
     }
 
-    znaxis = (npy_intp) outbufsize;  // The output array is just one dimension.
+    // Previously this used outbufsize as the size to use for the new Numpy
+    // byte array. However outbufsize is usually larger than necessary to
+    // store all the compressed data exactly; instead use the exact size
+    // of the compressed data from the heapsize plus the size of the table
+    // itself
+    heapsize = (unsigned long long) Fptr->heapsize;
+    znaxis = (npy_intp) (Fptr->heapstart + heapsize);
+
+    if (znaxis < outbufsize) {
+        // Go ahead and truncate to the size in znaxis to free the
+        // redundant allocation
+        // TODO: Add error handling
+        outbuf = realloc(outbuf, (size_t) znaxis);
+    }
+
     tmp = (PyArrayObject*) PyArray_SimpleNewFromData(1, &znaxis, NPY_UBYTE,
                                                      outbuf);
 
-    heapsize = (unsigned long long) fileptr->Fptr->heapsize;
 
     // Leaves refcount of tmp untouched, so its refcount should remain as 1
     retval = Py_BuildValue("KN", heapsize, tmp);
 
 fail:
+    if (columns != NULL) {
+        PyMem_Free(columns);
+        Fptr->tableptr = NULL;
+    }
+
     if (fileptr != NULL) {
         status = 1; // Disable header-related errors
         fits_close_file(fileptr, &status);
@@ -934,9 +982,6 @@ fail:
         }
     }
 
-    if (columns != NULL) {
-        PyMem_Free(columns);
-    }
     Py_XDECREF(indata);
 
     // Clear any messages remaining in CFITSIO's error stack
@@ -972,10 +1017,14 @@ PyObject* compression_decompress_hdu(PyObject* self, PyObject* args)
         return NULL;
     }
 
-    // Grab a pointer to the input data from the HDU's compData attribute
+    // Grab a pointer to the input data from the HDU's compressed_data
+    // attribute
     get_hdu_data_base(hdu, &inbuf, &inbufsize);
+    if (PyErr_Occurred()) {
+        return NULL;
+    }
 
-    open_from_hdu(&fileptr, &inbuf, &inbufsize, hdu, columns);
+    open_from_hdu(&fileptr, &inbuf, &inbufsize, hdu, &columns);
     if (PyErr_Occurred()) {
         return NULL;
     }
@@ -1005,6 +1054,11 @@ PyObject* compression_decompress_hdu(PyObject* self, PyObject* args)
     }
 
 fail:
+    if (columns != NULL) {
+        PyMem_Free(columns);
+        fileptr->Fptr->tableptr = NULL;
+    }
+
     if (fileptr != NULL) {
         status = 1;// Disable header-related errors
         fits_close_file(fileptr, &status);
@@ -1014,9 +1068,6 @@ fail:
         }
     }
 
-    if (columns != NULL) {
-        PyMem_Free(columns);
-    }
     PyMem_Free(znaxis);
 
     // Clear any messages remaining in CFITSIO's error stack
