@@ -4,13 +4,15 @@ This module handles the conversion of various VOTABLE datatypes
 to/from TABLEDATA_ and BINARY_ formats.
 """
 
-from __future__ import division, absolute_import
+from __future__ import absolute_import, division, print_function, unicode_literals
+from ...extern import six
+from ...extern.six.moves import xrange
 
 # STDLIB
 import re
 import sys
-from struct import unpack as struct_unpack
-from struct import pack as struct_pack
+from struct import unpack as _struct_unpack
+from struct import pack as _struct_pack
 
 # THIRD-PARTY
 import numpy as np
@@ -22,7 +24,6 @@ from ...utils.xml.writer import xml_escape_cdata
 # LOCAL
 from .exceptions import (vo_raise, vo_warn, warn_or_raise, W01,
     W30, W31, W39, W46, W47, W49, W51, E01, E02, E03, E04, E05, E06)
-from .util import IS_PY3K
 
 
 __all__ = ['get_converter', 'Converter', 'table_column_to_votable_datatype']
@@ -42,6 +43,17 @@ _empty_bytes = b''
 _zero_byte = b'\0'
 
 
+if six.PY3:
+    struct_unpack = _struct_unpack
+    struct_pack = _struct_pack
+else:
+    def struct_unpack(format, s):
+        return _struct_unpack(format.encode('ascii'), s)
+
+    def struct_pack(format, *args):
+        return _struct_pack(format.encode('ascii'), *args)
+
+
 if sys.byteorder == 'little':
     def _ensure_bigendian(x):
         if x.dtype.byteorder != '>':
@@ -70,6 +82,75 @@ def _make_masked_array(data, mask):
         return ma.array(np.array(data))
 
 
+def bitarray_to_bool(data, length):
+    """
+    Converts a bit array (a string of bits in a bytes object) to a
+    boolean Numpy array.
+
+    Parameters
+    ----------
+    data : bytes
+        The bit array.  The most significant byte is read first.
+
+    length : int
+        The number of bits to read.  The least significant bits in the
+        data bytes beyond length will be ignored.
+
+    Returns
+    -------
+    array : numpy bool array
+    """
+    results = []
+    for byte in data:
+        if not six.PY3:
+            byte = ord(byte)
+        for bit_no in range(7, -1, -1):
+            bit = byte & (1 << bit_no)
+            bit = (bit != 0)
+            results.append(bit)
+            if len(results) == length:
+                break
+        if len(results) == length:
+            break
+
+    return np.array(results, dtype='b1')
+
+
+def bool_to_bitarray(value):
+    """
+    Converts a numpy boolean array to a bit array (a string of bits in
+    a bytes object).
+
+    Parameters
+    ----------
+    value : numpy bool array
+
+    Returns
+    -------
+    bit_array : bytes
+        The first value in the input array will be the most
+        significant bit in the result.  The length will be `floor((N +
+        7) / 8)` where `N` is the length of `value`.
+    """
+    value = value.flat
+    bit_no = 7
+    byte = 0
+    bytes = []
+    for v in value:
+        if v:
+            byte |= 1 << bit_no
+        if bit_no == 0:
+            bytes.append(byte)
+            bit_no = 7
+            byte = 0
+        else:
+            bit_no -= 1
+    if bit_no != 7:
+        bytes.append(byte)
+
+    return struct_pack("%sB" % len(bytes), *bytes)
+
+
 class Converter(object):
     """
     The base class for all converters.  Each subclass handles
@@ -257,7 +338,7 @@ class Char(Converter):
 
     def output(self, value, mask):
         if mask:
-            return u''
+            return ''
         if not isinstance(value, str):
             value = value.decode('ascii')
         return xml_escape_cdata(value)
@@ -274,7 +355,7 @@ class Char(Converter):
         return s, False
 
     def _binoutput_var(self, value, mask):
-        if mask or value is None or value == u'':
+        if mask or value is None or value == '':
             return _zero_int
         return self._write_length(len(value)) + value
 
@@ -290,7 +371,7 @@ class UnicodeChar(Converter):
 
     Missing values are not handled for string or unicode types.
     """
-    default = u''
+    default = ''
 
     def __init__(self, field, config={}, pos=None):
         Converter.__init__(self, field, config, pos)
@@ -321,8 +402,8 @@ class UnicodeChar(Converter):
 
     def output(self, value, mask):
         if mask:
-            return u''
-        return xml_escape_cdata(unicode(value))
+            return ''
+        return xml_escape_cdata(six.text_type(value))
 
     def _binparse_var(self, read):
         length = self._parse_length(read)
@@ -344,7 +425,7 @@ class UnicodeChar(Converter):
 
     def _binoutput_fixed(self, value, mask):
         if mask:
-            value = u''
+            value = ''
         return struct_pack(self._struct_format, value.encode('utf_16_be'))
 
 
@@ -389,7 +470,7 @@ class VarArray(Array):
     def output(self, value, mask):
         output = self._base.output
         result = [output(x, m) for x, m in np.broadcast(value, value.mask)]
-        return u' '.join(result)
+        return ' '.join(result)
 
     def binparse(self, read):
         length = self._parse_length(read)
@@ -485,6 +566,8 @@ class NumericArray(Array):
         self.default[...] = self._base.default
 
     def parse(self, value, config={}, pos=None):
+        if config['version_1_3_or_later'] and value == '':
+            return np.zeros(self._arraysize, dtype=self._base.format), True
         parts = self._splitter(value, config, pos)
         if len(parts) != self._items:
             warn_or_raise(E02, E02, (self._items, len(parts)), config, pos)
@@ -518,7 +601,7 @@ class NumericArray(Array):
         base_output = self._base.output
         value = np.asarray(value)
         mask = np.asarray(mask)
-        return u' '.join(base_output(x, m) for x, m in
+        return ' '.join(base_output(x, m) for x, m in
                          zip(value.flat, mask.flat))
 
     def binparse(self, read):
@@ -573,26 +656,26 @@ class FloatingPoint(Numeric):
 
         precision = field.precision
         width = field.width
-        format_parts = [u'%']
+        format_parts = ['%']
 
         if width is not None:
-            format_parts.append(unicode(width))
+            format_parts.append(six.text_type(width))
 
         if precision is None:
-            format_parts.append(u's')
+            format_parts.append('s')
         elif precision.startswith("E"):
-            format_parts.append(u'.%dg' % int(precision[1:]))
+            format_parts.append('.%dg' % int(precision[1:]))
         elif precision.startswith("F"):
-            format_parts.append(u'.%df' % int(precision[1:]))
+            format_parts.append('.%df' % int(precision[1:]))
         else:
-            format_parts.append(u'.%df' % int(precision))
+            format_parts.append('.%df' % int(precision))
 
-        self._output_format = u''.join(format_parts)
+        self._output_format = ''.join(format_parts)
 
         self.nan = np.array(np.nan, self.format)
 
         if self.null is None:
-            self._null_output = u'NaN'
+            self._null_output = 'NaN'
             self._null_binoutput = self.binoutput(self.nan, False)
             self.filter_array = self._filter_nan
         else:
@@ -631,16 +714,16 @@ class FloatingPoint(Numeric):
             return self._null_output
         if np.isfinite(value):
             result = self._output_format % value
-            if (self._output_format[-1] == u's' and
-                result.endswith(u'.0')):
+            if (self._output_format[-1] == 's' and
+                result.endswith('.0')):
                 result = result[:-2]
             return result
         elif np.isnan(value):
-            return u'NaN'
+            return 'NaN'
         elif np.isposinf(value):
-            return u'+InF'
+            return '+InF'
         elif np.isneginf(value):
-            return u'-InF'
+            return '-InF'
         # Should never raise
         vo_raise("Invalid floating point value '%s'" % value)
 
@@ -684,10 +767,13 @@ class Integer(Numeric):
 
     def parse(self, value, config={}, pos=None):
         mask = False
-        if isinstance(value, basestring):
+        if isinstance(value, six.string_types):
             value = value.lower()
             if value == '':
-                warn_or_raise(W49, W49, (), config, pos)
+                if config['version_1_3_or_later']:
+                    mask = True
+                else:
+                    warn_or_raise(W49, W49, (), config, pos)
                 if self.null is not None:
                     value = self.null
                 else:
@@ -721,9 +807,9 @@ class Integer(Numeric):
         if mask:
             if self.null is None:
                 warn_or_raise(W31, W31)
-                return u'NaN'
-            return unicode(self.null)
-        return unicode(value)
+                return 'NaN'
+            return six.text_type(self.null)
+        return six.text_type(value)
 
     def binoutput(self, value, mask):
         if mask:
@@ -789,7 +875,7 @@ class ComplexArrayVarArray(VarArray):
 
     def parse(self, value, config={}, pos=None):
         if value.strip() == '':
-            return ma.array([]), False
+            return ma.array([]), True
 
         parts = self._splitter(value, config, pos)
         items = self._base._items
@@ -815,7 +901,7 @@ class ComplexVarArray(VarArray):
 
     def parse(self, value, config={}, pos=None):
         if value.strip() == '':
-            return ma.array([]), False
+            return ma.array([]), True
 
         parts = self._splitter(value, config, pos)
         parse_parts = self._base.parse_parts
@@ -895,17 +981,17 @@ class Complex(FloatingPoint, Array):
     def output(self, value, mask):
         if mask:
             if self.null is None:
-                return u'NaN'
+                return 'NaN'
             else:
                 value = self.null
         real = self._output_format % value.real
         imag = self._output_format % value.imag
-        if self._output_format[-1] == u's':
-            if real.endswith(u'.0'):
+        if self._output_format[-1] == 's':
+            if real.endswith('.0'):
                 real = real[:-2]
-            if imag.endswith(u'.0'):
+            if imag.endswith('.0'):
                 imag = imag[:-2]
-        return real + u' ' + imag
+        return real + ' ' + imag
 
 
 class FloatComplex(Complex):
@@ -947,25 +1033,13 @@ class BitArray(NumericArray):
 
     def output(self, value, mask):
         value = np.asarray(value)
-        mapping = {False: u'0', True: u'1'}
-        return u''.join(mapping[x] for x in value.flat)
+        mapping = {False: '0', True: '1'}
+        return ''.join(mapping[x] for x in value.flat)
 
     def binparse(self, read):
         data = read(self._bytes)
-        results = []
-        for byte in data:
-            if not IS_PY3K:
-                byte = ord(byte)
-            for bit_no in range(7, -1, -1):
-                bit = byte & (1 << bit_no)
-                bit = (bit != 0)
-                results.append(bit)
-                if len(results) == self._items:
-                    break
-            if len(results) == self._items:
-                break
-
-        result = np.array(results, dtype='b1').reshape(self._arraysize)
+        result = bitarray_to_bool(data, self._items)
+        result = result.reshape(self._arraysize)
         result_mask = np.zeros(self._arraysize, dtype='b1')
         return result, result_mask
 
@@ -973,25 +1047,7 @@ class BitArray(NumericArray):
         if np.any(mask):
             vo_warn(W39)
 
-        value = value.flat
-        bit_no = 7
-        byte = 0
-        bytes = []
-        for v in value:
-            if v:
-                byte |= 1 << bit_no
-            if bit_no == 0:
-                bytes.append(byte)
-                bit_no = 7
-                byte = 0
-            else:
-                bit_no -= 1
-        if bit_no != 7:
-            bytes.append(byte)
-
-        assert len(bytes) == self._bytes
-
-        return struct_pack("%sB" % len(bytes), *bytes)
+        return bool_to_bitarray(value)
 
 
 class Bit(Converter):
@@ -1011,7 +1067,8 @@ class Bit(Converter):
     def parse(self, value, config={}, pos=None):
         mapping = {'1': True, '0': False}
         if value is False or value.strip() == '':
-            warn_or_raise(W49, W49, (), config, pos)
+            if not config['version_1_3_or_later']:
+                warn_or_raise(W49, W49, (), config, pos)
             return False, True
         else:
             try:
@@ -1024,9 +1081,9 @@ class Bit(Converter):
             vo_warn(W39)
 
         if value:
-            return u'1'
+            return '1'
         else:
-            return u'0'
+            return '0'
 
     def binparse(self, read):
         data = read(1)
@@ -1056,7 +1113,7 @@ class BooleanArray(NumericArray):
         result = []
         result_mask = []
         for char in data:
-            if not IS_PY3K:
+            if not six.PY3:
                 char = ord(char)
             value, mask = binparse(char)
             result.append(value)
@@ -1092,6 +1149,8 @@ class Boolean(Converter):
         Converter.__init__(self, field, config, pos)
 
     def parse(self, value, config={}, pos=None):
+        if value == '':
+            return False, True
         if value is False:
             return False, True
         mapping = {'TRUE'  : (True, False),
@@ -1111,10 +1170,10 @@ class Boolean(Converter):
 
     def output(self, value, mask):
         if mask:
-            return u'?'
+            return '?'
         if value:
-            return u'T'
-        return u'F'
+            return 'T'
+        return 'F'
 
     def binparse(self, read):
         value = ord(read(1))
@@ -1232,11 +1291,7 @@ numpy_dtype_to_field_mapping = {
 }
 
 
-# numpy 1.4.1 doesn't have a "bytes_" type
-if hasattr(np, 'bytes_'):
-    numpy_dtype_to_field_mapping[np.bytes_().dtype.num] = 'char'
-else:
-    numpy_dtype_to_field_mapping[np.str_().dtype.num] = 'char'
+numpy_dtype_to_field_mapping[np.bytes_().dtype.num] = 'char'
 
 
 def _all_bytes(column):
@@ -1248,7 +1303,7 @@ def _all_bytes(column):
 
 def _all_unicode(column):
     for x in column:
-        if not isinstance(x, unicode):
+        if not isinstance(x, six.text_type):
             return False
     return True
 
@@ -1345,7 +1400,7 @@ def table_column_to_votable_datatype(column):
         if isinstance(column[0], bytes):
             if _all_bytes(column[1:]):
                 return {'datatype': 'char', 'arraysize': '*'}
-        elif isinstance(column[0], unicode):
+        elif isinstance(column[0], six.text_type):
             if _all_unicode(column[1:]):
                 return {'datatype': 'unicodeChar', 'arraysize': '*'}
         elif isinstance(column[0], np.ndarray):
