@@ -1,15 +1,20 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+
 """
 This file contains a contains the high-level functions to read a
 VOTable file.
 """
 
-from __future__ import division, absolute_import
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+from ...extern import six
 
 # STDLIB
 import io
 import os
 import sys
+import textwrap
 import warnings
 
 # LOCAL
@@ -20,7 +25,8 @@ from ...utils import data
 from ...config import ConfigurationItem
 
 
-__all__ = ['parse', 'parse_single_table', 'from_table', 'writeto', 'validate']
+__all__ = ['parse', 'parse_single_table', 'from_table', 'writeto', 'validate',
+           'reset_vo_warnings']
 
 
 PEDANTIC = ConfigurationItem(
@@ -31,7 +37,7 @@ PEDANTIC = ConfigurationItem(
 
 def parse(source, columns=None, invalid='exception', pedantic=None,
           chunk_size=tree.DEFAULT_CHUNK_SIZE, table_number=None,
-          table_id=None, filename=None,
+          table_id=None, filename=None, unit_format=None,
           _debug_python_based_parser=False):
     """
     Parses a VOTABLE_ xml file (or file-like object), and returns a
@@ -84,6 +90,17 @@ def parse(source, columns=None, invalid='exception', pedantic=None,
         Therefore, *filename* is only required when source is a
         file-like object.
 
+    unit_format : str, astropy.units.format.Base instance or None, optional
+        The unit format to use when parsing unit attributes.  If a
+        string, must be the name of a unit formatter. The built-in
+        formats include ``generic``, ``fits``, ``cds``, and
+        ``vounit``.  A custom formatter may be provided by passing a
+        `astropy.units.format.Base` instance.  If `None` (default),
+        the unit format to use will be the one specified by the
+        VOTable specification (which is `cds` up to version 1.2 of
+        VOTable, and (probably) `vounit` in future versions of the
+        spec).
+
     Returns
     -------
     votable : `astropy.io.votable.tree.VOTableFile` object
@@ -104,9 +121,10 @@ def parse(source, columns=None, invalid='exception', pedantic=None,
         'pedantic'     :     pedantic,
         'chunk_size'   :   chunk_size,
         'table_number' : table_number,
-        'filename'     :     filename}
+        'filename'     :     filename,
+        'unit_format'  :  unit_format}
 
-    if filename is None and isinstance(source, basestring):
+    if filename is None and isinstance(source, six.string_types):
         config['filename'] = source
 
     with iterparser.get_xml_iterator(
@@ -157,7 +175,7 @@ def writeto(table, file):
     table.to_xml(file, _debug_python_based_parser=True)
 
 
-def validate(source, output=sys.stdout, xmllint=False, filename=None):
+def validate(source, output=None, xmllint=False, filename=None):
     """
     Prints a validation report for the given file.
 
@@ -186,10 +204,12 @@ def validate(source, output=sys.stdout, xmllint=False, filename=None):
         Returns `True` if no warnings were found.  If `output` is
         `None`, the return value will be a string.
     """
-    import textwrap
-    from . import converters, xmlutil
+
     from ...utils.console import print_code_line, color_print
 
+    if output is None:
+        output = sys.stdout
+
     return_as_str = False
     if output is None:
         output = io.StringIO()
@@ -197,13 +217,7 @@ def validate(source, output=sys.stdout, xmllint=False, filename=None):
     lines = []
     votable = None
 
-    # This is a special variable used by the Python warnings
-    # infrastructure to keep track of warnings that have already been
-    # seen.  Since we want to get every single warning out of this, we
-    # have to delete all of them first.
-    for module in (exceptions, converters, tree, xmlutil):
-        if hasattr(module, '__warningregistry__'):
-            del module.__warningregistry__
+    reset_vo_warnings()
 
     with data.get_readable_fileobj(source, encoding='binary') as fd:
         content = fd.read()
@@ -211,7 +225,7 @@ def validate(source, output=sys.stdout, xmllint=False, filename=None):
     content_buffer.seek(0)
 
     if filename is None:
-        if isinstance(source, basestring):
+        if isinstance(source, six.string_types):
             filename = source
         elif hasattr(source, 'name'):
             filename = source.name
@@ -227,10 +241,12 @@ def validate(source, output=sys.stdout, xmllint=False, filename=None):
             votable = parse(content_buffer, pedantic=False, filename=filename)
         except ValueError as e:
             lines.append(str(e))
-    lines = [str(x.message) for x in warning_lines] + lines
+
+    lines = [str(x.message) for x in warning_lines if
+             issubclass(x.category, exceptions.VOWarning)] + lines
 
     content_buffer.seek(0)
-    output.write(u"Validation report for {0}\n\n".format(filename))
+    output.write("Validation report for {0}\n\n".format(filename))
 
     if len(lines):
         xml_lines = iterparser.xml_readlines(content_buffer)
@@ -240,7 +256,7 @@ def validate(source, output=sys.stdout, xmllint=False, filename=None):
 
             if not w['is_something']:
                 output.write(w['message'])
-                output.write(u'\n\n')
+                output.write('\n\n')
             else:
                 line = xml_lines[w['nline'] - 1]
                 warning = w['warning']
@@ -249,18 +265,18 @@ def validate(source, output=sys.stdout, xmllint=False, filename=None):
                 else:
                     color = 'red'
                 color_print(
-                    u'{0:d}: '.format(w['nline']), '',
+                    '{0:d}: '.format(w['nline']), '',
                     warning or 'EXC', color,
-                    u': ', '',
+                    ': ', '',
                     textwrap.fill(
                         w['message'],
-                        initial_indent=u'          ',
-                        subsequent_indent=u'  ').lstrip(),
+                        initial_indent='          ',
+                        subsequent_indent='  ').lstrip(),
                     file=output)
                 print_code_line(line, w['nchar'], file=output)
-            output.write(u'\n')
+            output.write('\n')
     else:
-        output.write(u'astropy.io.votable found no violations.\n\n')
+        output.write('astropy.io.votable found no violations.\n\n')
 
     success = 0
     if xmllint and os.path.exists(filename):
@@ -275,10 +291,10 @@ def validate(source, output=sys.stdout, xmllint=False, filename=None):
 
         if success != 0:
             output.write(
-                u'xmllint schema violations:\n\n')
+                'xmllint schema violations:\n\n')
             output.write(stderr)
         else:
-            output.write(u'xmllint passed\n')
+            output.write('xmllint passed\n')
 
     if return_as_str:
         return output.getvalue()
@@ -335,3 +351,24 @@ def is_votable(source):
             return True
     except ValueError:
         return False
+
+
+def reset_vo_warnings():
+    """
+    Resets all of the vo warning state so that warnings that
+    have already been emitted will be emitted again. This is
+    used, for example, by `validate` which must emit all
+    warnings each time it is called.
+
+    """
+    from . import converters, xmlutil
+
+    #-----------------------------------------------------------#
+    # This is a special variable used by the Python warnings    #
+    # infrastructure to keep track of warnings that have        #
+    # already been seen.  Since we want to get every single     #
+    # warning out of this, we have to delete all of them first. #
+    #-----------------------------------------------------------#
+    for module in (converters, exceptions, tree, xmlutil):
+        if hasattr(module, '__warningregistry__'):
+            del module.__warningregistry__
