@@ -3,7 +3,9 @@
 This is a set of regression tests for vo.
 """
 
-from __future__ import absolute_import, print_function
+from __future__ import absolute_import, division, print_function, unicode_literals
+from ....extern import six
+from ....extern.six.moves import xrange
 
 # STDLIB
 import difflib
@@ -13,7 +15,6 @@ import os
 import shutil
 import sys
 import tempfile
-import warnings
 
 # THIRD-PARTY
 from numpy.testing import assert_array_equal
@@ -22,17 +23,12 @@ import numpy as np
 # LOCAL
 from ..table import parse, parse_single_table, validate
 from .. import tree
-from ..util import IS_PY3K
 from ..exceptions import VOTableSpecError, VOWarning
 from ..xmlutil import validate_schema
-from ....utils.data import get_pkg_data_filename, get_pkg_data_fileobj, get_pkg_data_filenames
-from ....tests.helper import pytest, raises
+from ....utils.data import get_pkg_data_filename, get_pkg_data_filenames
+from ....tests.helper import pytest, raises, catch_warnings
 from ....utils.compat import gzip
 
-numpy_has_complex_bug = (
-    version.LooseVersion(np.__version__) < version.LooseVersion('1.5')
-    )
-
 # Determine the kind of float formatting in this build of Python
 if hasattr(sys, 'float_repr_style'):
     legacy_float_repr = (sys.float_repr_style == 'legacy')
@@ -52,12 +48,12 @@ def teardown_module():
     shutil.rmtree(TMP_DIR)
 
 
-def assert_validate_schema(filename):
+def assert_validate_schema(filename, version):
     if sys.platform.startswith('win'):
         return
 
     try:
-        rc, stdout, stderr = validate_schema(filename, '1.1')
+        rc, stdout, stderr = validate_schema(filename, version)
     except OSError:
         # If xmllint is not installed, we want the test to pass anyway
         return
@@ -89,7 +85,7 @@ def test_parse_single_table3():
         table_number=3, pedantic=False)
 
 
-def _test_regression(_python_based=False):
+def _test_regression(_python_based=False, binary_mode=1):
     # Read the VOTABLE
     votable = parse(
         get_pkg_data_filename('data/regression.xml'),
@@ -98,52 +94,64 @@ def _test_regression(_python_based=False):
     table = votable.get_first_table()
 
     dtypes = [
-        (('string test', 'string_test'), '|O8'),
-        (('fixed string test', 'string_test_2'), '|S10'),
-        ('unicode_test', '|O8'),
-        (('unicode test', 'fixed_unicode_test'), '<U10'),
-        (('string array test', 'string_array_test'), '|S4'),
-        ('unsignedByte', '|u1'),
-        ('short', '<i2'),
-        ('int', '<i4'),
-        ('long', '<i8'),
-        ('double', '<f8'),
-        ('float', '<f4'),
-        ('array', '|O8'),
-        ('bit', '|b1'),
-        ('bitarray', '|b1', (3, 2)),
-        ('bitvararray', '|O8'),
-        ('bitvararray2', '|O8'),
-        ('floatComplex', '<c8'),
-        ('doubleComplex', '<c16'),
-        ('doubleComplexArray', '|O8'),
-        ('doubleComplexArrayFixed', '<c16', (2,)),
-        ('boolean', '|b1'),
-        ('booleanArray', '|b1', (4,)),
-        ('nulls', '<i4'),
-        ('nulls_array', '<i4', (2, 2)),
-        ('precision1', '<f8'),
-        ('precision2', '<f8'),
-        ('doublearray', '|O8'),
-        ('bitarray2', '|b1', (16,))
+        ((str('string test'), str('string_test')), str('|O8')),
+        ((str('fixed string test'), str('string_test_2')), str('|S10')),
+        (str('unicode_test'), str('|O8')),
+        ((str('unicode test'), str('fixed_unicode_test')), str('<U10')),
+        ((str('string array test'), str('string_array_test')), str('|S4')),
+        (str('unsignedByte'), str('|u1')),
+        (str('short'), str('<i2')),
+        (str('int'), str('<i4')),
+        (str('long'), str('<i8')),
+        (str('double'), str('<f8')),
+        (str('float'), str('<f4')),
+        (str('array'), str('|O8')),
+        (str('bit'), str('|b1')),
+        (str('bitarray'), str('|b1'), (3, 2)),
+        (str('bitvararray'), str('|O8')),
+        (str('bitvararray2'), str('|O8')),
+        (str('floatComplex'), str('<c8')),
+        (str('doubleComplex'), str('<c16')),
+        (str('doubleComplexArray'), str('|O8')),
+        (str('doubleComplexArrayFixed'), str('<c16'), (2,)),
+        (str('boolean'), str('|b1')),
+        (str('booleanArray'), str('|b1'), (4,)),
+        (str('nulls'), str('<i4')),
+        (str('nulls_array'), str('<i4'), (2, 2)),
+        (str('precision1'), str('<f8')),
+        (str('precision2'), str('<f8')),
+        (str('doublearray'), str('|O8')),
+        (str('bitarray2'), str('|b1'), (16,))
         ]
     if sys.byteorder == 'big':
         new_dtypes = []
         for dtype in dtypes:
             dtype = list(dtype)
-            dtype[1] = dtype[1].replace('<', '>')
+            dtype[1] = dtype[1].replace(b'<', b'>')
             new_dtypes.append(tuple(dtype))
         dtypes = new_dtypes
     assert table.array.dtype == dtypes
 
     votable.to_xml(join(TMP_DIR, "regression.tabledata.xml"),
                    _debug_python_based_parser=_python_based)
-    assert_validate_schema(join(TMP_DIR, "regression.tabledata.xml"))
-    votable.get_first_table().format = 'binary'
+    assert_validate_schema(
+        join(TMP_DIR, "regression.tabledata.xml"),
+        votable.version)
+
+    if binary_mode == 1:
+        votable.get_first_table().format = 'binary'
+        votable.version = '1.1'
+    elif binary_mode == 2:
+        votable.get_first_table()._config['version_1_3_or_later'] = True
+        votable.get_first_table().format = 'binary2'
+        votable.version = '1.3'
+
     # Also try passing a file handle
     with open(join(TMP_DIR, "regression.binary.xml"), "wb") as fd:
         votable.to_xml(fd, _debug_python_based_parser=_python_based)
-    assert_validate_schema(join(TMP_DIR, "regression.binary.xml"))
+    assert_validate_schema(
+        join(TMP_DIR, "regression.binary.xml"),
+        votable.version)
     # Also try passing a file handle
     with open(join(TMP_DIR, "regression.binary.xml"), "rb") as fd:
         votable2 = parse(fd, pedantic=False,
@@ -152,10 +160,14 @@ def _test_regression(_python_based=False):
     votable2.to_xml(join(TMP_DIR, "regression.bin.tabledata.xml"),
                     _astropy_version="testing",
                     _debug_python_based_parser=_python_based)
-    assert_validate_schema(join(TMP_DIR, "regression.bin.tabledata.xml"))
+    assert_validate_schema(
+        join(TMP_DIR, "regression.bin.tabledata.xml"),
+        votable.version)
 
     with io.open(
-        get_pkg_data_filename('data/regression.bin.tabledata.truth.xml'),
+        get_pkg_data_filename(
+            'data/regression.bin.tabledata.truth.{0}.xml'.format(
+                votable.version)),
         'rt', encoding='utf-8') as fd:
         truth = fd.readlines()
     with io.open(
@@ -187,16 +199,21 @@ def _test_regression(_python_based=False):
     assert truth == output
 
 
-@pytest.mark.xfail('legacy_float_repr')
+@pytest.mark.xfail(str('legacy_float_repr'))
 def test_regression():
     _test_regression(False)
 
 
-@pytest.mark.xfail('legacy_float_repr')
+@pytest.mark.xfail(str('legacy_float_repr'))
 def test_regression_python_based_parser():
     _test_regression(True)
 
 
+@pytest.mark.xfail(str('legacy_float_repr'))
+def test_regression_binary2():
+    _test_regression(False, 2)
+
+
 class TestFixups:
     def setup_class(self):
         self.table = parse(
@@ -280,9 +297,10 @@ def test_select_columns_by_name():
 
 class TestParse:
     def setup_class(self):
-        self.table = parse(
+        self.votable = parse(
             get_pkg_data_filename('data/regression.xml'),
-            pedantic=False).get_first_table()
+            pedantic=False)
+        self.table = self.votable.get_first_table()
         self.array = self.table.array
         self.mask = self.table.array.mask
 
@@ -305,17 +323,17 @@ class TestParse:
         assert issubclass(self.array['unicode_test'].dtype.type,
                           np.object_)
         assert_array_equal(self.array['unicode_test'],
-                           [u"Ce\xe7i n'est pas un pipe",
-                            u'\u0bb5\u0ba3\u0b95\u0bcd\u0b95\u0bae\u0bcd',
-                            u'XXXX', u'', u''])
+                           ["Ce\xe7i n'est pas un pipe",
+                            '\u0bb5\u0ba3\u0b95\u0bcd\u0b95\u0bae\u0bcd',
+                            'XXXX', '', ''])
 
     def test_fixed_unicode_test(self):
         assert issubclass(self.array['fixed_unicode_test'].dtype.type,
                           np.unicode_)
         assert_array_equal(self.array['fixed_unicode_test'],
-                           [u"Ce\xe7i n'est",
-                            u'\u0bb5\u0ba3\u0b95\u0bcd\u0b95\u0bae\u0bcd',
-                            u'0123456789', u'', u''])
+                           ["Ce\xe7i n'est",
+                            '\u0bb5\u0ba3\u0b95\u0bcd\u0b95\u0bae\u0bcd',
+                            '0123456789', '', ''])
 
     def test_unsignedByte(self):
         assert issubclass(self.array['unsignedByte'].dtype.type,
@@ -396,7 +414,7 @@ class TestParse:
     def test_bitarray(self):
         assert issubclass(self.array['bitarray'].dtype.type,
                           np.bool_)
-        self.array['bitarray'].shape == (5, 3, 2)
+        assert self.array['bitarray'].shape == (5, 3, 2)
         assert_array_equal(self.array['bitarray'],
                            [[[ True, False],
                              [ True,  True],
@@ -481,7 +499,6 @@ class TestParse:
                 assert issubclass(a0.dtype.type, np.bool_)
                 assert_array_equal(a0, b0)
 
-    @pytest.mark.xfail('numpy_has_complex_bug')
     def test_floatComplex(self):
         assert issubclass(self.array['floatComplex'].dtype.type,
                           np.complex64)
@@ -490,7 +507,6 @@ class TestParse:
         assert_array_equal(self.mask['floatComplex'],
                            [True, False, False, True, True])
 
-    @pytest.mark.xfail('numpy_has_complex_bug')
     def test_doubleComplex(self):
         assert issubclass(self.array['doubleComplex'].dtype.type,
                           np.complex128)
@@ -500,7 +516,6 @@ class TestParse:
         assert_array_equal(self.mask['doubleComplex'],
                            [True, False, False, True, True])
 
-    @pytest.mark.xfail('numpy_has_complex_bug')
     def test_doubleComplexArray(self):
         assert issubclass(self.array['doubleComplexArray'].dtype.type,
                           np.object_)
@@ -584,6 +599,15 @@ class TestParse:
         assert not np.any(self.mask['bitarray2'][0])
         assert np.all(self.mask['bitarray2'][1:])
 
+    def test_get_coosys_by_id(self):
+        coosys = self.votable.get_coosys_by_id('J2000')
+        assert coosys.system == 'eq_FK5'
+
+    def test_get_field_by_utype(self):
+        fields = list(self.votable.get_fields_by_utype("myint"))
+        assert fields[0].name == "int"
+        assert fields[0].values.min == -1000
+
 
 class TestThroughTableData(TestParse):
     def setup_class(self):
@@ -591,13 +615,15 @@ class TestThroughTableData(TestParse):
             get_pkg_data_filename('data/regression.xml'),
             pedantic=False)
         votable.to_xml(join(TMP_DIR, "test_through_tabledata.xml"))
-        self.table = parse(join(TMP_DIR, "test_through_tabledata.xml"),
-                           pedantic=False).get_first_table()
+        self.votable = parse(join(TMP_DIR, "test_through_tabledata.xml"),
+                           pedantic=False)
+        self.table = self.votable.get_first_table()
         self.array = self.table.array
         self.mask = self.table.array.mask
 
     def test_schema(self):
-        assert_validate_schema(join(TMP_DIR, "test_through_tabledata.xml"))
+        assert_validate_schema(
+            join(TMP_DIR, "test_through_tabledata.xml"), '1.1')
 
 
 class TestThroughBinary(TestParse):
@@ -607,8 +633,9 @@ class TestThroughBinary(TestParse):
             pedantic=False)
         votable.get_first_table().format = 'binary'
         votable.to_xml(join(TMP_DIR, "test_through_binary.xml"))
-        self.table = parse(join(TMP_DIR, "test_through_binary.xml"),
-                           pedantic=False).get_first_table()
+        self.votable = parse(join(TMP_DIR, "test_through_binary.xml"),
+                           pedantic=False)
+        self.table = self.votable.get_first_table()
         self.array = self.table.array
         self.mask = self.table.array.mask
 
@@ -625,6 +652,27 @@ class TestThroughBinary(TestParse):
         assert not np.any(self.mask['bitarray2'])
 
 
+class TestThroughBinary2(TestParse):
+    def setup_class(self):
+        votable = parse(
+            get_pkg_data_filename('data/regression.xml'),
+            pedantic=False)
+        votable.version = '1.3'
+        votable.get_first_table()._config['version_1_3_or_later'] = True
+        votable.get_first_table().format = 'binary2'
+
+        votable.to_xml(join(TMP_DIR, "test_through_binary2.xml"))
+        self.votable = parse(join(TMP_DIR, "test_through_binary2.xml"),
+                           pedantic=False)
+        self.table = self.votable.get_first_table()
+        self.array = self.table.array
+        self.mask = self.table.array.mask
+
+    def test_get_coosys_by_id(self):
+        # No COOSYS in VOTable 1.2 or later
+        pass
+
+
 def table_from_scratch():
     from ..tree import VOTableFile, Resource, Table, Field
 
@@ -708,8 +756,8 @@ def test_build_from_scratch():
     assert_array_equal(
         table.array.mask, np.array([(False, [[False, False], [False, False]]),
                                     (False, [[False, False], [False, False]])],
-                                    dtype=[('filename', '?'),
-                                           ('matrix', '?', (2, 2))]))
+                                    dtype=[(str('filename'), str('?')),
+                                           (str('matrix'), str('?'), (2, 2))]))
 
 
 def test_validate():
@@ -738,7 +786,7 @@ def test_validate():
     output = output[1:-1]
 
     for line in difflib.unified_diff(truth, output):
-        if IS_PY3K:
+        if six.PY3:
             sys.stdout.write(
                 line.replace('\\n', '\n'))
         else:
@@ -767,9 +815,7 @@ def test_gzip_filehandles():
 
 
 def test_from_scratch_example():
-    with warnings.catch_warnings(record=True) as warning_lines:
-        warnings.resetwarnings()
-        warnings.simplefilter("always", VOWarning, append=True)
+    with catch_warnings(VOWarning) as warning_lines:
         try:
             _run_test_from_scratch_example()
         except ValueError as e:
@@ -821,3 +867,106 @@ def test_fileobj():
                 assert isinstance(fd, io.FileIO)
             else:
                 assert isinstance(fd, file)
+
+
+def test_nonstandard_units():
+    from .... import units as u
+
+    votable = parse(
+        get_pkg_data_filename('data/nonstandard_units.xml'),
+        pedantic=False)
+
+    assert isinstance(
+        votable.get_first_table().fields[0].unit, u.UnrecognizedUnit)
+
+    votable = parse(
+        get_pkg_data_filename('data/nonstandard_units.xml'),
+        pedantic=False,
+        unit_format='generic')
+
+    assert not isinstance(
+        votable.get_first_table().fields[0].unit, u.UnrecognizedUnit)
+
+
+def test_resource_structure():
+    # Based on issue #1223, as reported by @astro-friedel and @RayPlante
+    from astropy.io.votable import tree as vot
+
+    vtf = vot.VOTableFile()
+
+    r1 = vot.Resource()
+    vtf.resources.append(r1)
+    t1 = vot.Table(vtf)
+    t1.name = "t1"
+    t2 = vot.Table(vtf)
+    t2.name = 't2'
+    r1.tables.append(t1)
+    r1.tables.append(t2)
+
+    r2 = vot.Resource()
+    vtf.resources.append(r2)
+    t3 = vot.Table(vtf)
+    t3.name = "t3"
+    t4 = vot.Table(vtf)
+    t4.name = "t4"
+    r2.tables.append(t3)
+    r2.tables.append(t4)
+
+    r3 = vot.Resource()
+    vtf.resources.append(r3)
+    t5 = vot.Table(vtf)
+    t5.name = "t5"
+    t6 = vot.Table(vtf)
+    t6.name = "t6"
+    r3.tables.append(t5)
+    r3.tables.append(t6)
+
+    buff = io.BytesIO()
+    vtf.to_xml(buff)
+
+    buff.seek(0)
+    vtf2 = parse(buff)
+
+    assert len(vtf2.resources) == 3
+
+    for r in xrange(len(vtf2.resources)):
+        res = vtf2.resources[r]
+        assert len(res.tables) == 2
+        assert len(res.resources) == 0
+
+
+def test_no_resource_check():
+    output = io.StringIO()
+
+    # We can't test xmllint, because we can't rely on it being on the
+    # user's machine.
+    result = validate(get_pkg_data_filename('data/no_resource.xml'),
+                      output, xmllint=False)
+
+    assert result == False
+
+    output.seek(0)
+    output = output.readlines()
+
+    # Uncomment to generate new groundtruth
+    # with io.open('no_resource.txt', 'wt', encoding='utf-8') as fd:
+    #     fd.write(u''.join(output))
+
+    with io.open(
+        get_pkg_data_filename('data/no_resource.txt'),
+        'rt', encoding='utf-8') as fd:
+        truth = fd.readlines()
+
+    truth = truth[1:]
+    output = output[1:-1]
+
+    for line in difflib.unified_diff(truth, output):
+        if six.PY3:
+            sys.stdout.write(
+                line.replace('\\n', '\n'))
+        else:
+            sys.stdout.write(
+                line.encode('unicode_escape').
+                replace('\\n', '\n'))
+
+    assert truth == output
