@@ -14,7 +14,7 @@ from . import config
 from .utils.compat import inspect_getmodule
 from .utils.console import color_print
 from .utils.misc import find_current_module
-
+from .utils.exceptions import AstropyWarning, AstropyUserWarning
 
 __all__ = ['log', 'AstropyLogger', 'LoggingError']
 
@@ -40,7 +40,6 @@ class _AstLogIPYExc(Exception):
 
 
 # Read in configuration
-
 LOG_LEVEL = ConfigurationItem('log_level', 'INFO',
                               "Threshold for the logging messages. Logging "
                               "messages that are less severe than this level "
@@ -53,11 +52,11 @@ USE_COLOR = ConfigurationItem('use_color', True,
 LOG_WARNINGS = ConfigurationItem('log_warnings', True,
                                  "Whether to log warnings.warn calls")
 
-LOG_EXCEPTIONS = ConfigurationItem('log_exceptions', True,
+LOG_EXCEPTIONS = ConfigurationItem('log_exceptions', False,
                                    "Whether to log exceptions before raising "
                                    "them")
 
-LOG_TO_FILE = ConfigurationItem('log_to_file', True,
+LOG_TO_FILE = ConfigurationItem('log_to_file', False,
                                 "Whether to always log messages to a log "
                                 "file")
 
@@ -93,6 +92,38 @@ def _init_log():
     return log
 
 
+def _teardown_log():
+    """Shut down exception and warning logging (if enabled) and clear all
+    Astropy loggers from the logging module's cache.
+
+    This involves poking some logging module interals, so much if it is 'at
+    your own risk' and is allowed to pass silently if any exceptions occur.
+    """
+
+    global log
+
+    if log.exception_logging_enabled():
+        log.disable_exception_logging()
+
+    if log.warnings_logging_enabled():
+        log.disable_warnings_logging()
+
+    del log
+
+    # Now for the fun stuff...
+    try:
+        logging._acquireLock()
+        try:
+            loggerDict = logging.Logger.manager.loggerDict
+            for key in loggerDict.keys():
+                if key == 'astropy' or key.startswith('astropy.'):
+                    del loggerDict[key]
+        finally:
+            logging._releaseLock()
+    except:
+        pass
+
+
 Logger = logging.getLoggerClass()
 
 
@@ -129,12 +160,17 @@ class AstropyLogger(Logger):
     _showwarning_orig = None
 
     def _showwarning(self, *args, **kwargs):
+
+        # Bail out if we are not catching a warning from Astropy
+        if not isinstance(args[0], AstropyWarning):
+            return self._showwarning_orig(*args, **kwargs)
+
         warning = args[0]
         # Deliberately not using isinstance here: We want to display
         # the class name only when it's not the default class,
-        # UserWarning.  The name of subclasses of UserWarning should
+        # AstropyWarning.  The name of subclasses of AstropyWarning should
         # be displayed.
-        if type(warning) != UserWarning:
+        if type(warning) not in (AstropyWarning, AstropyUserWarning):
             message = '{0}: {1}'.format(warning.__class__.__name__, args[0])
         else:
             message = unicode(args[0])
@@ -205,10 +241,14 @@ class AstropyLogger(Logger):
     _excepthook_orig = None
 
     def _excepthook(self, etype, value, traceback):
-        tb = traceback
-        while tb.tb_next is not None:
-            tb = tb.tb_next
-        mod = inspect_getmodule(tb)
+
+        if traceback is None:
+            mod = None
+        else:
+            tb = traceback
+            while tb.tb_next is not None:
+                tb = tb.tb_next
+            mod = inspect_getmodule(tb)
 
         # include the the error type in the message.
         if len(value.args) > 0:
@@ -337,7 +377,8 @@ class AstropyLogger(Logger):
             color_print(record.levelname, 'brown', end='')
         else:
             color_print(record.levelname, 'red', end='')
-        print(": {0} [{1:s}]".format(record.msg, record.origin))
+        record.message = "{0} [{1:s}]".format(record.msg, record.origin)
+        print(": " + record.message)
 
     @contextmanager
     def log_to_file(self, filename, filter_level=None, filter_origin=None):
