@@ -51,10 +51,13 @@ This directive requires a single argument that must be a module or
 package. It accepts no options.
 
 .. note::
-    Like 'inheritance-diagram', 'automod-diagram' requires graphviz to
-    generate the inheritance diagram.
+    Like 'inheritance-diagram', 'automod-diagram' requires
+    `graphviz <http://www.graphviz.org/>`_ to generate the inheritance diagram.
 
 """
+
+import inspect
+import os
 import re
 
 from sphinx.ext.autosummary import Autosummary
@@ -87,8 +90,6 @@ class Automodsumm(AstropyAutosummary):
     option_spec['skip'] = _str_list_converter
 
     def run(self):
-        from inspect import isclass, isfunction
-
         self.warnings = []
         nodelist = []
 
@@ -120,12 +121,12 @@ class Automodsumm(AstropyAutosummary):
             if funconly and not clsonly:
                 cont = []
                 for nm, obj in zip(fqns, objs):
-                    if nm not in skipmap and isfunction(obj):
+                    if nm not in skipmap and inspect.isfunction(obj):
                         cont.append('~' + nm)
             elif clsonly:
                 cont = []
                 for nm, obj in zip(fqns, objs):
-                    if nm not in skipmap and isclass(obj):
+                    if nm not in skipmap and inspect.isclass(obj):
                         cont.append('~' + nm)
             else:
                 if clsonly and funconly:
@@ -146,8 +147,6 @@ class Automodsumm(AstropyAutosummary):
 #<-------------------automod-diagram stuff------------------------------------>
 class Automoddiagram(InheritanceDiagram):
     def run(self):
-        from inspect import isclass
-
         try:
             nms, objs = find_mod_objs(self.arguments[0], onlylocals=True)[1:]
         except ImportError:
@@ -158,7 +157,7 @@ class Automoddiagram(InheritanceDiagram):
         clsnms = []
         for n, o in zip(nms, objs):
 
-            if isclass(o):
+            if inspect.isclass(o):
                 clsnms.append(n)
 
         oldargs = self.arguments
@@ -172,8 +171,6 @@ class Automoddiagram(InheritanceDiagram):
 
 #<---------------------automodsumm generation stuff--------------------------->
 def process_automodsumm_generation(app):
-    import os
-
     env = app.builder.env
     ext = app.config.source_suffix
 
@@ -227,9 +224,6 @@ def automodsumm_to_autosummary_lines(fn, app):
 
 
     """
-    import os
-    from inspect import isfunction, isclass
-
     fullfn = os.path.join(app.builder.env.srcdir, fn)
 
     with open(fullfn) as fr:
@@ -285,9 +279,9 @@ def automodsumm_to_autosummary_lines(fn, app):
         for nm, fqn, obj in zip(*find_mod_objs(modnm, onlylocals=True)):
             if nm in toskip:
                 continue
-            if funcsonly and not isfunction(obj):
+            if funcsonly and not inspect.isfunction(obj):
                 continue
-            if clssonly and not isclass(obj):
+            if clssonly and not inspect.isclass(obj):
                 continue
             newlines.append(allindent + '~' + fqn)
 
@@ -304,7 +298,6 @@ def generate_automodsumm_docs(lines, srcfn, suffix='.rst', warn=None,
     autosummarized. Unlike generate_autosummary_docs, this function is
     called one file at a time.
     """
-    import os
 
     from sphinx.jinja2glue import BuiltinTemplateLoader
     from sphinx.ext.autosummary import import_by_name, get_documenter
@@ -476,9 +469,8 @@ def generate_automodsumm_docs(lines, srcfn, suffix='.rst', warn=None,
             ns['objtype'] = doc.objtype
             ns['underline'] = len(name) * '='
 
-            # We now check whether a reference file exists for the module
-            # being documented. This assumes that we are in _generated, and
-            # that ../ gives us the root of the docs. We first check if the
+            # We now check whether a file for reference footnotes exists for 
+            # the module being documented. We first check if the
             # current module is a file or a directory, as this will give a
             # different path for the reference file. For example, if
             # documenting astropy.wcs then the reference file is at
@@ -490,15 +482,21 @@ def generate_automodsumm_docs(lines, srcfn, suffix='.rst', warn=None,
                 mod_name_dir = mod_name.replace('.', '/').split('/', 1)[1]
             else:
                 mod_name_dir = mod_name
-            if not os.path.isdir(os.path.join(path, '..', mod_name_dir)) \
-               and os.path.isdir(os.path.join(path, '..', mod_name_dir.rsplit('/', 1)[0])):
+            if not os.path.isdir(os.path.join(base_path, mod_name_dir)) \
+               and os.path.isdir(os.path.join(base_path, mod_name_dir.rsplit('/', 1)[0])):
                 mod_name_dir = mod_name_dir.rsplit('/', 1)[0]
 
             # We then have to check whether it exists, and if so, we pass it
-            # to the template
-            reference_file = os.path.join('..', mod_name_dir, 'references.txt')
-            if os.path.exists(os.path.join(path, reference_file)):
-                ns['referencefile'] = reference_file
+            # to the template.  
+            if os.path.exists(os.path.join(base_path, mod_name_dir, 'references.txt')):
+                # An important subtlety here is that the path we pass in has 
+                # to be relative to the file being generated, so we have to
+                # figure out the right number of '..'s 
+                ndirsback = path.replace(base_path, '').count('/')
+                ref_file_rel_segments = ['..'] * ndirsback
+                ref_file_rel_segments.append(mod_name_dir)
+                ref_file_rel_segments.append('references.txt')
+                ns['referencefile'] = os.path.join(*ref_file_rel_segments)
 
             rendered = template.render(**ns)
             f.write(rendered)
