@@ -1,4 +1,12 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+from ...extern import six
+
+import pkgutil
+import os
+import types
+
 
 def test_imports():
     """
@@ -6,17 +14,14 @@ def test_imports():
     dependencies that sneak through
     """
 
-    from os.path import split
-    from types import ModuleType
-    from pkgutil import get_loader, walk_packages
-
     from ...utils import find_current_module
 
     pkgornm = find_current_module(1).__name__.split('.')[0]
 
-    if isinstance(pkgornm, basestring):
-        package = get_loader(pkgornm).load_module(pkgornm)
-    elif isinstance(pkgornm, ModuleType) and '__init__' in pkgornm.__file__:
+    if isinstance(pkgornm, six.string_types):
+        package = pkgutil.get_loader(pkgornm).load_module(pkgornm)
+    elif (isinstance(pkgornm, types.ModuleType) and
+            '__init__' in pkgornm.__file__):
         package = pkgornm
     else:
         msg = 'test_imports is not determining a valid package/package name'
@@ -25,10 +30,20 @@ def test_imports():
     if hasattr(package, '__path__'):
         pkgpath = package.__path__
     elif hasattr(package, '__file__'):
-        pkgpath = split(package.__file__)[0]
+        pkgpath = os.path.split(package.__file__)[0]
     else:
         raise AttributeError('package to generate config items for does not '
                              'have __file__ or __path__')
 
-    for imper, nm, ispkg in walk_packages(pkgpath, package.__name__ + '.'):
+    prefix = package.__name__ + '.'
+    for imper, nm, ispkg in pkgutil.walk_packages(pkgpath, prefix):
         imper.find_module(nm)
+
+
+def test_toplevel_namespace():
+    import astropy
+    d = dir(astropy)
+    assert 'os' not in d
+    assert 'log' in d
+    assert 'test' in d
+    assert 'sys' not in d
