@@ -6,9 +6,13 @@ Handles the "FITS" unit format.
 
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
+from ...extern.six.moves import zip
 
+import keyword
 import warnings
 
+from ...utils.exceptions import AstropyDeprecationWarning
+
 from . import generic
 from . import utils
 
@@ -23,14 +27,15 @@ class Fits(generic.Generic):
     name = 'fits'
 
     def __init__(self):
-        super(Fits, self).__init__()
+        # Build this on the class, so it only gets generated once.
+        if '_parser' not in Fits.__dict__:
+            Fits._parser, Fits._lexer = self._make_parser()
 
         if not '_units' in Fits.__dict__:
             Fits._units, Fits._deprecated_units = self._generate_unit_names()
 
     @staticmethod
     def _generate_unit_names():
-        import keyword
         from ... import units as u
         names = {}
         deprecated_names = set()
@@ -81,7 +86,7 @@ class Fits(generic.Generic):
             warnings.warn(
                 "The unit {0!r} has been deprecated in the FITS "
                 "standard.".format(unit),
-                DeprecationWarning)
+                AstropyDeprecationWarning)
 
         return cls._units[unit]
 
@@ -96,7 +101,7 @@ class Fits(generic.Generic):
             warnings.warn(
                 "The unit {0!r} has been deprecated in the FITS "
                 "standard.".format(name),
-                DeprecationWarning)
+                AstropyDeprecationWarning)
 
         return name
 
@@ -107,12 +112,12 @@ class Fits(generic.Generic):
         unit = utils.decompose_to_known_units(unit, self._get_unit_name)
 
         if isinstance(unit, core.CompositeUnit):
-            if unit.scale != 1.0:
+            if unit.scale != 1:
                 raise ValueError(
                     "The FITS unit format is not able to represent scale. "
                     "Multiply your data by {0:e}.".format(unit.scale))
 
-            pairs = zip(unit.bases, unit.powers)
+            pairs = list(zip(unit.bases, unit.powers))
             pairs.sort(key=lambda x: x[1], reverse=True)
 
             s = self._format_unit_list(pairs)
