@@ -9,11 +9,9 @@ from __future__ import (absolute_import, division, print_function,
 
 import os
 import re
-import sys
 
 from . import utils
 from .base import Base
-from ...utils.compat.fractions import Fraction
 
 
 class Generic(Base):
@@ -102,7 +100,7 @@ class Generic(Base):
             return t
 
         def t_UNIT(t):
-            r'[a-zA-Z][a-zA-Z_]*'
+            r'%|[a-zA-Z][a-zA-Z_]*'
             t.value = cls._get_unit(t)
             return t
 
@@ -128,11 +126,12 @@ class Generic(Base):
                  | factor division_product_of_units
                  | inverse_unit
                  | factor inverse_unit
+                 | factor
             '''
+            from ..core import Unit
             if len(p) == 2:
-                p[0] = p[1]
+                p[0] = Unit(p[1])
             else:
-                from ..core import Unit
                 p[0] = Unit(p[1] * p[2])
 
         def p_division_product_of_units(p):
@@ -306,9 +305,9 @@ class Generic(Base):
             function : function_name OPEN_PAREN unit_expression CLOSE_PAREN
             '''
             if p[1] == 'sqrt':
-                p[0] = p[3] ** -2.0
+                p[0] = p[3] ** 0.5
             else:
-               raise ValueError(
+                raise ValueError(
                    '{0!r} is not a recognized function'.format(p[1]))
 
         def p_error(p):
@@ -335,9 +334,11 @@ class Generic(Base):
 
     @classmethod
     def _parse_unit(cls, s):
-        from ..core import UnitBase
-        registry = UnitBase._registry
-        if s in registry:
+        from ..core import get_current_unit_registry
+        registry = get_current_unit_registry().registry
+        if s == '%':
+            return registry['percent']
+        elif s in registry:
             return registry[s]
         raise ValueError(
             '{0} is not a valid unit'.format(s))
@@ -352,11 +353,11 @@ class Generic(Base):
                 return self._parser.parse(s, lexer=self._lexer, debug=debug)
             except ValueError as e:
                 if str(e):
-                    raise ValueError("{0} in unit {1!r}".format(
+                    raise ValueError("{0} in string {1!r}".format(
                         str(e), s))
                 else:
                     raise ValueError(
-                        "Syntax error parsing unit {0!r}".format(s))
+                        "Syntax error parsing unit string {0!r}".format(s))
 
     def _get_unit_name(self, unit):
         return unit.get_format_name('generic')
@@ -369,15 +370,8 @@ class Generic(Base):
             if power == 1:
                 out.append(self._get_unit_name(base))
             else:
-                if not isinstance(power, Fraction):
-                    if power % 1.0 != 0.0:
-                        frac = Fraction.from_float(power)
-                        power = frac.limit_denominator(10)
-                        if power.denominator == 1:
-                            power = int(power.numerator)
-                    else:
-                        power = int(power)
-                if isinstance(power, Fraction):
+                power = utils.format_power(power)
+                if '/' in power:
                     out.append('{0}({1})'.format(
                         self._get_unit_name(base), power))
                 else:
@@ -389,25 +383,30 @@ class Generic(Base):
         from .. import core
 
         if isinstance(unit, core.CompositeUnit):
-            if unit.scale != 1 and self._show_scale:
-                s = '{0:e} '.format(unit.scale)
-            else:
-                s = ''
+            parts = []
+
+            if self._show_scale and unit.scale != 1:
+                parts.append('{0:g}'.format(unit.scale))
 
             if len(unit.bases):
                 positives, negatives = utils.get_grouped_by_powers(
                     unit.bases, unit.powers)
                 if len(positives):
-                    s += self._format_unit_list(positives)
-                elif s == '':
-                    s = '1'
+                    parts.append(self._format_unit_list(positives))
+                elif len(parts) == 0:
+                    parts.append('1')
 
                 if len(negatives):
-                    s += ' / ({0})'.format(self._format_unit_list(negatives))
-        elif isinstance(unit, core.NamedUnit):
-            s = self._get_unit_name(unit)
+                    parts.append('/')
+                    unit_list = self._format_unit_list(negatives)
+                    if len(negatives) == 1:
+                        parts.append('{0}'.format(unit_list))
+                    else:
+                        parts.append('({0})'.format(unit_list))
 
-        return s
+            return ' '.join(parts)
+        elif isinstance(unit, core.NamedUnit):
+            return self._get_unit_name(unit)
 
 
 class Unscaled(Generic):
