@@ -1,4 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+
 """
 Regression tests for the units package
 """
@@ -6,12 +7,12 @@ Regression tests for the units package
 from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
 
-import warnings
 
 import numpy as np
+from numpy.testing.utils import assert_allclose
 
-from ...tests.helper import pytest, raises
-from ...tests.compat import assert_allclose
+from ...extern.six.moves import cPickle as pickle
+from ...tests.helper import pytest, raises, catch_warnings
 from ...utils.compat.fractions import Fraction
 
 from ... import units as u
@@ -21,17 +22,22 @@ def test_getting_started():
     """
     Corresponds to "Getting Started" section in the docs.
     """
-    speed_unit = u.cm / u.s
-    x = speed_unit.to(u.mile / u.hour, 1)
-    assert_allclose(x, 0.02236936292054402)
-    speed_converter = speed_unit.get_converter("mile hour^-1")
-    x = speed_converter([1., 1000., 5000.])
-    assert_allclose(x, [2.23693629e-02, 2.23693629e+01, 1.11846815e+02])
+    from .. import imperial
+    with imperial.enable():
+        speed_unit = u.cm / u.s
+        x = speed_unit.to(imperial.mile / u.hour, 1)
+        assert_allclose(x, 0.02236936292054402)
+        speed_converter = speed_unit.get_converter("mile hour^-1")
+        x = speed_converter([1., 1000., 5000.])
+        assert_allclose(x, [2.23693629e-02, 2.23693629e+01, 1.11846815e+02])
 
 
-@raises(ValueError)
 def test_invalid_power():
-    u.m ** 0.3
+    x = u.m ** (1, 3)
+    assert isinstance(x.powers[0], Fraction)
+
+    x = u.m ** (1, 2)
+    assert isinstance(x.powers[0], float)
 
 
 def test_invalid_compare():
@@ -43,19 +49,12 @@ def test_convert():
 
 
 def test_convert_fail():
-    with pytest.raises(u.UnitsException):
+    with pytest.raises(u.UnitsError):
         u.cm.to(u.s, 1)
-    with pytest.raises(u.UnitsException):
+    with pytest.raises(u.UnitsError):
         (u.cm / u.s).to(u.m, 1)
 
 
-def test_is_equivalent():
-    assert u.m.is_equivalent(u.inch)
-    assert not (u.Hz.is_equivalent(u.J))
-    assert u.Hz.is_equivalent(u.J, u.spectral())
-    assert u.J.is_equivalent(u.Hz, u.spectral())
-
-
 def test_composite():
     assert (u.cm / u.s * u.h).get_converter(u.m)(1) == 36
     assert u.cm * u.cm == u.cm ** 2
@@ -73,92 +72,12 @@ def test_repr():
     assert repr(u.cm) == 'Unit("cm")'
 
 
-def test_spectral():
-    a = u.AA.to(u.Hz, 1, u.spectral())
-    assert_allclose(a, 2.9979245799999995e+18)
-    b = u.Hz.to(u.AA, a, u.spectral())
-    assert_allclose(b, 1)
-
-    a = u.AA.to(u.MHz, 1, u.spectral())
-    assert_allclose(a, 2.9979245799999995e+12)
-    b = u.MHz.to(u.AA, a, u.spectral())
-    assert_allclose(b, 1)
-
-    a = u.m.to(u.Hz, 1, u.spectral())
-    assert_allclose(a, 2.9979245799999995e+8)
-    b = u.Hz.to(u.m, a, u.spectral())
-    assert_allclose(b, 1)
-
-
-def test_spectral2():
-    a = u.nm.to(u.J, 500, u.spectral())
-    assert_allclose(a, 3.972891366538605e-19)
-    b = u.J.to(u.nm, a, u.spectral())
-    assert_allclose(b, 500)
-
-    a = u.AA.to(u.Hz, 1, u.spectral())
-    b = u.Hz.to(u.J, a, u.spectral())
-    c = u.AA.to(u.J, 1, u.spectral())
-    assert_allclose(b, c)
-
-
-def test_spectral3():
-    a = u.nm.to(u.Hz, [1000, 2000], u.spectral())
-    assert_allclose(a, [2.99792458e+14, 1.49896229e+14])
-
-
-def test_spectraldensity():
-
-    a = u.AA.to(u.Jy, 1, u.spectral_density(u.eV, 2.2))
-    assert_allclose(a, 1059416252057.8357, rtol=1e-4)
-
-    b = u.Jy.to(u.AA, a, u.spectral_density(u.eV, 2.2))
-    assert_allclose(b, 1)
-
-
-def test_spectraldensity2():
-    flambda = u.erg / u.angstrom / u.cm ** 2 / u.s
-    fnu = u.erg / u.Hz / u.cm ** 2 / u.s
-
-    a = flambda.to(fnu, 1, u.spectral_density(u.AA, 3500))
-    assert_allclose(a, 4.086160166177361e-12)
-
-
-def test_spectraldensity3():
-
-    # Define F_nu in Jy
-    f_nu = u.Jy
-
-    # Convert to ergs / cm^2 / s / Hz
-    assert_allclose(f_nu.to(u.erg / u.cm ** 2 / u.s / u.Hz, 1.), 1.e-23, 10)
-
-    # Convert to ergs / cm^2 / s at 10 Ghz
-    assert_allclose(f_nu.to(u.erg / u.cm ** 2 / u.s, 1.,
-                    equivalencies=u.spectral_density(u.GHz, 10)), 1.e-13, 10)
-
-    # Convert to ergs / cm^2 / s / micron at 1 Ghz
-    assert_allclose(f_nu.to(u.erg / u.cm ** 2 / u.s / u.micron, 1.,
-                    equivalencies=u.spectral_density(u.Hz, 1.e9)),
-                    3.335640951981521e-20, 10)
-
-    # Define F_lambda in ergs / cm^2 / s / micron
-    f_lambda = u.erg / u.cm ** 2 / u.s / u.micron
-
-    # Convert to Jy at 1 Ghz
-    assert_allclose(f_lambda.to(u.Jy, 1.,
-                    equivalencies=u.spectral_density(u.Hz, 1.e9)),
-                    1. / 3.335640951981521e-20, 10)
-
-    # Convert to ergs / cm^2 / s at 10 microns
-    assert_allclose(f_lambda.to(u.erg / u.cm ** 2 / u.s, 1.,
-                    equivalencies=u.spectral_density(u.micron, 10.)), 10., 10)
-
-
 def test_units_conversion():
     assert_allclose(u.kpc.to(u.Mpc), 0.001)
     assert_allclose(u.Mpc.to(u.kpc), 1000)
     assert_allclose(u.yr.to(u.Myr), 1.e-6)
     assert_allclose(u.AU.to(u.pc), 4.84813681e-6)
+    assert_allclose(u.cycle.to(u.rad), 6.283185307179586)
 
 
 def test_units_manipulation():
@@ -171,28 +90,41 @@ def test_decompose():
     assert u.Ry == u.Ry.decompose()
 
 
-def test_equivalent_units():
-    assert u.pound in u.g.find_equivalent_units()
-    assert u.J in u.Hz.find_equivalent_units(u.spectral())
+def test_dimensionless_to_si():
+    """
+    Issue #1150: Test for conversion of dimensionless quantities
+                 to the SI system
+    """
+
+    testunit = ((1.0 * u.kpc) / (1.0 * u.Mpc))
+
+    assert testunit.unit.physical_type == 'dimensionless'
+    assert_allclose(testunit.si, 0.001)
+
+
+def test_dimensionless_to_cgs():
+    """
+    Issue #1150: Test for conversion of dimensionless quantities
+                 to the CGS system
+    """
+
+    testunit = ((1.0 * u.m) / (1.0 * u.km))
+
+    assert testunit.unit.physical_type == 'dimensionless'
+    assert_allclose(testunit.cgs, 0.001)
 
 
 def test_unknown_unit():
-    with warnings.catch_warnings(record=True) as warning_lines:
-        warnings.resetwarnings()
-        warnings.simplefilter("always", u.UnitsWarning, append=True)
+    with catch_warnings(u.UnitsWarning) as warning_lines:
         u.Unit("FOO", parse_strict='warn')
 
-    assert warning_lines[0].category == u.UnitsWarning
     assert 'FOO' in str(warning_lines[0].message)
 
 
 def test_unknown_unit2():
-    with warnings.catch_warnings(record=True) as warning_lines:
-        warnings.resetwarnings()
-        warnings.simplefilter("always", u.UnitsWarning, append=True)
+    with catch_warnings(u.UnitsWarning) as warning_lines:
         assert u.Unit("m/s/kg", parse_strict='warn').to_string() == 'm/s/kg'
 
-    assert warning_lines[0].category == u.UnitsWarning
     assert 'm/s/kg' in str(warning_lines[0].message)
 
 
@@ -201,20 +133,48 @@ def test_unknown_unit3():
     assert isinstance(unit, u.UnrecognizedUnit)
     assert unit.name == "FOO"
 
+    unit2 = u.Unit("FOO", parse_strict='silent')
+    assert unit == unit2
+    assert unit.is_equivalent(unit2)
+
+    unit3 = u.Unit("BAR", parse_strict='silent')
+    assert unit != unit3
+    assert not unit.is_equivalent(unit3)
+
+    with pytest.raises(ValueError):
+        unit.get_converter(unit3)
+
+    x = unit.to_string('latex')
+    y = unit2.to_string('cgs')
+
+    with pytest.raises(ValueError):
+        unit4 = u.Unit("BAR", parse_strict='strict')
+
+    with pytest.raises(TypeError):
+        unit5 = u.Unit(None)
+
+
+@raises(TypeError)
+def test_invalid_scale():
+    x = ['a', 'b', 'c'] * u.m
+
+
+def test_cds_power():
+    unit = u.Unit("10+22/cm2", format="cds", parse_strict='silent')
+    assert unit.scale == 1e22
+
 
 def test_register():
-    try:
-        u.def_unit("foo", u.m ** 3, register=True)
-        assert hasattr(u, 'foo')
-    finally:
-        if hasattr(u, 'foo'):
-            del u.UnitBase._registry['foo']
-            del u.foo
+    foo = u.def_unit("foo", u.m ** 3, namespace=locals())
+    assert 'foo' in locals()
+    with u.add_enabled_units(foo):
+        assert 'foo' in u.get_current_unit_registry().registry
+    assert 'foo' not in u.get_current_unit_registry().registry
 
 
 def test_in_units():
     speed_unit = u.cm / u.s
-    x = speed_unit.in_units(u.mile / u.hour, 1)
+    x = speed_unit.in_units(u.pc / u.hour, 1)
 
 
 def test_null_unit():
@@ -223,7 +183,7 @@ def test_null_unit():
 
 def test_unrecognized_equivalency():
     assert u.m.is_equivalent('foo') is False
-    assert u.m.is_equivalent('foot') is True
+    assert u.m.is_equivalent('pc') is True
 
 
 @raises(TypeError)
@@ -234,14 +194,14 @@ def test_unit_noarg():
 def test_convertible_exception():
     try:
         u.AA.to(u.h * u.s ** 2)
-    except u.UnitsException as e:
+    except u.UnitsError as e:
         assert "length" in str(e)
 
 
 def test_convertible_exception2():
     try:
         u.m.to(u.s)
-    except u.UnitsException as e:
+    except u.UnitsError as e:
         assert "length" in str(e)
 
 
@@ -262,6 +222,9 @@ def test_steradian():
     results = u.sr.compose(units=u.cgs.bases)
     assert results[0].bases[0] is u.rad
 
+    results = u.sr.compose(units=u.cgs.__dict__)
+    assert results[0].bases[0] is u.sr
+
 
 def test_decompose_bases():
     """
@@ -289,16 +252,21 @@ def test_equiv_compose():
     assert u.Hz in composed
 
 
+def test_empty_compose():
+    with pytest.raises(u.UnitsException):
+        composed = u.m.compose(units=[])
+
+
 def test_compose_roundtrip():
     def _test_compose_roundtrip(unit):
         composed_list = unit.decompose().compose()
         found = False
         for composed in composed_list:
-            if len(composed._bases):
-                if composed._bases[0] is unit:
+            if len(composed.bases):
+                if composed.bases[0] is unit:
                     found = True
                     break
-            elif len(unit._bases) == 0:
+            elif len(unit.bases) == 0:
                 found = True
                 break
         assert found
@@ -313,7 +281,9 @@ def test_compose_roundtrip():
 
 def test_compose_cgs_to_si():
     def _test_compose_cgs_to_si(unit):
-        unit.to_system(u.si)
+        si = unit.to_system(u.si)
+        assert [x.is_equivalent(unit) for x in si]
+        assert si[0] == unit.si
 
     for val in u.cgs.__dict__.values():
         if (isinstance(val, u.UnitBase) and
@@ -325,12 +295,15 @@ def test_compose_si_to_cgs():
     def _test_compose_si_to_cgs(unit):
         # Can't convert things with Ampere to CGS without more context
         try:
-            unit.to_system(u.cgs)
-        except u.UnitsException:
+            cgs = unit.to_system(u.cgs)
+        except u.UnitsError:
             if u.A in unit.decompose().bases:
                 pass
             else:
                 raise
+        else:
+            assert [x.is_equivalent(unit) for x in cgs]
+            assert cgs[0] == unit.cgs
 
     for val in u.si.__dict__.values():
         if (isinstance(val, u.UnitBase) and
@@ -364,7 +337,7 @@ def test_self_compose():
     assert len(unit.compose(units=[u.g, u.s])) == 1
 
 
-@raises(u.UnitsException)
+@raises(u.UnitsError)
 def test_compose_failed():
     unit = u.kg
 
@@ -427,10 +400,7 @@ def test_endian_independence():
     for endian in ['<', '>']:
         for ntype in ['i', 'f']:
             for byte in ['4', '8']:
-                # Note, we have to use encode() because we've imported
-                # unicode_literals from __future__, and Numpy 1.4.1 crashes if
-                # a unicode dtype is passed.
-                x = np.array([1,2,3], dtype=(endian + ntype + byte).encode('ascii'))
+                x = np.array([1,2,3], dtype=(endian + ntype + byte))
                 u.m.to(u.cm, x)
 
 
@@ -449,24 +419,95 @@ def test_no_as():
 
 
 def test_pickling():
-    import cPickle
-
-    p = cPickle.dumps(u.m)
-    other = cPickle.loads(p)
+    p = pickle.dumps(u.m)
+    other = pickle.loads(p)
 
     assert other is u.m
 
-    try:
-        new_unit = u.IrreducibleUnit(['foo'], format={'baz': 'bar'})
-        p = cPickle.dumps(new_unit)
-    finally:
-        del u.UnitBase._registry['foo']
-
-    new_unit_copy = cPickle.loads(p)
+    new_unit = u.IrreducibleUnit(['foo'], format={'baz': 'bar'})
+    u.add_enabled_units([new_unit])
+    p = pickle.dumps(new_unit)
+    new_unit_copy = pickle.loads(p)
     assert new_unit_copy.names == ['foo']
     assert new_unit_copy.get_format_name('baz') == 'bar'
 
 
 @raises(ValueError)
 def test_duplicate_define():
-    u.def_unit('m')
+    u.def_unit('m', namespace=u.__dict__)
+
+
+def test_all_units():
+    from ...units.core import get_current_unit_registry
+    registry = get_current_unit_registry()
+    assert len(registry.all_units) > len(registry.non_prefix_units)
+
+
+def test_repr_latex():
+    assert u.m._repr_latex_() == u.m.to_string('latex')
+
+
+def test_operations_with_strings():
+    assert u.m / '5s' == (u.m / (5.0 * u.s))
+
+    assert u.m * '5s' == (5.0 * u.m * u.s)
+
+
+def test_comparison():
+    assert u.m > u.cm
+    assert u.m >= u.cm
+    assert u.cm < u.m
+    assert u.cm <= u.m
+
+    with pytest.raises(u.UnitsException):
+        u.m > u.kg
+
+
+def test_compose_into_arbitrary_units():
+    # Issue #1438
+    from ...constants import G
+    G.decompose([u.kg, u.km, u.Unit("15 s")])
+
+
+def test_unit_multiplication_with_string():
+    """Check that multiplication with strings produces the correct unit."""
+    u1 = u.cm
+    us = 'kg'
+    assert us * u1 == u.Unit(us) * u1
+    assert u1 * us == u1 * u.Unit(us)
+
+
+def test_unit_division_by_string():
+    """Check that multiplication with strings produces the correct unit."""
+    u1 = u.cm
+    us = 'kg'
+    assert us / u1 == u.Unit(us) / u1
+    assert u1 / us == u1 / u.Unit(us)
+
+
+def test_sorted_bases():
+    """See #1616."""
+    assert (u.m * u.Jy).bases == (u.Jy * u.m).bases
+
+
+def test_megabit():
+    """See #1543"""
+    assert u.Mbit is u.Mb
+    assert u.megabit is u.Mb
+
+    assert u.Mbyte is u.MB
+    assert u.megabyte is u.MB
+
+
+def test_composite_unit_get_format_name():
+    """See #1576"""
+    unit1 = u.Unit('nrad/s')
+    unit2 = u.Unit('Hz(1/2)')
+    assert (str(u.CompositeUnit(1, [unit1, unit2], [1, -1])) ==
+            'nrad / (Hz(1/2) s)')
+
+def test_unicode_policy():
+    from ...tests.helper import assert_follows_unicode_guidelines
+
+    assert_follows_unicode_guidelines(
+        u.degree, roundtrip=u.__dict__)
