@@ -4,21 +4,33 @@ A "grab bag" of relatively small general-purpose utilities that don't have
 a clear module/package to live in.
 """
 
-from __future__ import absolute_import
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
 
-import collections
 import contextlib
 import functools
+import inspect
+import json
 import os
+import signal
 import sys
 import textwrap
 import traceback
 import warnings
 
+from .exceptions import AstropyDeprecationWarning, AstropyPendingDeprecationWarning
+
+from ..extern import six
+from ..extern.six.moves import urllib
+
 
 __all__ = ['find_current_module', 'isiterable', 'deprecated', 'lazyproperty',
            'deprecated_attribute', 'silence', 'format_exception',
-           'NumpyRNGContext', 'is_path_hidden', 'walk_skip_hidden']
+           'NumpyRNGContext', 'find_api_page', 'is_path_hidden',
+           'walk_skip_hidden', 'JsonCustomEncoder', 'indent']
+
+__doctest_skip__ = ['find_current_module']
 
 
 def find_current_module(depth=1, finddiff=False):
@@ -97,13 +109,12 @@ def find_current_module(depth=1, finddiff=False):
         pkg.mod1
 
     """
-    from inspect import currentframe, ismodule
 
     # using a patched version of getmodule because the py 3.1 and 3.2 stdlib
     # is broken if the list of modules changes during import
     from .compat import inspect_getmodule
 
-    frm = currentframe()
+    frm = inspect.currentframe()
     for i in range(depth):
         frm = frm.f_back
         if frm is None:
@@ -116,9 +127,9 @@ def find_current_module(depth=1, finddiff=False):
         else:
             diffmods = []
             for fd in finddiff:
-                if ismodule(fd):
+                if inspect.ismodule(fd):
                     diffmods.append(fd)
-                elif isinstance(fd, basestring):
+                elif isinstance(fd, six.string_types):
                     diffmods.append(__import__(fd))
                 elif fd is True:
                     diffmods.append(currmod)
@@ -167,21 +178,21 @@ def find_mod_objs(modname, onlylocals=False):
         the other arguments)
 
     """
-    from inspect import ismodule
 
     __import__(modname)
     mod = sys.modules[modname]
 
     if hasattr(mod, '__all__'):
-        pkgitems = [(k, getattr(mod, k)) for k in mod.__all__]
+        pkgitems = [(k, mod.__dict__[k]) for k in mod.__all__]
     else:
-        pkgitems = [(k, getattr(mod, k)) for k in dir(mod) if k[0] != '_']
+        pkgitems = [(k, mod.__dict__[k]) for k in dir(mod) if k[0] != '_']
 
-    #filter out modules and pull the names and objs out
+    # filter out modules and pull the names and objs out
+    ismodule = inspect.ismodule
     localnames = [k for k, v in pkgitems if not ismodule(v)]
     objs = [v for k, v in pkgitems if not ismodule(v)]
 
-    #fully qualified names can be determined from the object's module
+    # fully qualified names can be determined from the object's module
     fqnames = []
     for obj, lnm in zip(objs, localnames):
         if hasattr(obj, '__module__') and hasattr(obj, '__name__'):
@@ -208,6 +219,17 @@ def isiterable(obj):
         return False
 
 
+def indent(s, shift=1, width=4):
+    """Indent a block of text.  The indentation is applied to each line."""
+
+    indented = '\n'.join(' ' * (width * shift) + l if l else ''
+                         for l in s.splitlines())
+    if s[-1] == '\n':
+        indented += '\n'
+
+    return indented
+
+
 class lazyproperty(object):
     """
     Works similarly to property(), but computes the value only once.
@@ -220,7 +242,7 @@ class lazyproperty(object):
         >>> class LazyTest(object):
         ...     @lazyproperty
         ...     def complicated_property(self):
-        ...         print 'Computing the value for complicated_property...'
+        ...         print('Computing the value for complicated_property...')
         ...         return 42
         ...
         >>> lt = LazyTest()
@@ -249,7 +271,7 @@ class lazyproperty(object):
     def __get__(self, obj, owner=None):
         if obj is None:
             return self
-        key = self._fget.func_name
+        key = self._fget.__name__
         if key not in obj.__dict__:
             val = self._fget(obj)
             obj.__dict__[key] = val
@@ -258,14 +280,21 @@ class lazyproperty(object):
             return obj.__dict__[key]
 
     def __set__(self, obj, val):
+        obj_dict = obj.__dict__
+        func_name = self._fget.__name__
         if self._fset:
-            self._fset(obj, val)
-        obj.__dict__[self._fget.func_name] = val
+            ret = self._fset(obj, val)
+            if ret is not None and obj_dict.get(func_name) is ret:
+                # By returning the value set the setter signals that it took
+                # over setting the value in obj.__dict__; this mechanism allows
+                # it to override the input value
+                return
+        obj_dict[func_name] = val
 
     def __delete__(self, obj):
         if self._fdel:
             self._fdel(obj)
-        key = self._fget.func_name
+        key = self._fget.__name__
         if key in obj.__dict__:
             del obj.__dict__[key]
 
@@ -282,7 +311,7 @@ class lazyproperty(object):
         args = [self._fget, self._fset, self._fdel, self.__doc__]
         args[arg] = f
         cls_ns = sys._getframe(1).f_locals
-        for k, v in cls_ns.items():
+        for k, v in six.iteritems(cls_ns):
             if v is self:
                 property_name = k
                 break
@@ -331,8 +360,8 @@ def deprecated(since, message='', name='', alternative='', pending=False,
         this alternative if provided.
 
     pending : bool, optional
-        If True, uses a PendingDeprecationWarning instead of a
-        DeprecationWarning.
+        If True, uses a AstropyPendingDeprecationWarning instead of a
+        AstropyDeprecationWarning.
     """
 
     def deprecate(func, message=message, name=name, alternative=alternative,
@@ -380,9 +409,9 @@ def deprecated(since, message='', name='', alternative='', pending=False,
         @functools.wraps(func)
         def deprecated_func(*args, **kwargs):
             if pending:
-                category = PendingDeprecationWarning
+                category = AstropyPendingDeprecationWarning
             else:
-                category = DeprecationWarning
+                category = AstropyDeprecationWarning
 
             warnings.warn(message, category, stacklevel=2)
 
@@ -446,8 +475,8 @@ def deprecated_attribute(name, since, message=None, alternative=None,
         user about this alternative if provided.
 
     pending : bool, optional
-        If True, uses a PendingDeprecationWarning instead of a
-        DeprecationWarning.
+        If True, uses a AstropyPendingDeprecationWarning instead of a
+        AstropyDeprecationWarning.
 
     Examples
     --------
@@ -579,6 +608,115 @@ class NumpyRNGContext(object):
         random.set_state(self.startstate)
 
 
+def find_api_page(obj, version=None, openinbrowser=True, timeout=None):
+    """
+    Determines the URL of the API page for the specified object, and
+    optionally open that page in a web browser.
+
+    .. note::
+        You must be connected to the internet for this to function even
+        if `openinbrowser` is False, unless you provide a local version of
+        the documentation to `version` (e.g., ``file:///path/to/docs``).
+
+    Parameters
+    ----------
+    obj
+        The object to open the docs for or its fully-qualified name
+        (as a str).
+    version : str
+        The doc version - either a version number like '0.1', 'dev' for
+        the development/latest docs, or a URL to point to a specific
+        location that should be the *base* of the documentation. Defaults to
+        latest if you are on aren't on a release, otherwise, the version you
+        are on.
+    openinbrowser : bool
+        If True, the `webbrowser` package will be used to open the doc
+        page in a new web browser window.
+    timeout : number, optional
+        The number of seconds to wait before timing-out the query to
+        the astropy documentation.  If not given, the default python
+        stdlib timeout will be used.
+    Returns
+    -------
+    url : str
+        The loaded URL
+
+    Raises
+    ------
+    ValueError
+        If the documentation can't be found
+
+    """
+    import webbrowser
+
+    from zlib import decompress
+
+    if (not isinstance(obj, six.string_types) and
+            hasattr(obj, '__module__') and
+            hasattr(obj, '__name__')):
+        obj = obj.__module__ + '.' + obj.__name__
+    elif inspect.ismodule(obj):
+        obj = obj.__name__
+
+    if version is None:
+        from .. import version
+
+        if version.release:
+            version = 'v' + version.version
+        else:
+            version = 'dev'
+
+    if '://' in version:
+        if version.endswith('index.html'):
+            baseurl = version[:-10]
+        elif version.endswith('/'):
+            baseurl = version
+        else:
+            baseurl = version + '/'
+    elif version == 'dev' or version == 'latest':
+        baseurl = 'http://devdocs.astropy.org/'
+    else:
+        baseurl = 'http://docs.astropy.org/en/{vers}/'.format(vers=version)
+
+    if timeout is None:
+        uf = urllib.request.urlopen(baseurl + 'objects.inv')
+    else:
+        uf = urllib.request.urlopen(baseurl + 'objects.inv', timeout=timeout)
+
+    try:
+        # we read these lines so that `oistr` only gets the compressed
+        # contents, not the header information
+        isvers = uf.readline().rstrip().decode('utf-8')  # intersphinx version line
+        proj = uf.readline().rstrip().decode('utf-8')  # project name
+        vers = uf.readline().rstrip().decode('utf-8')  # project version
+        uf.readline().rstrip().decode('utf-8')
+        oistr = uf.read()
+    finally:
+        uf.close()
+
+    oistr = decompress(oistr).decode('utf-8')
+
+    resurl = None
+
+    for l in oistr.strip().splitlines():
+        ls = l.split()
+        name = ls[0]
+        loc = ls[3]
+        if loc.endswith('$'):
+            loc = loc[:-1] + name
+
+        if name == obj:
+            resurl = baseurl + loc
+            break
+
+    if resurl is None:
+        raise ValueError('Could not find the docs for the object {obj}'.format(obj=obj))
+    elif openinbrowser:
+        webbrowser.open(resurl)
+
+    return resurl
+
+
 def signal_number_to_name(signum):
     """
     Given an OS signal number, returns a signal name.  If the signal
@@ -587,7 +725,6 @@ def signal_number_to_name(signum):
     # Since these numbers and names are platform specific, we use the
     # builtin signal module and build a reverse mapping.
 
-    import signal
     signal_to_name_map = dict(
         (k, v) for v, k in signal.__dict__.iteritems() if v.startswith('SIG'))
 
@@ -659,3 +796,36 @@ def walk_skip_hidden(top, onerror=None, followlinks=False):
         dirs[:] = [d for d in dirs if not is_path_hidden(d)]
         files[:] = [f for f in files if not is_path_hidden(f)]
         yield root, dirs, files
+
+
+class JsonCustomEncoder(json.JSONEncoder):
+    """Support for data types that JSON default encoder
+    does not do.
+
+    This includes:
+
+        * Numpy array or number
+        * Complex number
+        * Set
+        * Bytes (Python 3)
+
+    Examples
+    --------
+    >>> import json
+    >>> import numpy as np
+    >>> from astropy.utils.misc import JsonCustomEncoder
+    >>> json.dumps(np.arange(3), cls=JsonCustomEncoder)
+    '[0, 1, 2]'
+
+    """
+    def default(self, obj):
+        import numpy as np
+        if isinstance(obj, (np.ndarray, np.number)):
+            return obj.tolist()
+        elif isinstance(obj, (complex, np.complex)):
+            return [obj.real, obj.imag]
+        elif isinstance(obj, set):
+            return list(obj)
+        elif isinstance(obj, bytes):  # pragma: py3
+            return obj.decode()
+        return json.JSONEncoder.default(self, obj)
