@@ -1,14 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import division  # confidence high
-# We don't want the "division" symbol in the namespace, since it
-# should have only docstrings
-try:
-    del division
-except NameError:
-    # When building the configuration defaults in 3.x, the ``del division`` line
-    # throws a NameError because for some reason it seems to think it's already
-    # deleted.  This works around that.
-    pass
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 # It gets to be really tedious to type long docstrings in ANSI C
 # syntax (since multi-line string literals are not valid).
@@ -1735,7 +1726,7 @@ vice versa.
 
 Parameters
 ----------
-ctype : string
+ctype : str
     Required spectral ``CTYPEia``, maximum of 8 characters.  The first
     four characters are required to be given and are never modified.
     The remaining four, the algorithm code, are completely determined
@@ -2019,17 +2010,17 @@ like ``"log (m ** 2)"`` which is formally disallowed.
 Parameters
 ----------
 
-have : string
+have : str
     FITS unit string to convert from, with or without surrounding
     square brackets (for inline specifications); text following the
     closing bracket is ignored.
 
-want : string
+want : str
     FITS unit string to convert to, with or without surrounding square
     brackets (for inline specifications); text following the closing
     bracket is ignored.
 
-ctrl : string, optional
+ctrl : str, optional
     Do potentially unsafe translations of non-standard unit strings.
 
     Although ``\"S\"`` is commonly used to represent seconds, its
@@ -2099,7 +2090,7 @@ whitespace.
 
 Parameters
 ----------
-translate_units : string, optional
+translate_units : str, optional
     Do potentially unsafe translations of non-standard unit strings.
 
     Although ``\"S\"`` is commonly used to represent seconds, its
@@ -2200,7 +2191,7 @@ Parameters
 header : An `astropy.io.fits.Header`, string, or `None`.
   If ``None``, the object will be initialized to default values.
 
-key : string, optional
+key : str, optional
     The key referring to a particular WCS transform in the header.
     This may be either ``' '`` or ``'A'``-``'Z'`` and corresponds to
     the ``\"a\"`` part of ``\"CTYPEia\"``.  (*key* may only be
@@ -2265,3 +2256,62 @@ zsource = """
 
 An undefined value is represented by NaN.
 """
+
+WcsError = """
+Base class of all invalid WCS errors.
+"""
+
+SingularMatrix = """
+SingularMatrixError()
+
+The linear transformation matrix is singular.
+"""
+
+InconsistentAxisTypes = """
+InconsistentAxisTypesError()
+
+The WCS header inconsistent or unrecognized coordinate axis type(s).
+"""
+
+InvalidTransform = """
+InvalidTransformError()
+
+The WCS transformation is invalid, or the transformation parameters
+are invalid.
+"""
+
+InvalidCoordinate = """
+InvalidCoordinateError()
+
+One or more of the world coordinates is invalid.
+"""
+
+NoSolution = """
+NoSolutionError()
+
+No solution can be found in the given interval.
+"""
+
+InvalidSubimageSpecification = """
+InvalidSubimageSpecificationError()
+
+The subimage specification is invalid.
+"""
+
+NonseparableSubimageCoordinateSystem = """
+NonseparableSubimageCoordinateSystemError()
+
+Non-separable subimage coordinate system.
+"""
+
+NoWcsKeywordsFound = """
+NoWcsKeywordsFoundError()
+
+No WCS keywords were found in the given header.
+"""
+
+InvalidTabularParameters = """
+InvalidTabularParametersError()
+
+The given tabular parameters are invalid.
+"""
