@@ -1,7 +1,7 @@
 /*============================================================================
 
-  WCSLIB 4.16 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2012, Mark Calabretta
+  WCSLIB 4.19 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2013, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -22,10 +22,10 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: prj.h,v 4.16 2012/11/07 04:42:44 cal103 Exp $
+  $Id: prj.h,v 4.19 2013/09/29 14:17:51 mcalabre Exp $
 *=============================================================================
 *
-* WCSLIB 4.16 - C routines that implement the spherical map projections
+* WCSLIB 4.19 - C routines that implement the spherical map projections
 * recognized by the FITS World Coordinate System (WCS) standard.  Refer to
 *
 *   "Representations of world coordinates in FITS",
@@ -97,6 +97,7 @@
 *   - cscset(), cscx2s(), cscs2x():   CSC (COBE spherical cube)
 *   - qscset(), qscx2s(), qscs2x():   QSC (quadrilateralized spherical cube)
 *   - hpxset(), hpxx2s(), hpxs2x():   HPX (HEALPix)
+*   - xphset(), xphx2s(), xphs2x():   XPH (HEALPix polar, aka "butterfly")
 *
 * Argument checking (projection routines):
 * ----------------------------------------
@@ -564,7 +565,7 @@ extern const int CONIC, CONVENTIONAL, CYLINDRICAL, POLYCONIC,
 extern const char prj_categories[9][32];
 
 extern const int  prj_ncode;
-extern const char prj_codes[27][4];
+extern const char prj_codes[28][4];
 
 #ifdef PRJX2S_ARGS
 #undef PRJX2S_ARGS
@@ -743,6 +744,10 @@ int hpxset(struct prjprm *prj);
 int hpxx2s(PRJX2S_ARGS);
 int hpxs2x(PRJS2X_ARGS);
 
+int xphset(struct prjprm *prj);
+int xphx2s(PRJX2S_ARGS);
+int xphs2x(PRJS2X_ARGS);
+
 
 /* Deprecated. */
 #define prjini_errmsg prj_errmsg
